/*
	Various POSIX functions not available with CodeWarrior
*/

#include "maccomp.h"

/* lseek as provided by CodeWarrior is COMPLETELY broken as
   far as its use to increase file size is concerned.
   Hence, we must rewrite it to a large extend.
   
   The first rewrite we tried used lseek first, then
   tried to fix the call if lseek failed. However, 
   this did not work.
   */
   	
off_t
Mylseek(int fd, off_t offset, int whence)
{
	off_t res, curr, end, off1, off2;
	
	/* current position */
	curr= tell(fd);
	/* end of file position */
	end= lseek(fd, 0, SEEK_END);
	
	/* We now compute two values : off1 and off2.
	   off1 is offset relative to file start
	   off2 is offset relative to file end.
	   lseek can be used only if off1 >= 0 and off2 <= 0
	 */
	 
	 /* if off2 > 0, we add characters after the end of the file */
	
	switch(whence)
		{
		case SEEK_SET:
			off1= offset;
			off2= offset - end;
		break;
		case SEEK_CUR:
			off1= curr + offset;
			off2= off1 - end;
		break;
		case SEEK_END:
			off1= end + offset;
			off2= offset;
		break;
		default:
			perror("unexpected argument to seek");
			exit(1);
		}
	
	if (off1 >= 0 && off2 <= 0)
	{
		/* as we moved, we can no longer use CUR */
		res= lseek(fd,off1,SEEK_SET);
	}
	else if (off1 < 0)
		res= -1;
	else
	{
		int i, good;
		char c= 0;
		/* All previous seeks put us at the end of the file */
		for(i=0; i < off2; i++)
			good= write(fd,&c,1);
		/*res= lseek(fd, 0, SEEK_END);*/
		res= tell(fd);	
	}
	return res;

}
