# Mdc2lst.tcl --
#
# Parser for Manuel de Codage Files.
# Transform them in Tcl lists

#  2000 Serge Rosmorduc,
# quipe EC.art, Laboratoire LRIA, Universit de Paris 8
#
# 15 rue Catulienne
# 93236 St Denis Cedex
# FRANCE
#
# Tout droits rservs.
# 
# voyez le fichier ../LICENCE.txt pour plus d'informations
# See the ../LICENCEE.txt file for more information
#

package provide mdc2lst 0.0

namespace eval mdc2lst {
    variable VARS

    proc getToken {} {
	
    }
    
    proc next {} {
	
    }
    
    proc tokenize {l} {
	variable VARS
	set result {}
	set end 0
	set res [regexp -start $end -indices -inline -- $VARS(regexp) $l]
	# We find which element is not empty, and then we have 
	# The code
	set num 0
	while {[llength $res] != 0} {
	    incr num
	    set end [lindex [lindex $res 0] 1]
	    set name $VARS(tokennames)
	    foreach p [lrange $res 1 end] {
		break
		if {[lindex $p 1] == $end} {
		    lappend result [list [lindex $name 0] [string range $l [lindex [lindex $res 0] 0] $end]]
		    break;
		}
		set name [lrange $name 1 end]
	    }
	    incr end
	    set res [regexp -start $end -indices -inline -- $VARS(regexp) $l]
	}
	puts "$num token read"
	return $result
    }
    
    set VARS(tokens) {
	{(?:[A-Z]|aA)[0-9]+A*|[A-Za-z`]+}	"SIGN"
	{\:}	":"
	{\*}	"*"
	{\&\&?}	"&"
	{\(}	"BPAR"
	{\)}	"EPAR"
	{-|\+s|\{[lL][0-9]+,[0-9]+\}|\{\{[0-9]+,[0-9]+,[0-9]+\}\}}	"SEP"
	{!!|!!=[0-9]+%}	"LINE"
	{\+[jbictl](?:[^+]|\++[^s])*\++s}	"ALPHA"
	{\|[^-]*-}	"SUPER"
	{\+dg|\+gd|\+lr|\+lr|\#[be]?|\$[rb]?}	"TOGGLE"
	{\\!}	"KERNING"
	{\.} 	"SPACE"
	{\.\.} 	"HALFSPACE"
	{[/hv]?/}	"HATCHING"
	{/[hv]}	"HATCHING"
	{\^|\?|\?\?|\[\{|\[\[|\[\|\[\'|\[\&|\}\]|\]\]|\"\]|\'\]|\&\]}	"SPECIAL"
	{\<[SFH]?[hbme]?[1-3]?} "BCAR"
	{>|[HhsSfF]?[1-3]>} "ECAR"
	{\#\#|\#(?=[a-zA-Z])} "OVER"
	{\#[1-4]+}	"HATCHING"
	{\\|\\[sRrt]?[0-9]*}	"MODIF"
	{\+\+COORDS\=(?:[^+]|\++[^s])*\++s}	"COORDS"
	{\+\+(?:[^+]|\++[^s])*\++s}	"COMMENT"
	{\@[^\n]*[\n]}	"COMMENT"
    }

    proc prepareRegexp {} {
	variable VARS
	set VARS(regexp) ""
	set VARS(tokennames) ""
	set i 1
	foreach {reg repl} $VARS(tokens) {
	    if {$i > 1} {
		append VARS(regexp) "|"
	    }
	    incr i
	    lappend VARS(tokennames) $repl
	    #append VARS(regexp) "(?:$reg)"
	    append VARS(regexp) "($reg)"
	}
	#append VARS(regexp) ")"
    }
    
    prepareRegexp
    proc test {} {
	set f [open toto]
	set txt [read $f]
	close $f
	foreach l [tokenize $txt] {
	    puts "Token :$l"
	}
    }

    ParseMdc {l} {
	switch -- [getToken $l] {
	    SIGN {}
	    BPAR
	    EPAR
	    SEP
	    ALPHA
	    SUPER
	    TOGGLE
	    KERNING
	    SPACE
	    HALFSPACE
	    HATCHING
	    SPECIAL
	    BCAR
	    ECAR
	    OVER
	    HATCHING
	    MODIF
	    COORDS
	    COMMENT
	    default {error "syntax error"}
	}
    }
}
