/*
 * Parser from Manuel de Codage to Tcl lists.
 * The goal of this parser is to allow easy manipulation of manuel de
 * codage data through Tcl.
 * 
 * The parser is a simple recursive descend parser. Yacc or Bison
 * parsers have a problem : it's difficult to deallocate memory after
 * an error. In our case, this is really annoying.
 * 
 */
/*  1998, 2000 Serge Rosmorduc,
 * quipe EC.art, Laboratoire LRIA, Universit de Paris 8
 *
 * 15 rue Catulienne
 * 93236 St Denis Cedex
 * FRANCE
 *
 * Tout droits rservs.
 * 
 * voyez le fichier ../LICENCE.txt pour plus d'informations
 * See the ../LICENCEE.txt file for more information
 *
 * $Id$
 *
 */

/*
 * $Log$
 */


parseMDC()
{
  /*
   * mdcfile ::= [SEPARATION] (KERNING* textitem [SEPARATOR])*
   * textitem ::= cadrat [HACHING] | SIMPLEITEM
   */
  if (getToken() == SEPARATOR)
    {
      nextToken();
    }
  /* start(textitem)= SIMPLEITEM U start(cadrat)
   *                = SIMPLEITEM U '(' U STARTCONSTRUCT U STARTCCART
   *                  U GRAMMAR U SIGN
   */

  while (getToken() == SIMPLEITEM 
	 || getToken() == SIGN
	 || getToken() == '('
	 || getToken() == GRAMMAR	 
	 || getToken() == STARTCCART
	 || getToken() == STARTCONSTRUCT
	 || getToken() == KERNING)
    {
      while (getToken() == KERNING)
	{
	  nextToken();
	}
      if (getToken() == SIMPLEITEM)
	{
	  nextToken();
	}
      else
	{
	  parseCadrat();
	  if (getToken() == HACHING)
	    {
	      nextToken();
	    }
	}
      if (getToken() == SEPARATOR)
	{
	  nextToken();
	}
    }
}

void
parseCadrat()
{
  /*
    cadrat ::= subcadrat ( ':' KERNING* subcadrat)*
   */
  parseSubCadrat();
  while (getToken() == ":")
    {
      nextToken();
      while (getToken() == KERNING)
	{
	  nextToken();
	}
      parseSubCadrat();
    }
}

void
parseSubCadrat()
{
  /*
   * subcadrat ::= inhierlist ('*' KERNING* inhierlist)*
   */
  
  parseInHierList();
  while (getToken() == "*")
    {
      nextToken();
      while (getToken() == KERNING)
	{
	  nextToken();
	}
      parseInHierList();
    }
}

void
parseInHierList()
{
  /*
   * inhierlist ::= lig
   *                | '(' mdctext ')'
   *	            | STARTCONSTRUCT mdctext ENDCONSTRUCT
   *	            | STARTCART mdctext ENDCART
   * start(lig)= GRAMMAR U SIGN
   */

  if (getToken() == '(')
    {
      nextToken();
      parseMDC();
      if (getToken() == ')')
	nextToken();
    }
  else if (getToken() == STARTCONSTRUCT)
    {
      nextToken();
      parseMDC();
      if (getToken() == ENDCONSTRUCT)
	nextToken();
    }
  else if (getToken() == STARTCCART)
    {
      nextToken();
      parseMDC();
      if (getToken() == ENDCART)
	nextToken();
    }
  else if (getToken() == GRAMMAR || getToken() == SIGN)
    {
      parseLigs();
    }
}

parseLig()
{
  /*
   * lig ::= hieroglyphs ('&' hieroglyphs)*
   * hieroglyphs ::= hieroglyph [OVERWRITE hieroglyph]
   *
   */

  parseHieroglyph();
  if (getToken() == OVERWRITE)
    {
      nextToken();
      parseHieroglyph();
    }
  while (getToken() == '&')
    {
      parseHieroglyph();
      if (getToken() == OVERWRITE)
	{
	  nextToken();
	  parseHieroglyph();
	}
    }
}

void
parseHieroglyph()
{
  /*
   * hieroglyph ::= [GRAMMAR] SIGN MODIFIER* [WORDEND|SENTENCEEND]
   */

  if (getToken() == GRAMMAR)
    {
      nextToken();
    }
  if (getToken() == SIGN)
    {
      
      nextToken();
      while (getToken() == MODIFIER)
	{
	  nextToken();
	}
      if (getToken() == WORDEND || getToken() == SENTENCEEND)
	{
	  nextToken();
	}
    }
  else
    {
      // error
    }
}
