######################################################################
#
# Package for browsing the dictionnary
#
# Multiple browsers can be opened on the dictionnary.
#
# Typically, a browser contains a window
# and a list. When text is entered in the top window,
# the list will be updated so that the text it displays
# be *after* the top window's text in alphabetic order.

#  set SESHLIB [pwd]
#  set env(SESHLIB) [pwd]
#  puts $env(SESHLIB)  
#  lappend auto_path $env(SESHLIB) \
#      [file join $env(SESHLIB) stooop]\
#      [file join $env(SESHLIB) scwoop]

package require seshutils
package require seshbase
package require dico
package require yellow
package require legendes


package provide dicobrowse 0.0

namespace eval dicobrowse {
    namespace export create
    namespace export setfirst
    namespace export next load
    namespace export getval
    namespace export setfirst
    
    variable VARS
    

    proc create {w args} {
	variable VARS

	array set opt {
	    -head MTRI
	    -unnorm dicobrowse::identity
	}
	
	set newargs {}
	
	foreach {a b} $args {
	    if [info exists opt($a)] {
		set opt($a) $b
	    } else {
		lappend newargs $a
		lappend newargs $b
	    }
	}

	set VARS($w,head) $opt(-head)
	set VARS($w,unnorm) $opt(-unnorm)
	set VARS($w,atend) 0

	frame $w
	eval entry $w.entr -textvariable dicobrowse::VARS($w,first)\
		$newargs

	yellow::create $w.entr [legendes::trans "Type here the beginning\n of the word you search"]
	
	eval "listbox $w.list -width 20 -height 10 -yscroll"\
		[list "dicobrowse::setbar $w"] {-takefocus 1}\
		$newargs

	#scrollbar $w.s -command "$w.list yview"
	scrollbar $w.s -command "dicobrowse::getbar $w"
	
	grid $w.entr -
	grid $w.list $w.s -sticky news

	grid columnconfigure $w 0 -weight 1
	grid rowconfigure $w 1 -weight 1

	bind $w.entr <Return> [list dicobrowse::init $w]
	bind $w <Destroy> {dicobrowse::free %W}
	bind $w.list <Return> [list dicobrowse::loadbr $w]
	bind $w.list <Double-Button-1> [list dicobrowse::loadbr $w]
	bind $w.list <Down> [list dicobrowse::down $w]

	init $w

	return $w
    }
    
    proc identity {x} {
	return $x
    }

    proc free {w} {
	variable VARS
	seshbase::close_scan $VARS($w,scan)
	foreach n [array names VAR $w,* ] {
	    unset VARS($n)
	}
    }
    
    proc setbar {w a b} {
	#$w.s set $a [expr $a + 0.2]
	$w.s set $a  [expr $a + 0.2]
    }

    proc getbar {w args} {
	set com [lindex $args 0]
	switch -- $com {
	    moveto {
		set v [lindex $args 1]
		if {$v > 0.5} {
		    next $w
		}
	    }
	    scroll {
		set val [lindex $args 1]
		set unit [lindex $args 2]
		set yv [lindex [$w.list yview] 1]
		if {$yv >= 0.5} {
		    next $w
		}
	    }
	}
	eval $w.list yview $args
    }
    
    proc down {w} {
	set yv [lindex [$w.list yview] 1]
	if {$yv >= 0.5} {
	    next $w
	}
    }

    proc getval {w} {
	variable VARS
	return [dico::find_index\
		$VARS($w,head)\
		[$w.list get active]]
    }
    
    proc setfirst {w first} {
	variable VARS
	set VARS($w,first) $first
	init $w
    }

    proc listfrom {w num {exclude 0}} {
	variable VARS

	set t [seshbase::scanget $VARS($w,scan) $num]
	if $exclude {
	    set t [lrange $t 1 end]
	}

	foreach item $t {
	    $w.list insert end [$VARS($w,unnorm) [lrange $item 1 end]]
	}
    }

    proc listexpand {w pos} {
	switch $pos {
	    end {
		set h [expr 2 * [$w.list cget -height]]
		listfrom $w $h 1
	    }
	    start {
		error "I'm not able to do that yet"
	    }
	}
    }
    
    proc setload {w fonc} {
	variable VARS
	set VARS($w,callback) $fonc
    }

    proc loadbr {w} {
	variable VARS
	$VARS($w,callback) $w
    }

    proc init {w} {
	variable VARS
	set VARS($w,scan) [seshbase::open_scan \
		$VARS($w,head)\
		[dico::tag_for_indexing $VARS($w,head) $VARS($w,first)]]
	set h [expr 2 * [$w.list cget -height]]
	$w.list delete 0 end
	listfrom $w $h
    }

    proc next {w} {
	variable VARS
	set last [lindex [$w.list yview] 1]
	if {$last >= 1} {
	    listexpand $w end
	}
	#$w.list yview scroll 1 units
    }
}

#  # essai
#  dicobrowse::create .w
#  grid .w -sticky news
#  grid rowconfigure . 0 -weight 1
#  grid columnconfigure . 0 -weight 1


