# seshbase.tcl --
#
#        basic database stuff for sesh.
#        SUBJECT TO CHANGE.
#
# Copyright (c) 1998 Serge Rosmorduc
#

#
# All information about sesh is 
# stored in two databases : pub and priv
# priv is someone's base. pub is a compilation of personnal bases.
# pub should only be built using the function load_file
# export functions will allow information exchange between persons.
# Each "card" will be owned by one person
#
#

package provide seshbase 0.0

package require Dbinter
package require seshutils
package require seshuser

namespace eval seshbase {
    variable VARS
    namespace export lread;		# reads a list of values
    namespace export lwrite;		# append a list of values to a list of values
    namespace export read1;		# reads only from one base
    namespace export write_ortho;	# write a word orthographies
    namespace export read_ortho;	# reads .....
    namespace export openb closeb
    namespace export readb;		# reads from the bases (private and public)
    namespace export writeb;		# writes to the default base
    namespace export get_max;		# gets a new index in default base
    namespace export get_mode set_mode;	# choses the default base.
    namespace export isnotmine
    namespace export set_ortho_hook
    namespace export user_dump;		# dump user database
    
    namespace export open_scan 
    namespace export close_scan
    namespace export scanget
    

    # VARS(BASE,OWN,...)
    # where BASE = "idx" or "dico" 
    # and OWN = pub (global base) or priv (private base)
    # : info about bases
    #
    #
    # VARS(BASE,OWN,fd) : handle for opened file
    # VARS(BASE,mode) : opening mode for the base : pub or priv
    #     if "priv", you write in user's base; if "pub", in global base

    variable FILES  
    # base file name for the bases :
    # FILES(pub) or FILES(priv)
    variable USERID
    # mode = 1 (user) or 0 (supervisor)

    # default value for mode : private.

    set VARS(idx,opened) 0

    catch {
	set FILES(pub) [file join $::SESHLIB newdic]
	set FILES(priv) [file join [seshuser::getvar PATH] newdic]
    }

    proc openb  {base} {
	variable VARS
	variable FILES
	set VARS($base,pub,fd) -1
	set VARS($base,priv,fd) -1
	set VARS($base,mode) $VARS(mode)
	set mode $VARS(mode)
	if {[string compare $base "idx"] == 0} {
	    set taille 16
	    set f1 "$FILES(pub)_IDX"
	    set f2 "$FILES(priv)_IDX"
	} else {
	    set f1 "$FILES(pub)"
	    set f2 "$FILES(priv)"
	    set taille 128
	}

	if {[info exists VARS($base,opened)] && $VARS($base,opened)} {
	    error "database already opened !!"
	}

	set VARS($base,opened) 1

	if {[string compare $mode "priv"] == 0} {
	    if [catch {set VARS($base,pub,fd) [open_db $f1 $taille r]}] {
		set VARS($base,pub,fd) -1
	    }
	    set VARS($base,priv,fd) [open_db $f2 $taille w]
	} else {
	    set VARS($base,pub,fd) [open_db $f1 $taille w]
	    if [catch {set VARS($base,priv,fd) [open_db $f2 $taille r]}] {
		set VARS($base,priv,fd) -1
	    }
	}
    }

    
    proc closeb {base} {
	variable VARS
	if {$VARS($base,pub,fd) != -1 } {
	    close_db $VARS($base,pub,fd)
	}
	if {$VARS($base,priv,fd) != -1 } {
	    close_db $VARS($base,priv,fd)
	}
	set VARS($base,opened) 0
    }

    proc writeb {base key value} {
	variable VARS
	set cible $VARS($base,mode)
	put_db $VARS($base,$cible,fd) $key $value
    }
    
    proc readb {base key} {
	variable VARS
	set res ""
	if { $VARS($base,priv,fd) != -1 } {
	    set res [get_db $VARS($base,priv,fd) $key]
	}
	if {[string length $res] != 0} {
	    return $res
	} else {
	    if {$VARS($base,pub,fd) != -1} {
		set res [get_db $VARS($base,pub,fd) $key]
	    }
	    return $res
	}
    }

    proc lread {base key} {
	variable VARS
	set res1 "" 
	set res2 ""
	if { $VARS($base,priv,fd) != -1 } {
	    set res1 [get_db $VARS($base,priv,fd) $key]
	}
	if { $VARS($base,pub,fd) != -1 } {
	    set res2 [get_db $VARS($base,pub,fd) $key]
	}
	return [concat $res1 $res2]
    }

    proc read1 {base key} {
	variable VARS
	set where $VARS($base,mode)
	set res [get_db $VARS($base,$where,fd) $key]
	return $res
    }

    proc lwrite {base key values} {
	variable VARS
	set cible $VARS($base,mode)
	set vals [get_db $VARS($base,$cible,fd) $key]
	set vals [concat $vals $values]
	set vals [uniq $vals]
	put_db $VARS($base,$cible,fd) $key $vals
    }

    proc write_ortho {trans orths} {
	variable VARS
	#openb idx
	lwrite idx [list ORTHO $trans] $orths
	if [info exists VARS(ortho_hook)] {
	    $VARS(ortho_hook) $trans $orths
	}
	#closeb idx
    }

    proc read_ortho {txt} {
	openb idx
	set res [lread idx [list ORTHO $txt]]
	closeb idx
	return $res
    }

    # set_ortho_hook --
    #    defines a hook function that will be called 
    #    when some new orthography is saved in the dictionnary.
    # args --
    #    hook : a function that takes two arguments
    #           trans  : a translitteration
    #           val    : new orthography value
    
    proc set_ortho_hook {hook} {
	variable VARS
	set VARS(ortho_hook) $hook
    }
    
    proc set_mode {mode} {
	variable VARS
	set VARS(mode) $mode
    }

    proc get_mode {} {
	variable VARS
	return $VARS(mode)
    }

    # Get a new index for current user
    # dico should have been opened first

    proc get_max {} {
	set id [seshuser::getvar ID]
	set MAX  [read1 dico MAX]
	if { $MAX == "" } {
	    set MAX 0
	}
	incr MAX
	seshbase::writeb dico MAX $MAX
	return ${id}_${MAX}
    }

    proc isnotmine {idrecord} {
	set id [seshuser::getvar ID]
	if [regexp "^${id}_" $idrecord] {
	    return 0
	} else {
	    return 1
	}
    }

    proc update_max {idx} {
	set id [seshuser::getvar ID]
	if [regexp "^${id}_(\[0-9\]*)\$" $idx tmp nbr] {
	    set MAX [read1 dico MAX]
	    if { $MAX == "" } {
		set MAX 0
	    }
	    if {$nbr > $MAX} {
		seshbase::writeb dico MAX $nbr
	    }
	}
    }

    proc user_dump {} {
	variable FILES
	set db [open_db "$FILES(priv)"]
	set txt [dump_db $db]
	close_db $db
	return $txt
    }

    ######################################################################
    # scan of a base  :
    # 
    # open_scan TAG <start> --
    #   sends back an identifier
    # close_scan <id> --
    #   end of the scan
    #
    # scanget <id> <num> --
    #   get (at most) <num> elements from the  base,
    #   starting with the current starting point.
    #
    # isscanend <id> --
    #   are there other elements in the scan ?
    #
    # variables used : 
    #
    # the first and last elements of a scan are used
    #
    # VARS(maxscan) : scan maximum index
    # VARS(scan,$id,start) : scan $id first element
    # VARS(scan,$id,$base,end) : scan $id last element
    # VARS(scan,$id,$base,tail) : elements read in the base, but not sent
    
    set VARS(maxscan) 0

    proc open_scan {TAG start} {
	variable VARS
	set id  [incr VARS(maxscan)]
	set VARS(scan,$id,start) $start
	set VARS(scan,$id,pub,end) $start
	set VARS(scan,$id,priv,end) $start
	set VARS(scan,$id,tag) $TAG
	set VARS(scan,$id,pub,atend) 0
	set VARS(scan,$id,priv,atend) 0
	set VARS(scan,$id,pub,tail) {}
	set VARS(scan,$id,priv,tail) {}
	return $id
    }

    proc close_scan {id} {
	variable VARS
	unset VARS(scan,$id,start) 
	unset VARS(scan,$id,priv,end)
	unset VARS(scan,$id,pub,end)
    }

    # scanaux --
    # id : scan id
    # base : pub or priv
    # num : number of elements to get
    # auxiliary function used by scanget.
    # gets at most num elements from base.

    #
    # btdb is rather stupid when it comes to non existing files.
    # we are careful to test every option here. Anything opened
    # is closed.
    
    proc scanaux {id base num} {
	variable VARS

	set res $VARS(scan,$id,$base,tail)

	if {[llength $res] >= $num} {
	    return $res
	}

	openb idx
	set db $VARS(idx,$base,fd)

	if {$db == -1} {
	    set res {}
	} else {
	    set good [expr ! $VARS(scan,$id,$base,atend)]
	    if [catch {set i [get_db_idx $db $VARS(scan,$id,$base,end)]}] {
		set res ""
	    } else {
		for {set j 0} {$j < $num && $good} {incr j} {
		    set last [getfrom_db_idx -key $i]
		    
		    set tag_is_bad [string compare \
			    [lindex $last 0] \
			    $VARS(scan,$id,tag)]
		    
		    if $tag_is_bad {
			set good 0
			set VARS(scan,$id,$base,atend) 1
		    } else {
			lappend res $last
			set VARS(scan,$id,$base,end) $last
			set good [incr_db_idx $i 1]
		    }
		}
		if {! $good} {
		    set VARS(scan,$id,$base,atend) 1
		}
		free_db_idx $i
	    }
	}
	closeb idx
	return $res
    }


    proc scanget {id num} {
	variable VARS
	set l1 [scanaux $id pub $num]
	set l2 [scanaux $id priv $num]

	# now, if possible, l1 and l2 contain 
	# enough elements.
	set i 0
	set res {}
	while {$i < $num && [not_empty l1 l2]} {
	    get_best res l1 l2
	    incr i
	}
	set VARS(scan,$id,pub,tail) $l1
	set VARS(scan,$id,priv,tail) $l2

	return $res
    }

    proc not_empty {l1 l2} {
	upvar $l1 la
	upvar $l2 lb
	return [expr [llength $la] != 0 || [llength $lb] != 0]
    }

    proc get_best {res l1 l2} {
	upvar $l1 la
	upvar $l2 lb
	upvar $res liste
	set lgtha [llength $la]
	set lgthb [llength $lb]
	
	if {$lgtha != 0 && $lgthb != 0} {
	    set a [lindex $la 0]
	    set b [lindex $lb 0]
	    if {[string compare $a $b] < 0} {
		# we take a
		lappend liste $a
		set la [lrange $la 1 end]
	    } else {
		# we take b
		lappend liste $b
		set lb [lrange $lb 1 end]
	    }
	} else {
	    if {$lgtha != 0} {
		lappend liste [lindex $la 0]
		set la [lrange $la 1 end]
	    }
	    if {$lgthb != 0} {
		lappend liste [lindex $lb 0]
		set lb [lrange $lb 1 end]
	    }
	}
    }


    proc isscanend {id} {
	error "not written yet. Maybe never"
    }

    set_mode priv
}

