#
# widget to display and modify the font database.
#

#
# The window is cut in three parts :
#
# part a) : navigator.
# part b) : current glyph
# part c) : graphical display of glyphs

# Navigator :
#
# Allows :
#	to select a set of glyphs :
# 		by official font family
#		by declared font family
#		by user (current user/other user)
#
#	to create a new glyph.
#

package require seshw

namespace eval DBFontDisplay {
    variable VAR
    variable drawer 0

    proc create {w appli} {
	variable VAR

	set VAR($w,appli) $appli
	set VAR($w,modified) 0

	# A note on ScrollableFrame : ScrollableFrame *doesn't* have scrollbars !
	# it's meant to be used with ScrolledWindow

	ScrolledWindow $w

	ScrollableFrame $w.fr
	$w setwidget $w.fr

	set win [$w.fr getframe]
	
	setChanged $w 0

	for {set i 0} {$i < 256} {incr i} {
	    set lab $win.charNum$i
	    label $lab -text $i
	    button [getButton $w $i]
	    set img [image create bitmap [getImageName $w $i]]
	    [getButton $w $i] configure  -width 40 -height 40 -image [getImageName $w $i]
 	    grid $lab [getButton $w $i] 
	    bind [getButton $w $i] <Double-Button-1> {DBFontDisplay::openChar %W}
	    setButtonWindow [getButton $w $i] $w
	    setButtonCharNum [getButton $w $i] $i
	}
	
	bind $w <Destroy>  {DBFontDisplay::deleteDBFontDisplay %W}
	return $w
    }

    proc setButtonWindow {b w} {
	variable VAR
	set VAR(button,$b,window) $w
    }

    proc getButtonWindow {b} {
	variable VAR
	return $VAR(button,$b,window)
    }

    proc setButtonCharNum {b i} {
	variable VAR
	set VAR(button,$b,num) $i
    }

    proc getButtonCharNum {b} {
	variable VAR
	return $VAR(button,$b,num)
    }
    
    proc deleteDBFontDisplay {w} {
	variable VAR
	puts "goodBye $w"
	# Free ressources :
	array unset VAR($w,*)
	# free the ressources associated with buttons :
	# TO DO
    }

    proc openChar {b} {
	variable VAR
	set w [getButtonWindow $b]
	set app [getAppli [getButtonWindow $b]]
	set font [getFont [getButtonWindow $b]]
	set num [getButtonCharNum $b]
	set txt [BZRFont::getChar $font $num]
	# The following line creates a dependency between FontDisplay and the CharEditAppli.
	# This should not be. A simple solution would be to use a callBack
	CharEditAppli::editFontChar $app $txt [list DBFontDisplay::setChar $w $num] "[BZRFont::getFileName $font]($num)" $w
    }

    proc setChar {w num txt} {
	BZRFont::setChar [getFont $w] $num $txt
	updateChar $w $num
	setChanged $w 1
    }

    proc getAppli {w} {
	variable VAR
	return $VAR($w,appli)
    }

    # For use with pack !
    proc listButtons {w} {
	set r [list]
	for {set i 0} {$i < 256} {incr i} {
	    lappend r [getButton $w $i]
	}
	return $r
    }

    proc getButton {w i} {
	#return [$w.fr getframe].char$i.b
	return [$w.fr getframe].b$i
    }

    proc getDrawer {} {
	variable drawer
	if {[string compare $drawer "0"] == 0} {
	    seshw::create_passive .dummy -htline 40
	}
	set drawer .dummy
	return $drawer
    }
    
    proc updateChar {w i} {
	set f [getFont $w]
	if {[BZRFont::hasChar $f $i]} {
	    set txt [BZRFont::getChar $f $i]
	    setDisplayChar $w $i $txt
	}
    }
    
    proc setDisplayChar {w num txt} {
	set d [getDrawer]
	tksesh_defchar temp $txt
	seshw::set_text $d temp
	seshw::autosize $d
	set img [image create bitmap [getImageName $w $num] -data [$d toxbm 1.0 end]]
	[getButton $w $num] configure -image [getImageName $w $num]
    }

    proc getImageName {w num} {
	return "fonticon $w $num"
    }

    proc setFont {w f} {
	variable VAR
	set VAR($w,font) $f
	for {set i 0} {$i < 256} {incr i} {
	    updateChar $w $i
	}
	setChanged $w 0
    }
    
    proc getFont {w} {
	variable VAR 
	return $VAR($w,font)
    }

    proc getChanged {w} {
	variable VAR
	return $VAR($w,changed)
    }

    proc setChanged {w v} {
	variable VAR
	set VAR($w,changed) $v
    }
    
}
