# Defines toolbars for the font editor application
# A toolbar controls an editor application.


namespace eval CharEditToolBar {

    variable VAR
    
    # Describe and define the state buttons.
    # fields : button text, button help, state controled by the button.
    variable stateButtons {
	"Edit" "Edit, move contours, etc." edit
	"Add"  "Create new contours" add
	"Delete" "Delete stuff" delete
	"Add Guide" "Add guide lines" addguide
	"spline <-> Line" "transform splines into lines and vz." splinetoline
	"select" "select" select
	"lower" "lowers a contour" lower
	"split" "adds a control point on a curve" split
	"reverse" "transforms a positive contour into a negative one and vz." reverse
    }
    
    proc add {w} {
	variable VAR
	variable stateButtons
	
	set tb [$w addtoolbar]
	# select black/white contour.
	set bbox1 [frame $tb.bbox1]

	radiobutton $bbox1.black -indicatoron 0 -background black -width 1 -selectcolor black -value "P" -borderwidth 5 -variable CharEditToolBar::VAR($w,fill) -command [list CharEditToolBar::setBlack $w]
	DynamicHelp::register $bbox1.black balloon "Created contours will be filled in black" 
	radiobutton $bbox1.white -indicatoron 0 -background white -width 1 -selectcolor white -value "N" -borderwidth 5 -variable CharEditToolBar::VAR($w,fill) -command [list CharEditToolBar::setWhite $w]
	DynamicHelp::register $bbox1.white balloon "Created contours will be filled in white" 
	pack $bbox1.black $bbox1.white -side left 

#  	$bbox1 add -width 1 -height 1 \
#  		-background black -helptext "Created contours will be filled in black"\
#  		-command [CharEditToolBar::setBlack $w]
#  	$bbox1 add -width 1 -height 1 \
#  		-background white -helptext "Created contours will be filled in white"\
#  		-command [CharEditToolBar::setWhite $w]
	setEditor $w -1
	$w showtoolbar 0 1
	pack $bbox1 -side left
	addSeparator $w 0 0

	# select operation.
	# will be linked automagically to the variables.
	set bbox2 [frame $tb.bbox2]
	set i 0
	foreach {btext bhelp bstate} $stateButtons {
	    set mypath $bbox2.button$i
	    incr i

	    setStateButtonPath $w $bstate $mypath
	    radiobutton $mypath -indicatoron 0  -value $bstate -text $btext 
	    DynamicHelp::register $mypath balloon  $bhelp
	    pack $mypath -side left 
	}
	pack $bbox2 -side left
	addSeparator $w 0 1

	set tb1 [$w addtoolbar]
	set scaler [scale $tb1.scale -from -10  -to 10 \
		     -tickinterval 5.0 -resolution 0.1 -length 200\
		     -command [list CharEditToolBar::scaleEditor $w] -orient horizontal]
	pack $scaler -side left
	setScaler $w $scaler
	addSeparator $w 1 1
	$w showtoolbar 1 1
    }

    
    proc setScaler {w s} {
	variable VAR
	set VAR($w,scaler) $s
    }

    proc getScaler {w} {
	variable VAR
	return $VAR($w,scaler)
    }

    proc delete {w} {
    }

    proc addSeparator {w tbnum i} {
	pack [Separator [$w gettoolbar $tbnum].sep$i -orient vertical] -side left -fill y -padx 4
    }

    proc setEditor {w e} {
	variable VAR	
	set VAR($w,editor) $e
	bindStates $w
    }

    proc setStateButtonPath {w state path} {
	variable VAR
	set VAR($w,statepath,$state) $path
    }

    proc getStateButtonPath {w state} {
	variable VAR
	return $VAR($w,statepath,$state)
    }

    proc bindStates {w} {
	variable VAR
	variable stateButtons
	
	set editor [getEditor $w]
	# Links the states of w with an actual editor

	if {$editor != -1} {
	    set var CharEdit::($editor,state)
	    
	    foreach {btext bhelp bstate} $stateButtons {
		set p [getStateButtonPath $w $bstate]
		$p configure -variable $var
	    }
	    set s [CharEdit::getScale $editor]
	    [getScaler $w] set $s
	    set VAR($w,fill) [CharEdit::getFill $editor]
	}
    }


    # Should probably move to apply
    proc getEditor {w} {
	variable VAR	
	return $VAR($w,editor)
    }
    
    proc hasEditor {w} {
	if {[getEditor $w]  != -1} {
	    return 1
	} else {
	    return 0
	}

    }

    # Auxiliary actions for functions.
    # values will be retrieved and set through setEditor and getEditor.
    
    proc setWhite {w} {
	if {[hasEditor $w]} {
	    CharEdit::setUnFilled [getEditor $w]
	}
    }

    proc setBlack {w} {
	if {[hasEditor $w]} {
	    CharEdit::setFilled [getEditor $w]
	}
    }
    
    proc scaleEditor {w scale} {
	if {[hasEditor $w]} {
	    CharEdit::scaleall [getEditor $w] $scale
	}
    }
}
