# Clean and nicer version of the sign editor

# * Using the nice BWidgets
# * interfaced with the database
# * multiple windows
# * undo/redo facilities

# Export facilities for bzrfonts
# dynamic cut and paste.

# BWidget Implementation

# Note on font edition :
# Load Font : create a new font window in the navigator.
# New Font 
# Close Current Font
# 
# Interaction between the font windows and the char windows :
# Double-click on a char in the font windows : create an editor for this
# 	char. Font name and char number appear in the title. 
#
#
# drag and drop ????
#
#

package require seshuser
seshuser::loaduser

package require BWidget 1.3

#TitleFrame .tf

namespace eval CharEditAppli {
    variable VAR
    set VAR(VERSION) 2
    set VAR(date) {Mon Jun  4 14:54:39 CEST 2001}

    proc create {w} {
	variable VAR
	set VAR($w,numEdit) 0
	set VAR($w,numFont) 0
	setEditor $w -1 -1
	set descmenu [buildMenu $w]
	    
	set CharEditAppli [MainFrame $w\
		-textvariable CharEditAppli::VAR($w,status)\
		-menu $descmenu\
		]
	$w addindicator -text "The TKsesh Font Editor"
	$w addindicator -text "Version $VAR(VERSION) ($VAR(date))"

	set VAR($w,status) ready
	
	set paned [$w getframe].pw
	PanedWindow $paned -side bottom
	
	set p1 [$paned add -minsize 300]
	set p2 [$paned add -minsize 100]
	
	CharEditToolBar::add $w
	
  	NoteBook [getCharBook $w] -width 300 -height 300
	#createSignWindow $w

	#$win.book compute_size
	NoteBook [getFontBook $w] -width 100 -height 300
	
	pack $paned -fill both -expand yes
	pack [getCharBook $w] -fill both -expand yes
	pack [getFontBook $w] -fill both -expand yes
	#pack $p1.book $p2.chooser -fill both -expand yes -padx 4 -pady 4 -side left
	#grid $win.book $win.chooser -sticky news
	#grid  $win.chooser -sticky news
	#grid rowconfigure $win 0 -weight 1
	#grid columnconfigure $win 0 -weight 1
	bind $w <Destroy> [list CharEditAppli::destroy $w]
    }

    proc destroy {w} {
	variable VAR
	array unset VAR $w,*
    }

    proc buildMenu {w} {
	set res {
	    "&File" all file 0 {
		{
		    command "&New Font" {} "Create a new BZR Font" {}
		    -command {CharEditAppli::newFont WINDOW}
		}
		{
		    command "&Load Font" {} "Load a font from a bzr file" {}
		    -command {CharEditAppli::loadFont WINDOW}
		}
		{
		    command "&Save Font" {} "save a font in a bzr file" {}
		    -command {CharEditAppli::saveFont WINDOW}
		}
		{
		    command "&Save Font As" {} "save a font in a bzr file" {}
		    -command {CharEditAppli::saveFontAs WINDOW}
		}
		{
		    command "&Export Font to MF" {} "save a font in a metafont file for use with HieroTeX. Warning : currently doesn't fix path winding.\nUse it just after *loading* a font." {}
		    -command {CharEditAppli::exportFontMF WINDOW}
		}
		{
		    command "&Export Font to Type 1" {} "save a font as a postscript type 1 file. Warning : currently doesn't fix path winding.\nUse it just after *loading* a font." {}
		    -command {CharEditAppli::exportFontType1 WINDOW}
		}

		{
		    command "Delete Font window" {} "Delete current font window" {}
		    -command {CharEditAppli::deleteUpperFontWindow WINDOW}
		}
		separator
		{
		    command "&New sign window" {} "Open a new sign window" {}
		    -command {CharEditAppli::createSignWindow WINDOW}
		}
		{
		    command "&Delete sign window" {} "Delete current sign window" {}
		    -command {CharEditAppli::deleteUpperSignWindow WINDOW}
		}
		{
		    command "&Validate sign window" {} "Save the current sign where it belongs" {}
		    -command {CharEditAppli::validateUpperSign WINDOW}
		}
		separator
		{
		    command "Load Image" {} "Load Background image" {} -command {CharEditAppli::loadImage WINDOW}
		}
		{
		    command "Clear Image" {} "Clear Background image" {} -command {CharEditAppli::clearImage WINDOW}
		}
		separator
		{
		    command "E&xit" {} "Exit BWidget demo" {} -command exit
		}
	    }

	    "&Edit" all edit 0 {
		{
		    command "&Undo" {} "Undo last command" {}
		    -command {CharEditAppli::undo WINDOW}
		}
		separator
		{
		    command "Cut" {} "Cut selected font part" {}
		    -command {CharEditAppli::cut WINDOW}
		}
		{
		    command "Copy" {} "Copy selected font part " {}
		    -command {CharEditAppli::copy WINDOW}
		}
		{
		    command "Paste" {} "Paste  selected font part" {}
		    -command {CharEditAppli::paste WINDOW}
		}
	    }
	    "&Options" all options 0 {
	    }
	}
	regsub -all -- WINDOW $res $w res
	return $res
    }
    
    proc getCharBook {w} {
	set f [[$w getframe].pw getframe 0]
	return $f.book 
    }
    
    proc getFontBook {w} {
	set f [[$w getframe].pw getframe 1]
	return $f.book
    }

    proc createSignWindow {w} {
	variable VAR
	incr VAR($w,numEdit)
	set i $VAR($w,numEdit)
	set win [$w getframe]

	set editorID editor$i
	
  	set page [[getCharBook $w] insert end $editorID -text "Editor $i" -raisecmd [list CharEditAppli::selectEditor $w $editorID]]
	
	# as the canvas hasn't got any scaling facilities,
	# using the canvas scroll commands would require us to use *THREE* coordinate systems.
	# So, we do it by hand.
	# ScrolledWindow $page.scroll -auto both -scrollbar both

	frame $page.scroll
	#set editorWidget  [new CharEdit $page.scroll -xscrollcommand "exec echo $page.scroll.xs set" \
	#	-yscrollcommand "exec echo $page.scroll.ys set"]

	set editorWidget  [new CharEdit $page.scroll]
	

	set e $widget::($editorWidget,path)

	scrollbar $page.scroll.xs -command "CharEdit::xview $editorWidget" -orient horizontal
	scrollbar $page.scroll.ys -command "CharEdit::yview $editorWidget" -orient vertical

	CharEdit::setscale $editorWidget 10 20 300
	CharEdit::repere $editorWidget
	CharEdit::setXScrollBar $editorWidget $page.scroll.xs
	CharEdit::setYScrollBar $editorWidget $page.scroll.ys
	CharEdit::notifyScrollBars $editorWidget

	grid $e $page.scroll.ys -sticky news
	grid $page.scroll.xs - -sticky ew
	grid columnconfigure $page.scroll 0 -weight 1
	grid rowconfigure $page.scroll 0 -weight 1

	#$page.scroll setwidget $e

	setEditor $w $editorID $editorWidget

	#pack $e -fill both -expand yes
	pack $page.scroll -fill both -expand yes
	[getCharBook $w] raise $editorID
	return $editorID
    }
    
    proc selectEditor {w i} {
	CharEditToolBar::setEditor $w [getEditorObject $w $i]
    }

    # getEditorObject --
    # 	returns the editor Object for a given id.
    #
    
    proc getEditorObject {w i} {
	variable VAR
	return $VAR($w,editor,$i)
    }

    # setEditor --
    # 	register the editor Object for a given id.
    #
    
    proc setEditor {w i e} {
	variable VAR
	set VAR($w,editor,$i) $e
    }

    # undo --
    # 	undo the last command in the current font editor
    #
    proc undo {w} {	
	set e [getEditorObject $w [getCurrentEditorPage $w]]
	CharEdit::undo $e
    }

    proc getCurrentEditor {w} {
	set e [getEditorObject $w [getCurrentEditorPage $w]]
	return $e
    }

    
    proc getCurrentEditorPage {w} {
	set win [getCharBook $w]
	return [$win raise]
    }

    proc getCurrentFontPage {w} {
	set win [getFontBook $w]
	return [$win raise]
    }

    proc getCurrentFontWindow {w} {
	return [getFontWindow $w [getCurrentFontPage $w]]
    }

    proc deleteUpperFontWindow {w} {
	set win [getFontBook $w]
	# catch to protect us if there is no page to destroy.
	#catch {}
	    set page [getCurrentFontPage $w]
	    set fwin [getCurrentFontWindow $w]
	    set allClear 1
	    foreach s [getCharEditsForFont $w $fwin] {
		set allClear [expr $allClear && [deleteSignWindow $w $s]]
	    }
	    if {! $allClear} {
		MessageDlg .couldNotDo -icon error -type ok \
			-message "There are still edited signs in this font.\n I could not delete the font window."
		return
	    }
	    if {[FontDisplay::getChanged $fwin]} {
		set answer [MessageDlg .validateDestroy -icon warning -type okcancel \
			-message "This font hasn't been saved.\nYou will lose your edits.\nProceed ?"]
		if {$answer == 1} {
		    return
		}
	    }
	    $win delete $page
	    # Should probably be elsewhere :
	    variable VAR
	    array unset VAR $w,fontwin,$fwin,*
	    set l [$win pages]
	    if {[llength $l] > 0} {
		$win raise [lindex $l 0]
	    }
	#{}
    }

    
    proc deleteUpperSignWindow {w} {
	set result 0
	set win [getCharBook $w]
	# catch to protect us if there is no page to destroy.
	catch {
	    set page [getCurrentEditorPage $w]
	    set edit [getCurrentEditor $w]
	    set result [deleteSignWindow $w $page]
	}
	return $result
    }

    # CharEditAppli::deleteSignWindow --
    # 	deletes a sign displaying window
    # Parameters:
    #	w : the chareditappli the window belongs to
    #	p : the page to delete
    # Options:
    # Results:
    # 	1 if the window was deleted else 0 
    # Side Effects:
    #	the page is deleted
    # Use:
    #	PUBLIC
    # Called by:
    
    proc deleteSignWindow {w p} {
	set charbook [getCharBook $w]
	set edit [getEditorObject $w $p]
	if {[CharEdit::getChanged $edit]} {
	    set label [[getCharBook $w] itemcget $p -text]
	    set answer [MessageDlg .validateDestroy -icon warning -type okcancel \
		    -message "char $label hasn't been saved.\nYou will lose your edits.\nProceed ?"]
	    if {$answer == 1} {
		return 0
	    }
	}
	unRegisterCharEditForFont $w $p
	$charbook delete $p
	set l [$charbook pages]
	if {[llength $l] > 0} {
	    $charbook raise [lindex $l 0]
	}
	return 1
    }

    proc validateUpperSign {w} {
	variable VAR
	# catch to protect us if there is no page
	catch {
	    set page [getCurrentEditorPage $w]
	    set edit [getCurrentEditor $w]
	    set txt [CharEdit::asText $edit]
	    CharEdit::setChanged $edit 0
	    eval [getCharWValidator $w $page] [list $txt]
	}
    }
    
    # loadFont --
    # 	Loads a BZR font
    #	Create a corresponding font window.

    proc loadFont {w} {
	variable VAR
	set patterns {{{BZR file} {.bzr}} {Any *}}
	set name [tk_getOpenFile \
		-defaultextension .bzr -filetypes $patterns]
	
	set b [new BZRFont]
	BZRFont::load $b $name
	
	addFontWindow $w $b
	# temporary :
	#  	createSignWindow $w
	#  	set e [getCurrentEditor $w]
	#  	set txt [BZRFont::getChar $b 32]
	#  	CharEdit::setText $e $txt
	# SUPRESS THE NEXT LINE : 
	# set VAR($w,font) $b
    }

    proc newFont {w} {
	variable VAR
	set b [new BZRFont]
	addFontWindow $w $b
    }

    # saveFont --
    # 	save a BZR font

    proc saveFont {w} {
	set fwin [getCurrentFontWindow $w]
	set font [FontDisplay::getFont $fwin]
	BZRFont::saveAs $font [BZRFont::getFileName $font]
    }

    proc saveFontAs {w} {
	set patterns {{{BZR file} {.bzr}} {Any *}}
	set name [tk_getSaveFile \
		-defaultextension .bzr -filetypes $patterns]
	set fwin [getCurrentFontWindow $w]
	set font [FontDisplay::getFont $fwin]
	BZRFont::setFileName $font $name
	BZRFont::saveAs $font $name
	[getFontBook $w] itemconfigure [getCurrentFontPage $w] -text $name
    }

    proc exportFontMF {w} {
	set patterns {{{Metafont file} {.mf}} {Any *}}
	set name [tk_getSaveFile \
		-defaultextension .bzr -filetypes $patterns]
	set fwin [getCurrentFontWindow $w]
	set font [FontDisplay::getFont $fwin]
	BZRFont::exportAsMF $font $name
    }

    proc exportFontType1 {w} {
	set patterns {{{postscript type 1} {.pfa}} {Any *}}
	set name [tk_getSaveFile \
		-defaultextension .bzr -filetypes $patterns]
	set fwin [getCurrentFontWindow $w]
	set font [FontDisplay::getFont $fwin]
	BZRFont::exportAsType1 $font $name
    }

    
    # displayChar --
    # 	displays the num th char from font in the uppermost window.
    #proc displayChar {w font num} {}

    # setCharFromEditor --
    #	sets the numth char in font from the editor window ed
    #proc setCharFromEditor {w font num edi} {}

    # clearCharEdit --
    #	clears a char editor.
    #	if the char has been edited and not saved,
    #	display a warning message.
    #
    #	returns:	1 if we should proceed, 0 if this operation was cancelled.

    proc clearCharEdit {w ced} {
	if {[CharEdit::getChanged $ced]} {
	    MessageDialog .reallyClear -title "Really clear?" -icon question -type okcancel\
		    -message "Warning : char has been modified, but not saved.\n Really clear it ?"
	    set res [Message::create .reallyClear]
	    if {$res != 0} {
		return 0
	    }
	}
	CharEdit::clear $ced
	CharEdit::setChanged 0
	return 1
    }

    # addFontWindow --
    # 	creates a new font window.
    #	w the base window application
    #	b the font
    #
    #	returns : an id for the the window.
    
    proc addFontWindow {w b} {
	variable VAR
	incr VAR($w,numFont)
	set win [$w getframe]
	set pageID "font$VAR($w,numFont)"
	set page [[getFontBook $w] insert end $pageID -raisecmd [list CharEditAppli::selectFont $w $pageID]]
	[getFontBook $w] itemconfigure $pageID -text [BZRFont::getFileName $b]
	set fd [FontDisplay::create [getFontWindow $w $pageID] $w]
	FontDisplay::setFont $fd $b
	pack $fd -fill both -expand yes

	[getFontBook $w] raise $pageID
	initCharEditsForFont $w $fd
	return $pageID
    }

    # getFontWindow --
    #	returns a path for the given font window
    proc getFontWindow {w id} {
	set book [getFontBook $w]
	set path [$book getframe $id].frame
    }
    
    #
    proc selectFont {w id} {
    }
    
    # editChar --
    #	open an editing window for a char
    #	
    #	w : the application window
    #	txt : the char described in the usual font format (see CharEdit.tcl)
    #	validator : a function call which shall be eval-uated when the char is validated.
    #			it should either save the char in its font (for bzrfont chars)
    #			or save the char in the database.
    #			The last argument of the validator will be the text.
    #	label : label to use for the editchar window
    # Results:
    #	sends back the charEdit object which was created.
    
    
    proc editChar {w txt validator label} {
	set i [createSignWindow $w]
	# Set the name for the sign window.
	[getCharBook $w] itemconfigure $i -text $label
	set e [getEditorObject $w $i]
	CharEdit::setText $e $txt
	setCharWValidator $w $i $validator
	return $i
    }


   # truc::fonction --
    # 	description...
    # Parameters:
    #	f: the font window
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    
    proc editFontChar {w txt validator label f} {
	set i [editChar $w $txt $validator $label]
	registerCharEditForFont $w $f $i
    }

    # registerCharEditForFont --
    # 	remembers that a given char editor correspond to a given FontDisplay
    # Parameters:
    #	w :
    #	f : the font editor window
    #	e : the char editor *page*
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:

    proc registerCharEditForFont {w f e} {
	variable VAR
	set VAR($w,isEditFor,$e) $f
	lappend VAR($w,fontwin,$f,charEdits) $e
    }

    # unRegisterCharEditForFont --
    #	un-register a charEdit. To be used when closing the said char editor.
    
    proc unRegisterCharEditForFont {w e} {
	variable VAR
	set f [getFontForChar $w $e]
	unset VAR($w,isEditFor,$e)
	listreplace VAR($w,fontwin,$f,charEdits) $e
    }
    
    proc getFontForChar {w e} {
	variable VAR
	return $VAR($w,isEditFor,$e)
    }

    # truc::fonction --
    # 	description...
    # Parameters:
    #	w :
    #	f : the font window
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    
    proc initCharEditsForFont {w f} {
	variable VAR
	set VAR($w,fontwin,$f,charEdits) [list]
    }

    # truc::fonction --
    # 	description...
    # Parameters:
    #	w :
    #	f : the font *window*
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:

    proc getCharEditsForFont {w f} {
	variable VAR
	return $VAR($w,fontwin,$f,charEdits)
    }

    proc setCharWValidator {w i v} {
	variable VAR
	set VAR($w,$i,validator) $v
    }
    
    proc getCharWValidator {w i} {
	variable VAR
	return $VAR($w,$i,validator)
    }

    proc copy {w} {
	setCharClipboard $w [CharEdit::getSelection [getCurrentEditor $w]]
    }

    proc paste {w} {	
	set txt [getCharClipboard $w]
	CharEdit::pasteAction [getCurrentEditor $w] $txt
    }

    proc cut {w} {

    }

    proc loadImage {w} {
	set patterns {{{gif file} {.gif}} {Any *}}
	set name [tk_getOpenFile \
		      -defaultextension .gif -filetypes $patterns]
	set e [getCurrentEditor $w]
	CharEdit::loadimg $e $name
    }

    proc clearImage {w} {
	set e [getCurrentEditor $w]
	CharEdit::delimg $e
    }

    proc setCharClipboard {w txt} {
	variable VAR
	set VAR($w,clipboard) $txt
    }

    proc getCharClipboard {w} {
	variable VAR
	return $VAR($w,clipboard)
    }
}

CharEditAppli::create .frame
pack .frame -fill both -expand yes



