#
#	Simple SVG-reading system.
#	Currently, only understand svg and path tags

package require xml 2.0

namespace eval svg {
    proc elementStart {name attlist} {
	puts "Element start ==> $name has attributes $attlist"
    }

    # parsePath -- 
    # 	returns a standardized representation of a path.
    # 	the result will be a list, which each element representing a
    #	path element : command + arguments.
    #
    #	notes
    #	path commands are : M x y ; L x y ; Z ; H x; V x;
    #		M/m x y : move to x y	
    #		L/l x y : line to x y
    #		H x :	horizontal line to x
    #		V y :	vertical line to y
    #		C/c x1 y1 x2 y2 x y : cubic Bzier spline
    #		S/s x2 y2 x y : same, but x1 y1 is computed as the reflection of the previous x2 y2, or x y.
    #		Q/q x1 y1 x y : quadratic Bzier.
    #		T/t x y : is to Q/q what s/S is to C/c
    #		A/a rx ry rot largearc sweep x y : arc.
    #		Z/z : the end
    #
    #	Uppercase means "absolute coordinates". Lower case means relative coordinates.
    #
    #	Note that the arc commands won't be recognized for TKSESH font purporses

    proc parsePath {path} {
	set p [pathToList $path]
	set i 0
	set size [llength $p]

	set res [list]
	set n 2
	# previous command
	set command "M"

	while {$i < $size} {
	    # Current element
	    set curr [lindex $p $i]
	    switch -glob $curr {
		{[LlmMtT]} {set n 2}
		{[HhvV]} {set n 1}
		{[cC]} {set n 6}
		{[aA]} {set n 7}
		{[sSqQ]} {set n 4}
		{[z]} {set n 0}
		default {}
	    }

	    if {[string is double $curr]} {
		lappend res [concat $command [lrange $p $i [expr {$i + $n - 1}]]]
	    } else {
		set command $curr
		lappend res [lrange $p $i [expr {$i + $n}]]
		incr i 1
	    }
	    incr i $n
	}
	return $res
    }

    # pathToList --
    # 	transform a SVG path data into a nice Tcl list.
    # 	Conforms to SVG 1.0 specification (02 November 2000)

    proc pathToList {path} {
	# The parsing process is *not* simple, as the *#!&% !!! designers of 
	# svg systematically favored compacity over regularity.
	# We can't just separate elements. We have to RECOGNISE them (because such things as 
	# 3.4.4 are possible (and mean 3.4 0.4).
	#
	# real numbers : ([-+]?(?:\d*\.\d+|\d+\.?)(?:[eE][-+]?\d+)?)
	# command [aAtTQqSsCcVvHhLlZzMm]
	set real {[-+]?(?:\d*\.\d+|\d+\.?)(?:[eE][-+]?\d+)?}
	set keys  {[aAtTQqSsCcVvHhLlZzMm]}
	set command "($real|$keys)"
	set path [string map [list "," " "] $path ]
	regsub -all $command $path { \1 } path
	set path [string trim $path]
	regsub -all {\s+} $path { } path
	return [split $path]
    }

    # svgToInternal --
    # 	takes a svg path, and returns a Tcl list compatible with the sbzr library.
    #	Note that a given svg path may correspond to more than one sbzr contours.
    # 	" A path data segment must begin with either one of the "moveto"
    # 	  commands or one of the "arc" commands.  Subsequent "moveto" commands
    #	  (i.e., when the "moveto" is not the first command) represent the start of a new subpath"
    
    proc svgToInternal {path} {
	set res [list]
	foreach elt $path {
	    set comm [lindex $elt 0]
	    switch -- $comm {
		
	    }
	}
    }
}
