# Class representing a BZR font. See BZRFont.old for a Tcl only
# version.  As Tcl-only was a bit slow even with an AMD 900MH, whe
# chose to keep using a C library.
#
# 
# Note on BZR format :
#	1) size (Bdouble)
#	2) 
#
# Codes : see file bzrop.h in the src directory
#
# Note on Tcl Format for a given char :
# 	a list with : code charwidth llx lly urx ury contours*
#	a contour :
#	"CMP" sens firstPoint segments*
#	
#	spline segment list : s c1 c2 p2 
#	line segment list : l p2

package require Sbzr 2.0

class BZRFont {    
    
    proc BZRFont {this} {
	set ($this,fontfile) {None}
	#set ($this,font) [sbzr_newfont]
	setFontSize $this 14
	setFileName $this "unnamed.bzr"
	setName $this "BZR edited font"
	clearFont $this
    }

    proc ~BZRFont {this} {
    }

    proc load {this fileName} {
	clearFont $this
	set txt [sbzr_loadfont $fileName]
	setFontSize $this [lindex $txt 0]
	setName $this [lindex $txt 1]
	setFileName $this $fileName
	foreach ch [lrange $txt 3 end] {
	    setChar $this [lindex $ch 0] $ch
	}
    }

    proc saveAs {this fname} {
	set txt [list [getFontSize $this] [getName $this] {0 0 0 0}]
	for {set i 0} {$i < 256} {incr i} {
	    if {[hasChar $this $i]} {
		lappend txt [getChar $this $i]
	    }
	}
	sbzr_savefont $txt $fname
    }

    proc exportAsMF {this mfName} {
	# export in two stages :
	# a create a wrapper mf file
	set wrapper {
% This file defines the Metafont font FILENAME
true_design_size# := 14.000pt#;
input Hibzrstp;
font_coding_scheme := "Egyptian font FILENAME";
font_identifier := "FILENAME";
input FILENAME_font.mf;

def mamodif= identity  reflectedabout((w/2,h), (w/2,0)); enddef;
def depart= 128 -31; enddef;
input FILENAME_font.mf;
bye.
	}
	
	# compute actual file names
	set base [file rootname [file tail $mfName]]
	set wrapper [string map [list FILENAME $base] $wrapper]
	
	set f [open $mfName w]
	puts $f $wrapper
	close $f
	
	set auxFileName "[file rootname $mfName]_font.mf"
	set f [open $auxFileName w]
	# b create the real font file :
	# produce each sign.
	for {set i 0} {$i < 128} {incr i} {
	    if {[hasChar $this $i]} {
		set rep [getChar $this $i]
		set code [lindex $rep 0]
		set width [lindex $rep 1]
		set llx [lindex $rep 2]
		set lly [lindex $rep 3]
		set urx [lindex $rep 4]
		set ury [lindex $rep 5]

		puts $f "beginchar ($i, ${width}u#, [expr $ury - $lly]u#, 0.000u#);"
		puts $f "\"$base$i\";"
		# list the char control points
		set cId 0
		foreach c [lrange $rep 6 end] {
		    set pId 0
		    set wind [lindex $c 1]
		    set xs [lindex $c 2]
		    set ys [lindex $c 3]
		    puts $f "\tz$i\\$cId\\${pId}s = (${xs}u, ${ys}u);"
		    foreach s [lrange $c 4 end] {
			switch -exact -- [lindex $s 0] {
			    s {
				puts $f "\tz$i\\$cId\\${pId}c1 = ([lindex $s 1]u, [lindex $s 2]u);"
				puts $f "\tz$i\\$cId\\${pId}c2 = ([lindex $s 3]u, [lindex $s 4]u);"
				puts $f "\tz$i\\$cId\\${pId} = ([lindex $s 5]u, [lindex $s 6]u);"
			    }
			    l {
				puts $f "\tz$i\\$cId\\${pId} = ([lindex $s 1]u, [lindex $s 2]u);"
			    }
			}
			incr pId			
		    }
		    incr cId
		}
		# Build the char's contours
		set cId 0
		foreach c [lrange $rep 6 end] {
		    set pId 0
		    set wind [lindex $c 1]
		    puts $f "\tfill_or_unfill z$i\\$cId\\0s"
		    set lastSegmentId [expr [llength $c] - 5]
		    foreach s [lrange $c 4 end] {
			switch -exact -- [lindex $s 0] {
			    s {
				puts -nonewline $f "\t\t..controls z$i\\$cId\\${pId}c1 and "
				puts -nonewline $f "z$i\\$cId\\${pId}c2.."
			    }
			    l {
				puts -nonewline $f "\t\t--"
			    }
			}
			if {$pId == $lastSegmentId} {
			    puts $f "cycle;"
			} else {
			    puts $f "z$i\\$cId\\${pId}"
			}
			incr pId
		    }
		    incr cId
		}
		puts $f "endchar;"
	    }
	}
	close $f
    }
    # Public functions
    
    proc setFileName {this fileName} {
	set ($this,fileName) $fileName
    }

    proc getFileName {this} {
	return $($this,fileName)
    }

    proc setName {this name} {
	set ($this,name) $name
    }

    proc getName {this} {
	return $($this,name)
    }

    proc setFontSize {this size} {
	set ($this,size) $size
    }

    proc getFontSize {this} {
	return $($this,size)
    }

    proc setChar {this code txt} {
	set ($this,char,$code) $txt
	setHasChar $this $code 1
    }

    proc hasChar {this code} {
	return $($this,hasChar,$code)
    }

    proc setHasChar {this code has} {
	set ($this,hasChar,$code) $has
    }

    proc getChar {this code} {
	if {[hasChar $this $code]} {
	    set res $($this,char,$code)
	} else {
	    set res [list $code 0 0 0 0 0]
	}
	return $res
    }
    
    proc clearChar {this code} {
	set ($this,char,$code) [list]
	setHasChar $this $code 0
    }

    proc clearFont {this} {
	for {set i 0} {$i < 256} {incr i} {
	    clearChar $this $i
	}
    }
}
