# Defines toolbars for the font editor application
# A toolbar controls an editor application.


class CharEditToolBar {
    
    proc CharEditToolBar {this parent name args} SFrame {$parent $name $args} {
	# The parent is the Application Frame.
	# Describe and define the state buttons.
	# fields : button text, button help, state controled by the button.

	set ($this,stateButtons) {
	    "Edit" "Edit, move contours, etc." edit
	    "Add"  "Create new contours" add
	    "Delete" "Delete stuff" delete
	    "Add Guide" "Add guide lines" addguide
	    "spline <-> Line" "transform splines into lines and vz." splinetoline
	    "select" "select" select
	    "lower" "lowers a contour" lower
	    "split" "adds a control point on a curve" split
	    "reverse" "transforms a positive contour into a negative one and vz." reverse
	}
	
	
	virtual proc create {this} {
	    set w [Component::getPath $this]
	    Component::setPath $this [SMainFrame::addToolBar [Component::getParent $this]]
	    
	    # select black/white contour.
	    set bbox1 [Component::buildNewComponent SFrame $this -auto]
	    
	    set black [Component::buildNewComponent SRadioButton -indicatoron 0 \
		    -background black -width 1 -selectcolor black -value "P" \
		    -borderwidth 5 -variable CharEditToolBar::($this,fill)\
		    -command [list CharEditToolBar::setBlack $this]

	    Component::registerHelp $black "Created contours will be filled in black" 
	    
	    set white [Component::buildNewComponent SRadioButton -indicatoron 0 \
		    -background white -width 1 -selectcolor white -value "N"\
		    -borderwidth 5 -variable CharEditToolBar::VAR($w,fill) \
		    -command [list CharEditToolBar::setWhite $w]
	    
	    Component::registerHelp $white "Created contours will be filled in white" 

	    Component::pack [list $black $white] -side left 
	    
	    setEditor $this 0
	    SMainFrame::showtoolbar [Component::getParent $this] 0 1
	    Component::pack [list $bbox1] -side left
	    addSeparator $w 0 0
	    
	    # select operation.
	    # will be linked automagically to the variables.
	    set bbox2 [frame $tb.bbox2]
	    set i 0
	    foreach {btext bhelp bstate} $stateButtons {
		set mypath $bbox2.button$i
		incr i
		
		setStateButtonPath $w $bstate $mypath
		radiobutton $mypath -indicatoron 0  -value $bstate -text $btext 
		DynamicHelp::register $mypath balloon  $bhelp
		pack $mypath -side left 
	    }
	    pack $bbox2 -side left
	    addSeparator $w 0 1
	    
	    set tb1 [$w addtoolbar]
	    set scaler [scale $tb1.scale -from -10  -to 10 \
		    -tickinterval 5.0 -resolution 0.1 -length 200\
		    -command [list CharEditToolBar::scaleEditor $w] -orient horizontal]
	    pack $scaler -side left
	    setScaler $w $scaler
	    addSeparator $this 1 1
	    $w showtoolbar 1 1
	}
	
    
	proc setScaler {this s} {
	    set ($this,scaler) $s
	}
	
	proc getScaler {this} {
	    return ($this,scaler)
	}
	
	proc addSeparator {this tbnum i} {
	    pack [Separator [$w gettoolbar $tbnum].sep$i -orient vertical] -side left -fill y -padx 4
	}
	
	proc setEditor {this e} {
	    set ($this,editor) $e
	    bindStates $this
	}
	
	proc setStateButtonPath {this state path} {
	    set ($w,statepath,$state) $path
	}
	
	proc getStateButtonPath {this state} {
	    return $($this,statepath,$state)
	}
	
	proc bindStates {this} {
	    set editor [getEditor $this]
	    # Links the states of this with an actual editor
	    
	    if {$editor != -1} {
		set var CharEdit::($editor,state)
		
		foreach {btext bhelp bstate} $stateButtons {
		    set p [getStateButtonPath $w $bstate]
		    $p configure -variable $var
		}
		set s [CharEdit::getScale $editor]
		[getScaler $w] set $s
		set VAR($w,fill) [CharEdit::getFill $editor]
	    }
	}
	
	
	# Should probably move to apply
	proc getEditor {this} {
	    return $($this,editor)
	}
	
	proc hasEditor {this} {
	    if {[getEditor $w]  != -1} {
		return 1
	    } else {
		return 0
	    }
	    
	}
	
	# Auxiliary actions for functions.
	# values will be retrieved and set through setEditor and getEditor.
	
	proc setWhite {w} {
	    if {[hasEditor $w]} {
		CharEdit::setUnFilled [getEditor $w]
	    }
	}
	
	proc setBlack {w} {
	    if {[hasEditor $w]} {
		CharEdit::setFilled [getEditor $w]
	    }
	}
	
	proc scaleEditor {w scale} {
	    if {[hasEditor $w]} {
		CharEdit::scaleall [getEditor $w] $scale
	    }
	}
    }
}
    