#
# widget to display and select BZR font chars.
#

package require seshw

class FontDisplay {

    proc FontDisplay {this parent appli name args} SScrolledWindow {$parent $name $args} {
	
	set ($this,appli) $appli
	set ($this,modified) 0
	set ($this,drawingSystem) 0

	# A note on ScrollableFrame : ScrollableFrame *doesn't* have scrollbars !
	# it's meant to be used with ScrolledWindow
	
	set frame [[Component::buildNewComponent SScrollableFrame $this -auto]
	SScrolledWindow::setWidget $this $frame

	set win [SScrollableFrame::getFrame $frame]
	
	setChanged $this 0
	
	for {set i 0} {$i < 256} {incr i} {
	    set lab $win.charNum$i
	    label $lab -text $i
	    button [getButton $w $i]
	    set img [image create bitmap [getImageName $w $i]]
	    [getButton $w $i] configure  -width 40 -height 40 -image [getImageName $w $i]
 	    grid $lab [getButton $w $i] 
	    bind [getButton $w $i] <Double-Button-1> {FontDisplay::openChar %W}
	    setButtonWindow [getButton $w $i] $w
	    setButtonCharNum [getButton $w $i] $i
	}
	
	# We used to pack buttons one by one. It's soooooo expensive.
	# It's much faster to pack all buttons in one instruction :
	# eval pack [listButtons $w]
	# well, grid is fast enough, so we use grid. But we keep the previous instruction
	# as a useful reminder.
	
	return $w
    }

    proc setButtonWindow {b w} {
	set (button,$b,window) $w
    }

    proc getButtonWindow {b} {
	return $(button,$b,window)
    }

    proc setButtonCharNum {b i} {
	set (button,$b,num) $i
    }

    proc getButtonCharNum {b} {
	return $(button,$b,num)
    }
    
    proc deleteFontDisplay {this} {
	puts "goodBye $w"
	# Free ressources :
	array unset ($this,*)
	# free the ressources associated with buttons :
	# TO DO
    }

    proc openChar {b} {
	set w [getButtonWindow $b]
	set app [getAppli [getButtonWindow $b]]
	set font [getFont [getButtonWindow $b]]
	set num [getButtonCharNum $b]
	set txt [BZRFont::getChar $font $num]
	# The following line creates a dependency between FontDisplay and the CharEditAppli.
	# This should not be. A simple solution would be to use a callBack
	CharEditAppli::editFontChar $app $txt [list FontDisplay::setChar $w $num] "[BZRFont::getFileName $font]($num)" $w
    }

    proc setChar {this num txt} {
	BZRFont::setChar [getFont $w] $num $txt
	updateChar $w $num
	setChanged $w 1
    }

    proc getAppli {this} {
	return $($this,appli)
    }

    # For use with pack !
    proc listButtons {this} {
	set r [list]
	for {set i 0} {$i < 256} {incr i} {
	    lappend r [getButton $w $i]
	}
	return $r
    }

    proc getButton {this i} {
	#return [$w.fr getframe].char$i.b
	return [$w.fr getframe].b$i
    }

    proc getDrawingSystem {} {
	if {[string compare $($this,drawingSystem) "0"] == 0} {
	    seshw::create_passive .dummy -htline 40
	}
	set ($this,drawingSystem) .dummy
	return $($this,drawingSystem)
    }
    
    proc updateChar {this i} {
	set f [getFont $w]
	if {[BZRFont::hasChar $f $i]} {
	    set txt [BZRFont::getChar $f $i]
	    setDisplayChar $w $i $txt
	}
    }
    
    proc setDisplayChar {this num txt} {
	set d [getDrawingSystem]
	tksesh_defchar temp $txt
	seshw::set_text $d temp
	seshw::autosize $d
	set img [image create bitmap [getImageName $w $num] -data [$d toxbm 1.0 end]]
	[getButton $w $num] configure -image [getImageName $w $num]
    }

    proc getImageName {this num} {
	return "fonticon $w $num"
    }

    proc setFont {this f} {
	set ($this,font) $f
	for {set i 0} {$i < 256} {incr i} {
	    updateChar $w $i
	}
	setChanged $w 0
    }
    
    proc getFont {this} {
	return $($this,font)
    }

    proc getChanged {this} {
	return $($this,changed)
    }

    proc setChanged {this v} {
	set ($this,changed) $v
    }
    
}
