# Clean and nicer version of the sign editor

# * Using the nice BWidgets
# * interfaced with the database
# * multiple windows
# * undo/redo facilities

# Export facilities for bzrfonts
# dynamic cut and paste.

# BWidget Implementation

# Note on font edition :
# Load Font : create a new font window in the navigator.
# New Font 
# Close Current Font
# 
# Interaction between the font windows and the char windows :
# Double-click on a char in the font windows : create an editor for this
# 	char. Font name and char number appear in the title. 
#
#
# drag and drop ????
#
#

package require seshuser
seshuser::loaduser

package require BWidget 1.3

#TitleFrame .tf

class CharEditAppli {

    proc CharEditAppli {this parent name args} SMainFrame {$parent $name $args} {
	set ($this,numEdit) 0
	set ($this,numFont) 0
	setEditor $this -1 -1
    }

    proc ~CharEditAppli {this} {
    }

    
    virtual proc creatorHook {this} {


	# Menu is a read-only option. We must fix it
	# before widget creation
	set descmenu [buildMenu $this]
	Component::addArgs $this -menu $descmenu

	# Call parent procedure to create the widget
	SMainFrame::_creatorHook $this


	Component::configure $this -textvariable CharEditAppli::($this,status)
	
	SMainFrame::addIndicator $this -text "The TKsesh Font Editor"
	SMainFrame::addIndicator $this -text "Version [getVersion $this] ([getDate $this])"

	set ($this,status) ready

	set paned [Component::buildNewComponent SPanedWindow [SMainFrame::getFrame $this] -auto -side bottom]
	
	set p1 [SPanedWindow::add $paned -minsize 300]
	set p2 [SPanedWindow::add $paned -minsize 100]

	set ($this,container) $paned
	set ($this,chars) $p1
	set ($this,directories) $p2
	
	#set ($this,toolbar) [new CharEditToolBar $this -auto]
	
	# charBook and noteBook creation
	set ($this,charBook) [Component::buildNewComponent SNoteBook $p1 -auto -width 300 -height 300]
	set ($this,fontBook) [Component::buildNewComponent SNoteBook $p2 -auto -width 100 -height 300]
	
	#createSignWindow $w
	
	#$win.book compute_size
	
	Component::pack $paned -fill both -expand yes
	Component::pack [getCharBook $this] -fill both -expand yes
	Component::pack [getFontBook $this] -fill both -expand yes
    }

    proc getVersion {this} {return 2}

    proc getDate {this} {return {Mon Dec 24 17:13:59 CET 2001}}
    
    proc buildMenu {this} {
	set res {
	    "&File" all file 0 {
		{
		    command "&New Font" {} "Create a new BZR Font" {}
		    -command {CharEditAppli::newFont WINDOW}
		}
		{
		    command "&Load Font" {} "Load a font from a bzr file" {}
		    -command {CharEditAppli::loadFont WINDOW}
		}
		{
		    command "&Save Font" {} "save a font in a bzr file" {}
		    -command {CharEditAppli::saveFont WINDOW}
		}
		{
		    command "&Save Font As" {} "save a font in a bzr file" {}
		    -command {CharEditAppli::saveFontAs WINDOW}
		}
		{
		    command "&Export Font to MF" {} "save a font in a metafont file for use with HieroTeX" {}
		    -command {CharEditAppli::exportFontMF WINDOW}
		}
		{
		    command "Delete Font window" {} "Delete current font window" {}
		    -command {CharEditAppli::deleteUpperFontWindow WINDOW}
		}
		separator
		{
		    command "&New sign window" {} "Open a new sign window" {}
		    -command {CharEditAppli::createSignWindow WINDOW}
		}
		{
		    command "&Delete sign window" {} "Delete current sign window" {}
		    -command {CharEditAppli::deleteUpperSignWindow WINDOW}
		}
		{
		    command "&Validate sign window" {} "Save the current sign where it belongs" {}
		    -command {CharEditAppli::validateUpperSign WINDOW}
		}
		separator
		{
		    command "Load Image" {} "Load Background image" {} -command {CharEditAppli::loadImage WINDOW}
		}
		{
		    command "Clear Image" {} "Clear Background image" {} -command {CharEditAppli::clearImage WINDOW}
		}
		separator
		{
		    command "E&xit" {} "Exit BWidget demo" {} -command exit
		}
	    }

	    "&Edit" all edit 0 {
		{
		    command "&Undo" {} "Undo last command" {}
		    -command {CharEditAppli::undo WINDOW}
		}
		separator
		{
		    command "Cut" {} "Cut selected font part" {}
		    -command {CharEditAppli::cut WINDOW}
		}
		{
		    command "Copy" {} "Copy selected font part " {}
		    -command {CharEditAppli::copy WINDOW}
		}
		{
		    command "Paste" {} "Paste  selected font part" {}
		    -command {CharEditAppli::paste WINDOW}
		}
	    }
	    "&Options" all options 0 {
	    }
	}
	regsub -all -- WINDOW $res $this res
	return $res
    }
    
    proc getCharBook {this} {
	return $($this,charBook)
    }
    
    proc getFontBook {this} {
	return $($this,fontBook)
    }
    
    proc createSignWindow {this} {
	incr ($this,numEdit)
	set i $($this,numEdit)
	
	
	# SOLUTION : NOUVELLE CLASSE !!!!
  	set page [SNoteBook::insert [getCharBook $this] end]
	Component::configure $page -text "Editor $i"
	Component::configure $page -raisecmd [list CharEditAppli::selectEditor $this $page]
	
	# as the canvas hasn't got any scaling facilities,
	# using the canvas scroll commands would require us to use *THREE* coordinate systems.
	# So, we do it by hand.
	# ScrolledWindow $page.scroll -auto both -scrollbar both

	frame $page.scroll
	#set editorWidget  [new CharEdit $page.scroll -xscrollcommand "exec echo $page.scroll.xs set" \
	#	-yscrollcommand "exec echo $page.scroll.ys set"]

	set editorWidget  [new CharEdit $page.scroll]
	

	set e $thisidget::($editorWidget,path)

	scrollbar $page.scroll.xs -command "CharEdit::xview $editorWidget" -orient horizontal
	scrollbar $page.scroll.ys -command "CharEdit::yview $editorWidget" -orient vertical

	CharEdit::setscale $editorWidget 10 20 300
	CharEdit::repere $editorWidget
	CharEdit::setXScrollBar $editorWidget $page.scroll.xs
	CharEdit::setYScrollBar $editorWidget $page.scroll.ys
	CharEdit::notifyScrollBars $editorWidget

	grid $e $page.scroll.ys -sticky news
	grid $page.scroll.xs - -sticky ew
	grid columnconfigure $page.scroll 0 -weight 1
	grid rowconfigure $page.scroll 0 -weight 1

	#$page.scroll setwidget $e

	setEditor $this $editorID $editorWidget

	#pack $e -fill both -expand yes
	pack $page.scroll -fill both -expand yes
	[getCharBook $this] raise $editorID
	return $editorID
    }
    
    proc selectEditor {this i} {
	CharEditToolBar::setEditor $this [getEditorObject $this $i]
    }

    # getEditorObject --
    # 	returns the editor Object for a given id.
    #
    
    proc getEditorObject {this i} {
	return $($this,editor,$i)
    }

    # setEditor --
    # 	register the editor Object for a given id.
    #
    
    proc setEditor {this i e} {
	set ($this,editor,$i) $e
    }

    # undo --
    # 	undo the last command in the current font editor
    #
    proc undo {this} {	
	set e [getEditorObject $this [getCurrentEditorPage $this]]
	CharEdit::undo $e
    }

    proc getCurrentEditor {this} {
	set e [getEditorObject $this [getCurrentEditorPage $this]]
	return $e
    }

    
    proc getCurrentEditorPage {this} {
	set win [getCharBook $this]
	return [$thisin raise]
    }

    proc getCurrentFontPage {this} {
	set win [getFontBook $this]
	return [$thisin raise]
    }

    proc getCurrentFontWindow {this} {
	return [getFontWindow $this [getCurrentFontPage $this]]
    }

    proc deleteUpperFontWindow {this} {
	set win [getFontBook $this]
	# catch to protect us if there is no page to destroy.
	#catch {}
	    set page [getCurrentFontPage $this]
	    set fwin [getCurrentFontWindow $this]
	    set allClear 1
	    foreach s [getCharEditsForFont $this $fwin] {
		set allClear [expr $allClear && [deleteSignWindow $this $s]]
	    }
	    if {! $allClear} {
		MessageDlg .couldNotDo -icon error -type ok \
			-message "There are still edited signs in this font.\n I could not delete the font window."
		return
	    }
	    if {[FontDisplay::getChanged $fwin]} {
		set answer [MessageDlg .validateDestroy -icon warning -type okcancel \
			-message "This font hasn't been saved.\nYou will lose your edits.\nProceed ?"]
		if {$answer == 1} {
		    return
		}
	    }
	    $thisin delete $page
	    # Should probably be elsewhere :
	    set l [$thisin pages]
	    if {[llength $l] > 0} {
		$thisin raise [lindex $l 0]
	    }
	#{}
    }

    
    proc deleteUpperSignWindow {this} {
	set result 0
	set win [getCharBook $this]
	# catch to protect us if there is no page to destroy.
	catch {
	    set page [getCurrentEditorPage $this]
	    set edit [getCurrentEditor $this]
	    set result [deleteSignWindow $this $page]
	}
	return $result
    }

    # CharEditAppli::deleteSignWindow --
    # 	deletes a sign displaying window
    # Parameters:
    #	w : the chareditappli the window belongs to
    #	p : the page to delete
    # Options:
    # Results:
    # 	1 if the window was deleted else 0 
    # Side Effects:
    #	the page is deleted
    # Use:
    #	PUBLIC
    # Called by:
    
    proc deleteSignWindow {this p} {
	set charbook [getCharBook $this]
	set edit [getEditorObject $this $p]
	if {[CharEdit::getChanged $edit]} {
	    set label [[getCharBook $this] itemcget $p -text]
	    set answer [MessageDlg .validateDestroy -icon warning -type okcancel \
		    -message "char $label hasn't been saved.\nYou will lose your edits.\nProceed ?"]
	    if {$answer == 1} {
		return 0
	    }
	}
	unRegisterCharEditForFont $this $p
	$charbook delete $p
	set l [$charbook pages]
	if {[llength $l] > 0} {
	    $charbook raise [lindex $l 0]
	}
	return 1
    }

    proc validateUpperSign {this} {
	# catch to protect us if there is no page
	catch {
	    set page [getCurrentEditorPage $this]
	    set edit [getCurrentEditor $this]
	    set txt [CharEdit::asText $edit]
	    CharEdit::setChanged $edit 0
	    eval [getCharWValidator $this $page] [list $txt]
	}
    }
    
    # loadFont --
    # 	Loads a BZR font
    #	Create a corresponding font window.

    proc loadFont {this} {
	set patterns {{{BZR file} {.bzr}} {Any *}}
	set name [tk_getOpenFile \
		-defaultextension .bzr -filetypes $patterns]
	
	set b [new BZRFont]
	BZRFont::load $b $name
	
	addFontWindow $this $b
	# temporary :
	#  	createSignWindow $this
	#  	set e [getCurrentEditor $this]
	#  	set txt [BZRFont::getChar $b 32]
	#  	CharEdit::setText $e $txt
	# SUPRESS THE NEXT LINE : 
	# set VAR($this,font) $b
    }

    proc newFont {this} {
	set b [new BZRFont]
	addFontWindow $this $b
    }

    # saveFont --
    # 	save a BZR font

    proc saveFont {this} {
	set fwin [getCurrentFontWindow $this]
	set font [FontDisplay::getFont $fwin]
	BZRFont::saveAs $font [BZRFont::getFileName $font]
    }

    proc saveFontAs {this} {
	set patterns {{{BZR file} {.bzr}} {Any *}}
	set name [tk_getSaveFile \
		-defaultextension .bzr -filetypes $patterns]
	set fwin [getCurrentFontWindow $this]
	set font [FontDisplay::getFont $fwin]
	BZRFont::setFileName $font $name
	BZRFont::saveAs $font $name
	[getFontBook $this] itemconfigure [getCurrentFontPage $this] -text $name
    }

    proc exportFontMF {this} {
	set patterns {{{Metafont file} {.mf}} {Any *}}
	set name [tk_getSaveFile \
		-defaultextension .bzr -filetypes $patterns]
	set fwin [getCurrentFontWindow $this]
	set font [FontDisplay::getFont $fwin]
	BZRFont::exportAsMF $font $name
    }

    # displayChar --
    # 	displays the num th char from font in the uppermost window.
    #proc displayChar {this font num} {}

    # setCharFromEditor --
    #	sets the numth char in font from the editor window ed
    #proc setCharFromEditor {this font num edi} {}

    # clearCharEdit --
    #	clears a char editor.
    #	if the char has been edited and not saved,
    #	display a warning message.
    #
    #	returns:	1 if we should proceed, 0 if this operation was cancelled.

    proc clearCharEdit {this ced} {
	if {[CharEdit::getChanged $ced]} {
	    MessageDialog .reallyClear -title "Really clear?" -icon question -type okcancel\
		    -message "Warning : char has been modified, but not saved.\n Really clear it ?"
	    set res [Message::create .reallyClear]
	    if {$res != 0} {
		return 0
	    }
	}
	CharEdit::clear $ced
	CharEdit::setChanged 0
	return 1
    }

    # addFontWindow --
    # 	creates a new font window.
    #	w the base window application
    #	b the font
    #
    #	returns : an id for the the window.
    
    proc addFontWindow {this b} {
	incr ($this,numFont)
	set pageID "font$($this,numFont)"
	
	set page [SNoteBook::insert [getFontBook $this] end]

	SNoteBookPage::itemConfigure $page -raisecmd [list CharEditAppli::selectFont $this $page]

	SNoteBookPage::itemConfigure $page -text [BZRFont::getFileName $b]

	set fd [FontDisplay::create [getFontWindow $this $pageID] $this]
	FontDisplay::setFont $fd $b
	Component::pack $fd -fill both -expand yes
	
	NoteBookPage::raise $page
	initCharEditsForFont $this $fd
	return $page
    }

    # getFontWindow --
    #	returns a path for the given font window
    proc getFontWindow {this id} {
	set book [getFontBook $this]
	set path [$book getframe $id].frame
    }
    
    #
    proc selectFont {this id} {
    }
    
    # editChar --
    #	open an editing window for a char
    #	
    #	w : the application window
    #	txt : the char described in the usual font format (see CharEdit.tcl)
    #	validator : a function call which shall be eval-uated when the char is validated.
    #			it should either save the char in its font (for bzrfont chars)
    #			or save the char in the database.
    #			The last argument of the validator will be the text.
    #	label : label to use for the editchar window
    # Results:
    #	sends back the charEdit object which was created.
    
    
    proc editChar {this txt validator label} {
	set i [createSignWindow $this]
	# Set the name for the sign window.
	[getCharBook $this] itemconfigure $i -text $label
	set e [getEditorObject $this $i]
	CharEdit::setText $e $txt
	setCharWValidator $this $i $validator
	return $i
    }


   # truc::fonction --
    # 	description...
    # Parameters:
    #	f: the font window
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    
    proc editFontChar {this txt validator label f} {
	set i [editChar $this $txt $validator $label]
	registerCharEditForFont $this $f $i
    }

    # registerCharEditForFont --
    # 	remembers that a given char editor correspond to a given FontDisplay
    # Parameters:
    #	w :
    #	f : the font editor window
    #	e : the char editor *page*
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:

    proc registerCharEditForFont {this f e} {
	set ($this,isEditFor,$e) $f
	lappend ($this,fontwin,$f,charEdits) $e
    }

    # unRegisterCharEditForFont --
    #	un-register a charEdit. To be used when closing the said char editor.
    
    proc unRegisterCharEditForFont {this e} {
	set f [getFontForChar $this $e]
	unset ($this,isEditFor,$e)
	listreplace ($this,fontwin,$f,charEdits) $e
    }
    
    proc getFontForChar {this e} {
	return $($this,isEditFor,$e)
    }

    # truc::fonction --
    # 	description...
    # Parameters:
    #	w :
    #	f : the font window
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    
    proc initCharEditsForFont {this f} {
	set ($this,fontwin,$f,charEdits) [list]
    }

    # truc::fonction --
    # 	description...
    # Parameters:
    #	w :
    #	f : the font *window*
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:

    proc getCharEditsForFont {this f} {
	return $($this,fontwin,$f,charEdits)
    }

    proc setCharWValidator {this i v} {
	set ($this,$i,validator) $v
    }
    
    proc getCharWValidator {this i} {
	return $($this,$i,validator)
    }

    proc copy {this} {
	setCharClipboard $this [CharEdit::getSelection [getCurrentEditor $this]]
    }

    proc paste {this} {	
	set txt [getCharClipboard $this]
	CharEdit::pasteAction [getCurrentEditor $this] $txt
    }

    proc cut {this} {

    }

    proc loadImage {this} {
	set patterns {{{gif file} {.gif}} {Any *}}
	set name [tk_getOpenFile \
		      -defaultextension .gif -filetypes $patterns]
	set e [getCurrentEditor $this]
	CharEdit::loadimg $e $name
    }

    proc clearImage {this} {
	set e [getCurrentEditor $this]
	CharEdit::delimg $e
    }

    proc setCharClipboard {this txt} {
	set ($this,clipboard) $txt
    }

    proc getCharClipboard {this} {
	return $($this,clipboard)
    }
}

set top [new STopLevel ""]
set appli [Component::buildNewComponent CharEditAppli $top -auto]
pack [Component::getPath $appli] -fill both -expand yes
