# dicointer --
#
#       
#       Interface to the dictionary entry
#
# 
#

package require seshw
package require dico
package require dbbrowse
package require yellow
package require seshnormalize

package provide dicoload 0.0	;# This uses Tcl namespaces

namespace eval dicoload {
    namespace export create
    namespace export fromsesh
    namespace export open_dictionary

    variable WINDOWS 0
    variable LOC
    
    array set LOC ""
    proc create {w} {
	variable LOC
	frame $w
	set LOC($w,trans) ""
	set LOC($w,trad) ""
	label $w.ltrans -text [legendes::trans "Transliteration"]
	entry $w.trans -textvariable dicoload::LOC($w,trans) -font {Transliteration 18}
	label $w.ltrad -text [legendes::trans "translation"]
	entry $w.trad -textvariable dicoload::LOC($w,trad)
	label $w.lhie -text [legendes::trans "Hieroglyphs"]
	seshw::create_editor $w.s -notitle
	set dicoload::LOC($w,exact) 1
	checkbutton $w.exact -variable dicoload::LOC($w,exact)
	yellow::create $w.exact [legendes::trans "NOT IMPLEMENTED YET !\nIt should allow you to search for partial matches\nin the dictionary"]
	button $w.clear -text [legendes::trans "Clear"] -command [list dicoload::clear $w]
	button $w.go -text [legendes::trans "Find"] -command [list dicoload::create_windows $w]
	button $w.new -text [legendes::trans "New"] -command [list dicoload::create_windows $w 1]
	
	label $w.lexact -text [legendes::trans "Exact search"]

	dbbrowse::create $w.list \
		-font {Transliteration 18}\
		-unnorm seshnormalize::unnorm_translit\
		-norm seshnormalize::norm_translit
		
	dbbrowse::setload $w.list dicoload::from_list
	dbbrowse::setfirst $w.list A


	grid $w.ltrans - $w.trans -              -sticky ew 
	grid $w.lhie - $w.s     -               -sticky ew 
	grid $w.ltrad - $w.trad   -               -sticky ew 
	grid $w.lexact $w.exact - -                -sticky e 
	grid $w.clear - $w.go - $w.new - -sticky ew
	grid $w.list - - - - -sticky news
	grid columnconfigure $w 2 -weight 1
	grid rowconfigure $w 5 -weight 1
    }

    proc clear {w} {
	variable LOC
	set  LOC($w,trad) ""
	set LOC($w,trans) ""
	::seshw::clear_text $w.s.s
	set dicoload::LOC($w,exact) 1
    }

    proc create_one_window {} {
	variable WINDOWS
	incr WINDOWS
	toplevel .dico$WINDOWS
	dico::create .dico$WINDOWS.w
	bind .dico$WINDOWS.w <Destroy> [list destroy .dico$WINDOWS]
	grid .dico$WINDOWS.w -sticky news
	grid rowconfigure  .dico$WINDOWS 0 -weight 1
	grid columnconfigure .dico$WINDOWS 0 -weight 1
	return .dico$WINDOWS.w
    }

    proc create_windows {w {new 0}} {
	variable LOC

	set idlist {}

	if {! $new} {
	    if { $LOC($w,exact) } {

		if { $LOC($w,trans) != "" } {
		    set idlist [concat $idlist [dico::find_index MTRI $dicoload::LOC($w,trans)]]
		}

		if { $LOC($w,trad) != "" } {
		    set idlist [concat $idlist [dico::find_index TRAD $dicoload::LOC($w,trad)]]
		}

		if { [$w.s.s get 0.0 end ] != "" } {
		    set idlist [concat $idlist [dico::find_index WRI [$w.s.s get 0.0 end ]]]
		}
	    } else {
		return
	    }
	}
	
	if { $idlist != {} } {
	    set idlist [uniq $idlist]
	    foreach i $idlist {
		set my [create_one_window]
		dico::load $my $i
	    }
	} else {
	    set data [list\
			 [list MTRI $dicoload::LOC($w,trans)]\
		[list WRI [$w.s.s get 0.0 end ]]\
				  {GRA {}}]
	    if { $LOC($w,trad) != "" } {
		lappend data [list TRAD $LOC($w,trad)]
	    }
	    set my [create_one_window]
	    dico::fromtext $my \
		[list \
		     GRP $data]
	}
	
    }
    
    proc open_dictionary {} {
	#catch {
	#}
	if {! [winfo exists .dico]} {
	    toplevel .dico
	    dicoload::create .dico.g
	    grid .dico.g -sticky news
	    grid rowconfigure .dico 0 -weight 1
	    grid columnconfigure .dico 0 -weight 1
	} else {
	    raise .dico
	}
    }

    proc fromsesh {w} {
	set current_ref [seshw::getref $w]
	open_dictionary
	
	clipboard clear 
	clipboard append $current_ref

	set hie [seshw::get_selection $w]
	set tr [translit [seshw::get_selection $w -aslist]]
	set dicoload::LOC(.dico.g,trans) $tr
	seshw::set_text [seshw::gethwindow .dico.g.s] $hie
	focus .dico.g
    }
    
    proc from_list {w} {
	set v [dbbrowse::getval $w]
	foreach id $v {
	    set dw [create_one_window]
	    dico::load $dw $id
	}
    }
}

