#
# Essais d'un diteur de fonte
#
# Needs quite a lot of cleaning. But the ideas are sound.
#
# The principles is that coordinates are given by tk canvas,
# so we can use it for all moves (scaling, rotating).
# 
# When an element is changed by the program we call update
# 	Currently, it occurs in proc double {}, because we change one of the lines,
# 	and when a control point is moved, to change the controled items.

#
# GraphElt : graphical element on a fontedit
# 	window = the font editor
#       create_drawing : creates one or more canvas items
#		and send them back
#	get_drawing : sends back the canvas items number
#	update : change the canvas representation according to new specs.
#	draw : draw the element. Is it of any interest now ????
#	move x y : moves the element by x y
#	astext : the element as a text
#	getcolor : the element color
#	setred 
#	unsetred

#
# Contour
#
#
# ....
# 
# The next version should be a little cleaner.
# Perhaps we could introduce : atomGraphicElement and Compound GraphicElement
# Another nice move would be to use delegation. That is, there would be
# a line GraphicElement, and all lines would use This element for being drawn
# 

# Nous passons en mode orient objet !!!

#source /stooop-2.3/stooop.tcl
#source /home/serge/TR/TCL/scwoop-1.3/scwoop.tcl

package require stooop 3.7
namespace import stooop::*
package require scwoop
package require Sbzr
package require Tksesh
set env(SESHLIB) /home/rosmord/tksesh/tcllib


set tcl_precision 17
set i [new canvas .]
delete $i

# A number of utility functions.

# replaces element elt in list l by the elements in rep.
# if rep is not given, erase elt in l.

proc listreplace {l elt {rep {}}} {
    upvar $l list
    set i [lsearch -exact $list $elt]
    if { [string compare $rep {}] == 0} {
	set list [lreplace $list $i $i]	
    } else {
	set list [eval {lreplace $list $i $i} $rep]
    }
}

proc middle {a b} {
    return [expr {($a + $b) / 2.0}]
}

# Recursive drawing proc for a spline.

proc makecurve {x1 y1 x2 y2 x3 y3 x4 y4 prof} {
    if {$prof == 0} {
	return [list $x1 $y1 $x2 $y2 $x3 $y3 $x4 $y4]
    } else {
	set prof [expr $prof - 1]
	set p1x [middle $x1 $x2]
	set p1y [middle $y1  $y2]
	set p2x [middle $x2  $x3]
	set p2y [middle $y2  $y3]
	set p3x [middle $x3  $x4]
	set p3y [middle $y3  $y4]
	set m1x [middle $p1x $p2x]
	set m1y [middle $p1y $p2y]
	set m2x [middle $p3x $p2x]
	set m2y [middle $p3y $p2y]
	set m3x [middle $m1x $m2x]
	set m3y [middle $m1y $m2y]
	set l1 [makecurve $x1 $y1 $p1x $p1y $m1x $m1y $m3x $m3y $prof]
	set l2 [makecurve $m3x $m3y $m2x $m2y $p3x $p3y $x4 $y4 $prof]
	return [concat $l1 $l2]
    }
}

proc norm {p1 p2} {
    set dx [expr [point::x $p2] - [point::x $p1]]
    set dy [expr [point::y $p2] - [point::y $p1]]
    return [expr {sqrt($dx*$dx+$dy*$dy)}]
}

# Any graphic element on a fonteditor

class graphelt {
    proc graphelt {this window} {
	# Window est la fentre-fontedit comme objet (son numro)
	set ($this,window) $window
	# Le path associ  ladite fentre
	set ($this,canvas) $widget::($window,path)
    }

    proc ~graphelt {this} {
	if {[info exists graphelt::($this,drawing)] && $graphelt::($this,drawing) != "None" } {
	    foreach i $graphelt::($this,drawing)  {
		$graphelt::($this,canvas) delete $i
	    }
	}
    }

    virtual proc create_drawing {this};	#  creates the drawing form on the canvas.
    # Sends back the list of id of created data
    # the first one is supposed to be the most important
    
    proc get_drawing {this} {
	return [lindex $graphelt::($this,drawing) 0]
    }


    # Update is called when the data for an element has changed.
    # Its task is to change the graphical representation
    
    virtual proc update {this} {
	foreach i $graphelt::($this,drawing) {
	    eval $graphelt::($this,canvas) coords $i [graphelt::expandpoints $this]
	}
    }
    
    virtual proc expandpoints {this};

    virtual proc draw {this} {
	# set can $graphelt::($this,canvas)
     	set graphelt::($this,drawing) [create_drawing $this]
	# This graphElt is included in itself.
	tagAsMember $this $this
	}
    }

    # Tag all members of the current graphelt as members of the $group tagelt.
    proc tagAsMember {this group} {
	set gTag [alltagname $group]
	foreach i $graphelt::($this,drawing) {
	    $graphelt::($this,canvas) addtag  $gTag withtag $i
	}
    }

    # Tag name used for all graphical elements linked to
    # the contour.
    proc alltagname {this} {
	return "atag$this"
    }

    virtual proc move {this x y} {}

    virtual proc astext {this};		# sends back a textual representation for the element.

    virtual proc getcolor {this} {
	return black
    }

    virtual proc setred {this} {
	$graphelt::($this,canvas) itemconfig [get_drawing $this] -fill red
    }

    virtual proc unsetred {this} {
	#set bg $fontedit::($graphelt::($this,window),color)
	set bg [getcolor $this]
	$graphelt::($this,canvas) itemconfig [get_drawing $this] -fill $bg
    }
}

#
# Objet contour :
#
# Un contour est une liste de points de controle et de segments
# 
#
# Architecture :
#
# Un contour comprend des points et des segments. Sa 
# destruction entraine la destruction de tous ceux-ci.
#
# ATTENTION : ce sont les segments qui dtruisent les points, pas les contours.
#
# Points et segments sont lis. On doit ATTACHER les points aux segments, et les librer 
# Aprs coup.

class contour {

    proc contour {this window {sens T}} graphelt {$window} {
	set ($this,type) CMP
	set ($this,sens) $sens
	set ($this,selected) 0
	if  {$sens != "N"} {
	    set ($this,basecolor) black
	} else {
	    set ($this,basecolor) darkgray
	}
	set ($this,color) $contour::($this,basecolor)
	set ($this,segments) {}
	lappend fontedit::($window,contours) $this
    }

    #proc getwindow {this} 
    #	return $widget::($graphelt::($this,canvas),path)
    #

    proc ~contour {this} {
	foreach s $contour::($this,segments) {
	    delete $s
	}
	fontedit::auxdeletecontour $graphelt::($this,window) $this 
    }
    
    # Tag name used for main lines. Used 
    # for presentation-related stuff
    proc tagname {this} {
	return "tag$this"
    }

    proc expandpoints {this} {
	set res {}
	foreach i $contour::($this,points) {
	    set res [concat $res [point::coords $i]]
	}
	return $res
    }
    
    proc create_drawing {this} {
	return None
    }

    proc update {this} {
    }
    
    proc attachsegment {this seg} {
	lappend ($this,segments) $seg
    }

    proc detachsegment {this seg} {
	listreplace contour::($this,segments) $seg
    }

    proc replacesegment {this s newsegs} {  
	listreplace contour::($this,segments) $s $newsegs
    }
    
    proc toggleselect {this} {
	if $contour::($this,selected) {
	    unselect $this
	} else {
	    select $this
	}
    }

    proc select {this} {
	set w $graphelt::($this,window)
	set p $widget::($w,path)
	set s $fontedit::($w,selection)
	set ($this,color) yellow
	set ($this,selected) 1
	$p itemconfig [tagname $this] -fill yellow
	selection::add $s $this
    }

    proc unselect {this} {
	set w $graphelt::($this,window)
	set p $widget::($w,path)
	set ($this,selected) 0
	set c $contour::($this,basecolor)
	set contour::($this,color) $contour::($this,basecolor)
	$p itemconfig [tagname $this] -fill $c
	selection::suppress $fontedit::($w,selection) $this
    }
    
    proc astext {this} {
	set res [list $contour::($this,type) $contour::($this,sens)]
	set fseg [lindex $contour::($this,segments) 0]
	set res [concat $res [point::astext [segment::firstpoint $fseg]]]
	foreach s $contour::($this,segments) {
	    lappend res [segment::astext $s]
	}
	return $res
    }

    proc deletePointFrom {this point segments} {
	# If the point is an external control :
	# 0. between more than 3 segments : ????

	if {[llength $segments] != 2} return
    
	set s1 [lindex $segments 0]
	
	set s2 [lindex $segments 1]
	
	if {$s1 == $s2} return
    	
	set pos1 [lsearch -exact $contour::($this,segments) $s1]
	set pos2 [lsearch -exact $contour::($this,segments) $s2]
	if {$pos1 == -1 || $pos2 == -1} {error "heeeeeeeeeellllpppppp"}
	# Three real cases are possible :
	# the points follow one another
	# or one is the first point
	if {$pos1 == $pos2 + 1 || ($pos1 == 0 && $pos2 != 1)} {
	    set tmp $s1
	    set s1 $s2
	    set s2 $tmp
	}
	
	# (if none of these cases applied, pos2 is after pos1)
	
	set t 0
	
	set class1 [classof $s1]
	set class2 [classof $s2]
	
	if {[string compare $class1 "::linesegment"] == 0} {incr t}
	if {[string compare $class2 "::linesegment"] == 0} {incr t}
	
	# 0. between two splines : keep the external controls.
	# 1. between a line and a spline : create a spline
	# 2. between two lines : delete the point and create a new line.	

	set p1 [segment::firstpoint $s1]
	set p2 [segment::lastpoint $s2]
	
	switch -exact $t {
	    0 {
		set c1 [spline::firstcontrol $s1]
		set c2 [spline::secondcontrol $s2]
		point::double $c1 $p1
		point::double $c2 $p2
		set seg [new spline $this $p1 $c1 $c2 $p2]
	    }
	    1 {
		if {[string compare $class1 "::linesegment"] == 0} {
		    set c1 [segment::lastpoint $s1]
		    set c2 [spline::secondcontrol $s2]
		} else {
		    set c1 [spline::firstcontrol $s1]
		    set c2 [segment::firstpoint $s2]
		}
		#point::double $c1 $p1
		#point::double $c2 $p2
		set seg [new spline $this $p1 $c1 $c2 $p2]
	    }
	    2 {
		set seg [new linesegment $this $p1 $p2]
	    }
	}
	
	detachsegment $this $s2
	replacesegment $this $s1 $seg
	delete $s1
	delete $s2
    }
}

# Classe segments :
# Les contours " de base ", droits ou splines

class segment {
    proc segment {this owner} graphelt {$graphelt::($owner,window)} {
	set segment::($this,owner) $owner
	### contour::attachsegment  $owner $this NON NON et NON.
	### Les segments ne s'attachent pas n'importe o.
	set segment::($this,smooth) ""
    }
    
    proc complete_segment {this} {
	set res [graphelt::get_drawing $this]
	set can $graphelt::($this,canvas)
	$can bind $res <Enter> [list graphelt::setred $this]
	$can bind $res <Leave> [list graphelt::unsetred $this]
	$can bind $res <B1-Motion> [list segment::moveaction $this %x %y ]
	$can bind $res <Button-1> [list segment::clickaction $this %x %y ]
	$can bind $res <Shift-Button-1> [list segment::shiftclickaction $this %x %y ]
    }

    proc ~segment {this} {
	contour::detachsegment $segment::($this,owner) $this 
    }
    
    virtual proc create_drawing {this};	#  creates the drawing form on the canvas.

    proc getcolor {this} {
	return $contour::($segment::($this,owner),color)
    }

    proc draw {this} {
	graphelt::_draw $this;		# call parent class procedure
	$graphelt::($this,canvas) addtag [contour::tagname $segment::($this,owner)] withtag \
	    [graphelt::get_drawing $this]
	$graphelt::($this,canvas) addtag contourelt withtag [graphelt::get_drawing $this]
	tagAsMember $this $segment::($this,owner)
    }

    virtual proc update {this} {
	graphelt::_update $this
    }

    proc move {this x y} {
	set o $graphelt::($this,window)
	set w $graphelt::($this,canvas)
	set s $segment::($this,owner)
	if $contour::($s,selected) {
	    $w move selected\
		$fontedit::($o,dx) $fontedit::($o,dy)
	} else {
	    $w move [graphelt::alltagname $s] \
		$fontedit::($o,dx) $fontedit::($o,dy)
	}
    }

    virtual proc expandpoints {this}
    virtual proc firstpoint {this} {}
    virtual proc lastpoint {this} {}

    virtual proc double {this};		# cuts a segment in two

    virtual proc ContourControlPoint {this number}; # gets ContourControlPoint number i (starts with 0)

    virtual proc astext {this};		# sends back a textual representation for the element.

    virtual proc change {this};

    proc clickaction {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    {double}  {double $this}
	    {splinetoline} {change $this}
	    {delete} {delete $segment::($this,owner)}
	    {select} {contour::toggleselect $segment::($this,owner)}
	}
    }

    proc shiftclickaction {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    default  {double $this}
	}
    }

    proc moveaction {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    {edit}  {move $this $x $y}
	}
    }

}

class linesegment {
    proc linesegment {this owner p1 p2} segment {$owner} {
	ContourControlPoint::attach $p1 $this
	ContourControlPoint::attach $p2 $this
	set ($this,p1) $p1
	set ($this,p2) $p2
	graphelt::draw $this

	set res [graphelt::get_drawing $this]
	fontedit::lower $graphelt::($this,window) $res
	segment::complete_segment $this
    }
    
    proc ~linesegment {this} {
	ContourControlPoint::detach_and_delete_if_orphan $linesegment::($this,p1) $this
	ContourControlPoint::detach_and_delete_if_orphan $linesegment::($this,p2) $this
    }

    proc create_drawing {this} {
	set res [eval $graphelt::($this,canvas) create line [expandpoints $this] $segment::($this,smooth)\
		     -fill [segment::getcolor $this]]
	fontedit::lower $graphelt::($this,window) $res
	return $res
    }
    
    proc firstpoint {this} {return $linesegment::($this,p1)}
    proc lastpoint {this} {return $linesegment::($this,p2)}

    proc expandpoints {this} {
	set p1 $linesegment::($this,p1)
	set p2 $linesegment::($this,p2)
	return [concat [point::coords $p1] [point::coords $p2]]
    }

    proc double {this} {
	set p1 $linesegment::($this,p1)
	set p2 $linesegment::($this,p2)
	set p3 [ContourControlPoint::middle $p1 $p2]
	ContourControlPoint::detach $p2 $this
	set ($this,p2) $p3
	ContourControlPoint::attach $p3 $this
	graphelt::update $this
	set newseg [new linesegment $segment::($this,owner) $p3 $p2]
	contour::replacesegment $segment::($this,owner) $this [list $this $newseg]
    }

    proc change {this} {		# returns a spline which corresponds to the current element.
	set p1 $linesegment::($this,p1)
	set p2 $linesegment::($this,p2)
	set c [point::middle $p1 $p2]
	set pc1 [point::middle $p1 $c]
	set pc2 [point::middle $p2 $c]
	set c1 [new ContourControlPoint $pc1 $ContourControlPoint::($p1,owner)]
	set c2 [new ContourControlPoint $pc2 $ContourControlPoint::($p1,owner)]
	set res [new spline $segment::($this,owner) $p1 $c1 $c2 $p2]
	contour::replacesegment $segment::($this,owner) $this $res
	delete $pc1
	delete $pc2
	delete $c
	delete $this
	return $res
    }
    
    virtual proc astext {this} {
	return [concat l \
		    [point::astext $linesegment::($this,p2)]]
    }
}

class spline {
    proc spline {this owner p1 c1 c2 p2} segment {$owner} {
	ContourControlPoint::attach $p1 $this
	ContourControlPoint::attach $p2 $this
	ContourControlPoint::attach $c1 $this
	ContourControlPoint::attach $c2 $this
	set ($this,p1) $p1
	set ($this,p2) $p2
	set ($this,c1) $c1
	set ($this,c2) $c2
	set segment::($this,smooth) "-smooth 1"

	
	graphelt::draw $this

	set res [graphelt::get_drawing $this]
	fontedit::lower $graphelt::($this,window) $res
	segment::complete_segment $this
    }
    
    proc ~spline {this} {
	ContourControlPoint::detach_and_delete_if_orphan $spline::($this,p1) $this
	ContourControlPoint::detach_and_delete_if_orphan $spline::($this,p2) $this
	ContourControlPoint::detach_and_delete_if_orphan $spline::($this,c1) $this
	ContourControlPoint::detach_and_delete_if_orphan $spline::($this,c2) $this
    }


    proc firstpoint {this} {return $spline::($this,p1)}
    proc lastpoint {this} {return $spline::($this,p2)}
    proc firstcontrol {this} {return $spline::($this,c1)}
    proc secondcontrol {this} {return $spline::($this,c2)}

    proc update {this} {
	foreach {main auxline1 auxline2} $graphelt::($this,drawing) {}
	eval $graphelt::($this,canvas) coords $main  [expandpoints $this]
	set p1 $spline::($this,p1)
	set p2 $spline::($this,p2)
	set c1 $spline::($this,c1)
	set c2 $spline::($this,c2)
	eval $graphelt::($this,canvas) coords $auxline1 [point::coords $p1] [point::coords $c1]
	eval $graphelt::($this,canvas) coords $auxline2 [point::coords $p2] [point::coords $c2]
    }

    proc create_drawing {this} {
	set p1 $spline::($this,p1)
	set p2 $spline::($this,p2)
	set c1 $spline::($this,c1)
	set c2 $spline::($this,c2)
	set res1 [eval $graphelt::($this,canvas) create line [expandpoints $this] $segment::($this,smooth)\
		      -fill [graphelt::getcolor $this]] 
	#$graphelt::($this,canvas) addtag contourelt withtag $res1 
	set res2 [eval $graphelt::($this,canvas) create line [point::coords $p1] [point::coords $c1]  -fill darkgreen] 
	set res3 [eval $graphelt::($this,canvas) create line [point::coords $p2] [point::coords $c2]  -fill darkgreen]
	fontedit::lower $graphelt::($this,window) $res1
	fontedit::lower $graphelt::($this,window) $res2
	fontedit::lower $graphelt::($this,window) $res3
	return [list $res1 $res2 $res3]
    }

    proc expandpoints {this} {
	set p1 $spline::($this,p1)
	set p2 $spline::($this,p2)
	set c1 $spline::($this,c1)
	set c2 $spline::($this,c2)
	return [concat [point::coords $p1] \
		[eval makecurve \
		[point::coords $p1] \
		[point::coords $c1] \
		[point::coords $c2] \
		[point::coords $p2] \
		3\
		]]
	return [concat \
		    [point::coords $p1] \
		    [point::coords3in4 $p1 $c1] \
		    [point::coords3in4 $p2 $c2] \
		    [point::coords $p2] \
		   ]

    }

    proc double {this} {
	set p1 $spline::($this,p1)
	set p2 $spline::($this,p2)
	set c1 $spline::($this,c1)
	set c2 $spline::($this,c2)

	set o $ContourControlPoint::($p1,owner)
	set a [point::middle $p1 $c1]
	set b [point::middle $p2 $c2]
	set d [point::middle $c1 $c2]
	set e [point::middle $a $d]
	set f [point::middle $b $d]
	set g [point::middle $e $f]
	
	set pa [new ContourControlPoint $a $o]
	set pb [new ContourControlPoint $b $o]
	set pe [new ContourControlPoint $e $o]
	set pf [new ContourControlPoint $f $o]
	set pg [new ContourControlPoint $g $o]

	delete $a
	delete $b
	delete $d
	delete $e
	delete $f
	delete $g

	# new splines : p1 pa pe pg and pg pf pb p2
	# so : exeunt c1 c2que
	
	ContourControlPoint::detach $p2 $this
	ContourControlPoint::detach_and_delete_if_orphan $c1 $this
	ContourControlPoint::detach_and_delete_if_orphan $c2 $this
	
	set ($this,c1) $pa
	set ($this,c2) $pe
	set ($this,p2) $pg
	ContourControlPoint::attach $pa $this
	ContourControlPoint::attach $pe $this
	ContourControlPoint::attach $pg $this
	graphelt::update $this

	set newseg [new spline $segment::($this,owner) $pg $pf $pb $p2]
	contour::replacesegment $segment::($this,owner) $this [list $this $newseg]
    }    
    
    proc change {this} {		# returns a line which corresponds to the current element.
	set p1 $spline::($this,p1)
	set p2 $spline::($this,p2)
	set c1 $spline::($this,c1)
	set c2 $spline::($this,c2)
	set o $segment::($this,owner)
	set res [new linesegment $o $p1 $p2]
	contour::replacesegment $o $this $res
	delete $this
	return $res
    }

    virtual proc astext {this} {
	return [concat s \
		    [point::astext $spline::($this,c1)]\
		    [point::astext $spline::($this,c2)]\
		    [point::astext $spline::($this,p2)]]
    }
}

# Class stroke represents a complex stroke.
# It's basically a list of segments, with a width vector at each end.
# (for a start, maybe we'll make this width a constant).
#
# 
#class stroke {
    
#}

class point {
    proc point {this x y} {
	set ($this,x) $x 
	set ($this,y) $y
    }
    
    proc ~point {this} {}

    virtual proc x {this} {return $point::($this,x)}
    virtual proc y {this} {return $point::($this,y)}


    proc coords {this} {
	return [list [x $this] [y $this]]
    }

    virtual proc setcoords {this x y} {
	set ($this,x) $x
	set ($this,y) $y	
    }

    proc middle {this m} {
	set x0 [x $this]
	set x1 [x $m]
	set y0 [y $this]
	set y1 [y $m]
	return [new point [expr ($x0 + $x1) / 2.0] [expr ($y0 + $y1) / 2.0]]
    }

    proc coords3in4 {this a} {
	return [list [expr ([x $this] + 3.0 * [x $a])/ 4] \
		    [expr ([y $this] + 3.0 * [y $a])/ 4]]
    }

    virtual proc double {this orig} {
	set x [expr {[x $orig] + 2 * ([x $this] -  [x $orig])}]
	set y [expr {[y $orig] + 2 * ([y $this] -  [y $orig])}]
	setcoords $this $x $y
    }

    virtual proc astext {this} {
	set c [coords $this]
	return [format {%.4f %.4f} [lindex $c 0] [lindex $c 1]]
    }
}


# Control points : attached to segments which are attached to contours.
# Move to :
# 	ContourControlPoint
#	etc...
#	all derived from ControlPoint


# A ControlPoint is a point with associated controled forms

class ControlPoint {

    proc ControlPoint {this p} \
	    graphelt {$graphelt::($owner,window)} point {[point::x $p] [point::y $p]} {
	set w $graphelt::($owner,window)
	
	# Controled forms are stored in this variable :
	set ($this,controled) {}
	
	# build our representation
	
	graphelt::draw $this

	set can $graphelt::($this,canvas)
	set draw [graphelt::get_drawing $this]
	$can bind $draw <Enter> [list graphelt::setred $this]
	$can bind $draw <Leave> [list graphelt::unsetred $this]
	$can bind $draw <B1-Motion> "ControlPoint::moveaction $this %x %y; break"
	$can bind $draw <Button-1>  "ControlPoint::clickaction $this %x %y; break"
    } 

    proc ~ControlPoint {this} {
	set w $graphelt::($this,window)
	unset ($this,controled)
	# listreplace fontedit::($w,points) $this
	# destructeur
    }

    # Action to be taken when the point is moved.
    
    proc moveaction {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    {edit}  {move $this $x $y}
	}
    }

    # Action to be taken when the point is clicked
    
    proc clickaction {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    {select}  {}
	    {delete} {deletePoint $this}
	}
    }
    
    # State that point controls the "controled" graphelt.
    
    proc control {this controled} {
	set can $graphelt::($this,canvas)
	# Each graphelt has an associated tag which allows to
	# change it in a global way.
	# So, give this tag to the point
	graphelt::tagAsMember $this $segment::($segment,owner)
	lappend ($this,controled) $segment
    }

    # Unlinked a controled item from the point. If nothing is controled, destroy the point.
    
    proc detach_and_delete_if_orphan {this controled} {
	detach $this $controled
	if {[llength $ControlPoint::($this,controled)] == 0} {
	    delete $this
	}
    }

    # Unlinked a controled item from the point.

    proc detach {this controled} {
	listreplace ControlPoint::($this,controled) $controled
    }

    # moves the point to x, y.
    proc move {this x y} {
	set p [fontedit::ongrid $graphelt::($this,window) $x $y]
	set x [point::x $p]
	set y [point::y $p]
	delete $p
	$graphelt::($this,canvas) coords [graphelt::get_drawing $this] $x $y
	foreach i $ControlPoint::($this,controled) {
	    graphelt::update $i
	}
    }

    proc setcoords {this x y} {
	move $this $x $y
    }

    proc setred {this} {
	
    }

    proc unsetred {this} {
    }
    
    proc create_drawing {this} {
	set can $graphelt::($this,canvas)
	set x $point::($this,x)
	set y $point::($this,y)
	set circ [$can create image $x $y -anchor center -image $ControlPoint::(image)]
	$can addtag ControlPoint withtag $circ 
	unset point::($this,x)
	unset point::($this,y)
	return $circ
    }

    proc update {this} {}
    
    proc x {this} {
	set cs [$graphelt::($this,canvas) coords [graphelt::get_drawing $this]]
	return [lindex $cs 0] 
    }

    proc y {this} {
	set cs [$graphelt::($this,canvas) coords [graphelt::get_drawing $this]]
	return [lindex $cs 1]
    }

    proc middle {this m} {
	set p [point::middle $this $m]
	set ret [new ControlPoint $p $ControlPoint::($this,owner)]
	delete $p
	return $ret
    }

    proc astext {this} {
	set c [fontedit::realpoint $graphelt::($this,window) $this]
 	return [format {%.4f %.4f} [lindex $c 0] [lindex $c 1]]
    }
    
    # deletes a point.
    proc deletePoint {this} {
	#deletes a control point
	# If the point is an internal control, ask for its deletion.
    }

    set ControlPoint::(image) [image create bitmap -data {
	#define square_width 8
	#define square_height 8
	static unsigned char square_bits[] = {
	    0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xff, };
	} ]
}

class ContourControlPoint {

    proc ContourControlPoint {this p owner} \
	    graphelt {$graphelt::($owner,window)} point {[point::x $p] [point::y $p]} {
	set ($this,owner) $owner; # contour which owns the point
	set w $graphelt::($owner,window)
	# lappend fontedit::($w,points) $this
	set ($this,segments) {}
	graphelt::draw $this

	set can $graphelt::($this,canvas)
	set draw [graphelt::get_drawing $this]
	$can bind $draw <Enter> [list graphelt::setred $this]
	$can bind $draw <Leave> [list graphelt::unsetred $this]
	$can bind $draw <B1-Motion> "ContourControlPoint::moveaction $this %x %y; break"
	$can bind $draw <Button-1>  "ContourControlPoint::clickaction $this %x %y; break"
    } 

    proc ~ContourControlPoint {this} {
	set w $graphelt::($this,window)
	# listreplace fontedit::($w,points) $this
	# destructeur
    }

    # Action to be taken when the point is moved.
    
    proc moveaction {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    {edit}  {move $this $x $y}
	}
    }

    # Action to be taken when the point is clicked
    
    proc clickaction {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    {select}  {}
	    {delete} {deletePoint $this}
	}
    }
    
    # Attach a segment to a point  ?
    # In reality, this is a more general function. It adds the segment to the
    # list of controled stuff for this point
    
    proc attach {this segment} {
	graphelt::tagAsMember [graphelt::get_drawing $this] $segment::($segment,owner)
	lappend ($this,segments) $segment
    }

    # Unlinked a controled item from the point. If nothing is controled, destroy the point.
    
    proc detach_and_delete_if_orphan {this segment} {
	detach $this $segment
	if {[llength $ContourControlPoint::($this,segments)] == 0} {
	    delete $this
	}
    }

    # Unlinked a controled item from the point.

    proc detach {this segment} {
	listreplace ContourControlPoint::($this,segments) $segment
    }

    # moves the point to x, y.
    proc move {this x y} {
	#set dx [expr $x - [point::x $this]]
	#set dy [expr $y - [point::y $this]]
	#point::setcoords $this $x $y
	#$graphelt::($this,canvas) move $graphelt::($this,drawing) $dx $dy
	set p [fontedit::ongrid $graphelt::($this,window) $x $y]
	set x [point::x $p]
	set y [point::y $p]
	delete $p
	$graphelt::($this,canvas) coords [graphelt::get_drawing $this] $x $y
	foreach i $ContourControlPoint::($this,segments) {
	    graphelt::update $i
	}
    }

    proc setcoords {this x y} {
	move $this $x $y
    }

    proc setred {this} {
	
    }

    proc unsetred {this} {
    }
    
    proc create_drawing {this} {
	set can $graphelt::($this,canvas)
	set x $point::($this,x)
	set y $point::($this,y)
	set circ [$can create image $x $y -anchor center -image $ContourControlPoint::(image)]
	$can addtag ContourControlPoint withtag $circ 
	unset point::($this,x)
	unset point::($this,y)
	return $circ
    }

    proc update {this} {}
    
    proc x {this} {
	set cs [$graphelt::($this,canvas) coords [graphelt::get_drawing $this]]
	return [lindex $cs 0] 
    }

    proc y {this} {
	set cs [$graphelt::($this,canvas) coords [graphelt::get_drawing $this]]
	return [lindex $cs 1]
    }

    proc middle {this m} {
	set p [point::middle $this $m]
	set ret [new ContourControlPoint $p $ContourControlPoint::($this,owner)]
	delete $p
	return $ret
    }

    proc astext {this} {
	set c [fontedit::realpoint $graphelt::($this,window) $this]
 	return [format {%.4f %.4f} [lindex $c 0] [lindex $c 1]]
    }
    
    # deletes a point, but keep the contour clean
    proc deletePoint {this} {
	#deletes a control point
	# If the point is an internal control, ask for its deletion.
	if {[llength $ContourControlPoint::($this,segments)] == 1 && \
		[segment::isInternal $this]} {
	    segment::deletePoint $this
	} elseif {[llength $ContourControlPoint::($this,segments)] > 1} {
	    contour::deletePointFrom $ContourControlPoint::($this,owner) $this $ContourControlPoint::($this,segments)
	}
    }

    set ContourControlPoint::(image) [image create bitmap -data {
	#define square_width 8
	#define square_height 8
	static unsigned char square_bits[] = {
	    0xff, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0xff, };
	} ]
}


class image {
    proc image {this window name} graphelt {$window} {
	set ($this,x) 100
	set ($this,y) 100
	set ($this,file) $name

	graphelt::draw $this
	set ed $graphelt::($this,window)
	set fontedit::($ed,img) $this
    }

    proc ~image {this} {
	set can $graphelt::($this,canvas)
	#$can delete withtag $fontedit::($this,img)
	::image delete $image::($this,image)
    }

    proc create_drawing {this} {
	set i [::image create photo -file $image::($this,file)]
	set ($this,image) $i
	set can $graphelt::($this,canvas)
	$can delete withtag images
	set id [$can create image $image::($this,x) $image::($this,y) -image $i -anchor n]	
	$can addtag  images withtag $id
	$can lower $id

	$can bind $id <B1-Motion> [list image::moveaction $this %x %y ]
	$can bind $id <Button-1> [list image::clickaction $this %x %y ]
	return $id
    }

    proc clickaction {this x y} {}

    proc moveaction  {this x y} {
	set state $fontedit::($graphelt::($this,window),state)
	switch -exact $state {
	    default  {move $this $x $y}
	}
    }

    proc move {this x y} {
	set o $graphelt::($this,window)
	set ($this,x) $x
	set ($this,y) $y
	$graphelt::($this,canvas) move [graphelt::get_drawing $this] $fontedit::($o,dx)  $fontedit::($o,dy) 
    }
    
    proc lower {this} {
	$graphelt::($this,canvas) lower [graphelt::get_drawing $this]
    }
}

class selection {
    
    proc selection {this window} {
	set selection::($this,cont) {}
	set selection::($this,window) $window
    }
    
    proc ~selection {this} {
    }

    proc add {this cont} {
	set w $widget::($selection::($this,window),path)
	$w addtag selected withtag [graphelt::alltagname $cont]
	lappend selection::($this,cont) $cont
	update_selbox $this
    }

    proc suppress {this cont} {
	set w $widget::($selection::($this,window),path)
	$w dtag [graphelt::alltagname $cont] selected
	listreplace selection::($this,cont) $cont
	update_selbox $this
    }

    proc update_selbox {this} {
	set l [llength $selection::($this,cont)]
	delete_selbox $this
	if  {$l != 0} {
	    catch {create_selbox $this}
	}
    }

    proc create_selbox {this} {
	set w $widget::($selection::($this,window),path)
	foreach {llx ury urx lly} [$w bbox selected] {}
	# create a button at pos. urx and ury.
	# THERE SHOULD BE A CLASS FOR THIS.
	# I should in fact change ContourControlPoint and cut it into two levels.
	set b [$w create oval [expr $urx - 2] [expr $ury - 2]\
		   [expr $urx + 2] [expr $ury + 2] -fill darkgreen]
	$w bind $b <B1-Motion> [list selection::scale $this %x %y]
	$w addtag selected withtag $b
	set ($this,button) $b
    }

    proc delete_selbox {this} {
 	set w $widget::($selection::($this,window),path)
	catch {$w delete $selection::($this,button)}
    }

    proc scale {this x y} {
	set w $widget::($selection::($this,window),path)
	foreach {llx ury urx lly} [$w bbox selected] {}
	# scaling center :
	set sx [expr "($urx + $llx)/2.0"]
	set sy [expr "($ury + $lly)/2.0"]
	# scaling factor :
	set sf [expr "1.0*($x - $sx) / ($urx - $sx)"]
	$w scale selected $sx $sy $sf $sf
    }

    
    proc clear {this} {
	foreach i $selection::($this,cont) {
	    contour::unselect $this
	}
    }
}

##############################################################################
#
# Graphic guides
# Elements which control point placement
# Only activated guides are used for this.
#
##############################################################################

class GraphGuide {
    proc GraphGuide {this window} {
    }
    
    virtual proc attract {this point}
}

# We should build a simple "line" graphicelt, which would be used by contour and the others.

class LineGuide {
    proc LineGuide {this window p1 p2} graphelt {$this $window} GraphGuide {$this $window} {
	ContourControlPoint::attach $p1 $this
	ContourControlPoint::attach $p2 $this
	set ($this,p1) $p1
	set ($this,p2) $p2
	graphelt::draw $this
	set res [graphelt::get_drawing $this]
	fontedit::lower $graphelt::($this,window) $res
    }

    proc ~LineGuide {this} {
	ContourControlPoint::detach_and_delete_if_orphan $LineGuide::($this,p1) $this
	ContourControlPoint::detach_and_delete_if_orphan $LineGuide::($this,p2) $this
    }

    proc create_drawing {this} {
	set res [eval $graphelt::($this,canvas) create line [expandpoints $this] \
		-fill [graphelt::getcolor $this]]
	fontedit::lower $graphelt::($this,window) $res
	return $res
    }

    proc expandpoints {this} {
	set p1 $LineGuide::($this,p1)
	set p2 $LineGuide::($this,p2)
	return [concat [point::coords $p1] [point::coords $p2]]
    }

    #proc get_drawing {this} {
	#
    #}
    
    # proc draw {}

    #virtual proc move {this x y} {}

    proc astext {this} {
	return ""
    }
    
    # virtual proc getcolor {this}

    # virtual proc setred {this} 

    # virtual proc unsetred {this}

    virtual proc attract {this point} {
	set p1 $LineGuide::($this,p1)
	set p2 $LineGuide::($this,p2)
	set norm [norm $p1 $p2]
	set x [point::x $point]
	set y [point::y $point]
	set vx [expr [point::x $p2] - [point::x $p1]]
	set vy [expr [point::y $p2] - [point::y $p1]]
	set scal [expr {($x - [point::x $p1]) * $vx + ($x - [point::y $p1]) * $vy}]
	set x [expr [point::x $p1] + $scal * $vx / $norm ]
	set y [expr [point::y $p1] + $scal * $vy / $norm ]
	point::setcoords $point $x $y
    }
}
##############################################################################
#
# Widget Editeur :
# Une spcialisation de canvas
#
##############################################################################


class fontedit {

    proc fontedit {this base args} canvas {$base -width 400 -height 400 -bg white $args} {
	set ($this,ox) 0
	set ($this,oy) 0
	set ($this,oscale) 1;		#  original point to coordinate scale
	set ($this,scale) 0;		#  scale relative to initial size
	set ($this,rscale) 1;		#  oscale * exp(scale)
	set ($this,dscale) 1;		#  1 / rscale
	set ($this,contours) {}
	#set ($this,points) {}
	#set ($this,selected) None
	set ($this,selection) [new selection $this]
	set ($this,char) 32
	set ($this,fontfile) {None}
	set ($this,font) [sbzr_newfont]
	set ($this,grid) 0
	set ($this,gridp) 0
	set ($this,guides) {} ; # graphic guides
	set ($this,oldx) 0
	set ($this,oldy) 0
	set ($this,dx) 0
	set ($this,dy) 0
	set ($this,type) "P"

	# l'tat spcifie l'action  mener
	
	set fontedit::($this,state) add
	bind $widget::($this,path) <Button-1>\
	    [list fontedit::clickaction $this bouton1 %x %y]
	bind $widget::($this,path) <B1-Motion>\
	    [list fontedit::moveaction $this %x %y]	
	bind $widget::($this,path) <ButtonRelease-1>\
	    [list fontedit::releaseaction $this %x %y]
	

	set ($this,bg) [widget::cget $this -background]
    }
    
    proc ~fontedit {this} {
	clear $this
	if { $fontedit::($this,gridp) != 0 } {
	    delete $fontedit::($this,screenpoint)
	}
	delete $fontedit::($this,selection)
    }

    proc options {this} {}

    proc clear {this} {
	foreach c $fontedit::($this,contours) {
	    delete $c
	}
	#foreach p $fontedit::($this,points) {
	#    delete $p
	#}
    }

    proc setgrid {this gx gy} {
	set ($this,grid) 1
	set ($this,gridp) [new point [expr $gx * $fontedit::($this,rscale)] \
			       [expr $gy * $fontedit::($this,rscale)]]
			       
    }

    proc ongrid {this x y} {
	if {$fontedit::($this,grid)} {
	    set gx [point::x $fontedit::($this,gridp)]
	    set gy [point::y $fontedit::($this,gridp)]
	    set x [expr floor(1.0* $x / $gx) * $gx]
	    set y [expr floor(1.0* $y / $gy) * $gy]
	}
	return [new point $x $y]
    }

    #proc getwindow {this} {
    #return $widget::($this,path)
    #}
    
    proc fixscale {this} {
	set ($this,rscale) [expr $fontedit::($this,oscale) * exp($fontedit::($this,scale))]
	set ($this,dscale) [expr 1.0 / $fontedit::($this,rscale)]
    }

    proc setscale {this sc ox oy} {
	set ($this,ox) $ox
	set ($this,oy) $oy
	set ($this,oscale) $sc
	fixscale $this
    }

    proc ajoutecontour {this x y {type T}} {
	set c [new contour $this $type]
	switch -exact -- $type {
	    T {
		set pp1 [ongrid $this $x $y]
		set p1 [new ContourControlPoint $pp1 $c]
		set p2 [new ContourControlPoint $pp1 $c]
		delete $pp1
		set s1 [new linesegment $c $p1 $p2]
		set contour::($c,segments) [list $s1]
	    }
	    default {
		set pp1 [ongrid $this $x $y]
		set p1 [new ContourControlPoint $pp1 $c]
		set p2 [new ContourControlPoint $pp1 $c]
		delete $pp1
		set s1 [new linesegment $c $p1 $p2]
		set s2 [new linesegment $c $p2 $p1]
		set contour::($c,segments) [list $s1 $s2]
		#contour::attachsegment $c $s1
		#set contour::($c,segments) [list $s1]
	    }
	}
    }
    
    proc addGuide {this x y {type line}} {
	set pp1 [new point $x $y]
	set p1 [new ContourControlPoint $pp1 $this]
	set p2 [new ContourControlPoint $pp1 $this]
	delete $pp1
	set c [new LineGuide $this p1 p2]
    }

    proc auxdeletecontour {this contour} {
	listreplace fontedit::($this,contours) $contour
    }
    
    proc editclosest {this x y} {
	set can $widget::($this,path)
	set ident [$can find  closest $x $y]
	#if {[info exists ContourControlPoint::(control,$can,$ident)]} {
	#set fontedit::($this,selected) $ContourControlPoint::(control,$can,$ident)
	#}
    }
    
    proc clickaction {this bouton x y} {
	set ($this,oldx) $x
	set ($this,oldy) $y
	switch $fontedit::($this,state) {
	    add	{fontedit::ajoutecontour $this $x $y $fontedit::($this,type)}
	    edit {}
	    select {}
	    center {newcenter $this $x $y}
	    delete {}
	    addguide {fontedit::addGuide $this $x $y line}
	}
    }

    proc releaseaction {this x y} {
	switch $fontedit::($this,state) {
	    add	{}
	    edit {}
	    select {}
	}
    }

    proc moveaction {this x y} {
	set ($this,dx) [expr $x - $fontedit::($this,oldx)]
	set ($this,dy) [expr $y - $fontedit::($this,oldy)]
	set ($this,oldx) $x
	set ($this,oldy) $y	
	switch $fontedit::($this,state) {
	    add	{}
	    edit {}
	    select {}
	    scale {}
	}
    }

    proc newcenter {this x y} {
	set w $widget::($this,path)
	set centerx [expr [$w cget -width] / 2.0]
	set centery [expr [$w cget -height] / 2.0]
	set dx [expr  $centerx - $x]
	set dy [expr  $centery - $y]
	set ($this,ox)	[expr $fontedit::($this,ox) + $dx]
	set ($this,oy)	[expr $fontedit::($this,oy) + $dy]
	$w move all $dx $dy
    }

    proc scaleall {this scale} {
	set sc [expr  exp($scale - $fontedit::($this,scale))]
	set w $widget::($this,path)
	set centerx [expr [$w cget -width] / 2.0]
	set centery [expr [$w cget -height] / 2.0]
	set ($this,scale) $scale
	set ($this,ox)	[expr ($fontedit::($this,ox) - $centerx) * $sc  + $centerx]
	set ($this,oy)	[expr ($fontedit::($this,oy) - $centery) * $sc  + $centery]
	fixscale $this
	if {$sc != 0} {
	    $widget::($this,path) scale all $centerx $centery\
		$sc $sc
	}
    }


    # return a screen point from original coordinates
    proc screenpoint {this x y} {
	set sc $fontedit::($this,rscale)
	set xx [expr $x * $fontedit::($this,rscale)  + $fontedit::($this,ox)]
	set yy [expr (- $y )* $fontedit::($this,rscale) + $fontedit::($this,oy)]
	return [new point $xx $yy]
    }

    # return a file point from screen coordinates
    proc realpoint {this p} {
	set sc [expr exp($fontedit::($this,scale))]
	set x [point::x $p]
	set y [point::y $p]
	set xx [expr 1.0 * ($x - $fontedit::($this,ox)) * ($fontedit::($this,dscale))]
	set yy [expr 1.0 * (- $y + $fontedit::($this,oy)) * ($fontedit::($this,dscale))]
	return [list $xx $yy]
    }
    
    proc load {this font} {
	set ($this,fontfile) $font
	set ($this,char) 32
	set ($this,font) [sbzr_newfont $font]
	clear $this
	setchar $this 32
    }
    
    proc astext {this} {
	set res ""
	# We use tk canvas' bounding box. As it is not that precise,
	# we first have to zoom the text a little.
	# but we need to revert to the current zoom after.
	set scale $fontedit::($this,scale)
	foreach c $fontedit::($this,contours) {
	    lappend res [contour::astext $c]
	}
	scaleall $this 10
	foreach {llx ury  urx lly}  [$widget::($this,path) bb contourelt] {}
	set ll [new point $llx $lly]
	set ur [new point $urx $ury]
	set v1 [realpoint $this $ll]
	set v2 [realpoint $this $ur]
	set width [expr [lindex $v2 0] - [lindex $v1 0]]
	delete $ll
	delete $ur
	scaleall $this 0
	set res [concat $fontedit::($this,char) $width $v1 $v2 $res]
	scaleall $this $scale
	return $res
    }

    proc loadimg {this name} {
	set fontedit::($this,img) [new image $this $name]
    }
    
    proc delimg {this} {
	delete $fontedit::($this,img)
    }

    proc lower {this c} {
	set w $widget::($this,path)
	$w lower $c
	catch {image::lower $fontedit::($this,img)}
    }

    proc repere {this} {
	set p1 [screenpoint $this 0 0]
	set p2 [screenpoint $this 0 14]
	set p3 [screenpoint $this 14 0]
	set w $widget::($this,path)
	set u [$w create line [point::x $p1] [point::y $p1] [point::x $p2] [point::y $p2] -fill red]
	set v [$w create line [point::x $p1] [point::y $p1] [point::x $p3] [point::y $p3] -fill red]
	$w addtag repere withtag $u
	$w addtag repere withtag $v
    }

    
    proc addtext {this text} {
	foreach c $text {
	    set pt [screenpoint $this [lindex $c 2] [lindex $c 3]]
	    set sens [lindex $c 1]
	    set lecontour [new contour $this $sens]

	    set cpt [new ContourControlPoint $pt $lecontour]
	    set premier $cpt
	    delete $pt
	    
	    set l [llength [lrange $c 4 end]]
	    set i 0
	    foreach seg [lrange $c 4 end] {
		incr i
		switch -exact [lindex $seg 0] {
		    s { 
			set c1 [screenpoint $this [lindex $seg 1] [lindex $seg 2]]
			set c2 [screenpoint $this [lindex $seg 3] [lindex $seg 4]]
			set cc1 [new ContourControlPoint $c1 $lecontour]
			set cc2 [new ContourControlPoint $c2 $lecontour]
			delete $c1
			delete $c2
			if { $i == $l && [string compare $sens "T"] != 0} {
			    set cp2 $premier
			} else {
			    set p2 [screenpoint $this [lindex $seg 5] [lindex $seg 6]]
			    set cp2 [new ContourControlPoint $p2 $lecontour]
			    delete $p2
			}
			contour::attachsegment $lecontour [new spline $lecontour $cpt $cc1 $cc2 $cp2]
		    }
		    l {
			if { $i == $l && [string compare $sens "T"] != 0} {
			    set cp2 $premier
			} else {
			    set p2 [screenpoint $this [lindex $seg 1] [lindex $seg 2]]
			    set cp2 [new ContourControlPoint $p2 $lecontour]
			    delete $p2
			}
			contour::attachsegment $lecontour [new linesegment $lecontour $cpt $cp2]
		    }
		}
		set cpt $cp2
	    }
	}
    }

    proc setchar {this char} {
	set ($this,char) $char
	# destruction of everything
	set crep [sbzr_getchar $fontedit::($this,font) $char]
	addtext $this [lrange $crep 6 end]
    }

    proc save {this {name noname.bzr}} {
	sbzr_savefont $fontedit::($this,font) $name 
    }
    
    proc savechar {this} {
	sbzr_setchar $fontedit::($this,font) $fontedit::($this,char) \
	    [astext $this] 
    }

    proc getchar {this} {
	clear $this
	setchar $this $fontedit::($this,char)
    }

    proc nextchar {this} {
	fontedit::clear $this
	incr fontedit::($this,char)
	fontedit::setchar $this $fontedit::($this,char)
    }

    proc prevchar {this} {
	fontedit::clear $this
	incr fontedit::($this,char) -1
	fontedit::setchar $this $fontedit::($this,char)
    }
    
    proc invertlr {this} {
	$widget::($this,path) delete withtag repere
	foreach {x1 y1 x2 y2} [$widget::($this,path) bbox all] {}
	set centerx [expr ($x1 + $x2) / 2.0]
	set centery [expr ($y1 + $y2) / 2.0]
	$widget::($this,path) scale all $centerx $centery -1 1
	repere $this
    }

    proc invertud {this} {
	$widget::($this,path) delete withtag repere
	foreach {x1 y1 x2 y2} [$widget::($this,path) bbox all] {}
	set centerx [expr ($x1 + $x2) / 2.0]
	set centery [expr ($y1 + $y2) / 2.0]
	$widget::($this,path) scale all $centerx $centery 1 -1
	repere $this
    }

}

sesh .s
destroy .s
class sesh {
    proc sesh {this parentPath args} widget {
	[eval ::sesh $parentPath.$this $args]} {
	} 
    
    proc ~sesh {this} {
	destroy $widget::($this,path)
    }
    
    proc options {this} {
	return {}
    }
}


class export_sign {
    proc export_sign {this owner parentPath args} composite {[new frame $parentPath] $args} {
	set p $widget::($this,path)
	set ($this,owner) $owner
	composite::manage $this\
	    [new label $p -text "Enter code for exported sign :"] texte\
	    [new entry $p] entree\
	    [new button $p -text "export" -command [list export_sign::export $this]] ok \
	    [new button $p -text "cancel" -command [list export_sign::cancel $this]] cancel
	grid $composite::($this,texte,path) $composite::($this,entree,path)
	grid $composite::($this,ok,path) $composite::($this,cancel,path)
	composite::complete $this
    }
    
    proc ~export_sign {this} {
    }

    proc export {this} {
	clipboard clear
	set name [$composite::($this,entree,path) get]
	fonteditor::export $export_sign::($this,owner) $name
    }

    proc cancel {this} {
	fonteditor::cancel_export $export_sign::($this,owner)
    }

    proc options {this} {
	return {}
    }
}

class fonteditor {
    global env
    variable ICONDIR [file join $env(SESHLIB) icons]
    proc fonteditor {this parentPath args} composite {
	[new frame $parentPath] $args
    } {
	composite::manage $this\
	    [new frame $widget::($this,path)] menu \
	    [new fontedit $widget::($this,path)] edi \
	[new labelledFrame $widget::($this,path)] labelf
	    
	set labelpath $composite::($this,labelf,path)

	grid $composite::($this,menu,path) - -sticky ew
	addmenus $this

	grid $composite::($this,edi,path) $labelpath -sticky nsew
	grid rowconfigure $widget::($this,path) 0 -weight 0
	grid rowconfigure $widget::($this,path) 1 -weight 1
	grid rowconfigure $widget::($this,path) 2 -weight 0
	
	fontedit::setscale $composite::($this,edi) 10 20 300
	#fontedit::setgrid $composite::($this,edi) 2 2

	fontedit::repere $composite::($this,edi)


	# Mode choice

	set f0 [new frame $labelpath]
	set fpath0 $widget::($f0,path) 
	
	set edi $composite::($this,edi)
	
	set var fontedit::($edi,type)

	set pal1 [new radiobutton $fpath0 -variable $var -value "P" -text "black"]
	set pal2 [new radiobutton $fpath0 -variable $var -value "N" -text "white"]
	#set pal3 [new radiobutton $fpath0 -variable $var -value "T" -text "stroke"]
	#set pal4 [new radiobutton $fpath0 -variable $var -value "S" -text "closed"]
	
	grid $widget::($pal1,path) $widget::($pal2,path)
	
	set f1 [new frame $labelpath]
	set fpath1 $widget::($f1,path) 

	set var fontedit::($edi,state)
	
	set b1 [new radiobutton $fpath1 -variable $var -value add -text "add contour"]
	set b2 [new radiobutton $fpath1 -variable $var -value delete -text "delete"]
	set b3 [new radiobutton $fpath1 -variable $var -value edit -text "edit contour"]
	set b4 [new radiobutton $fpath1 -variable $var -value addguide -text "add guide"]
	set b5 [new radiobutton $fpath1 -variable $var -value double -text "double"]
	set b6 [new radiobutton $fpath1 -variable $var -value splinetoline -text "spline <-> line"]
	set b7 [new radiobutton $fpath1 -variable $var -value center -text "center"]
	set b8 [new radiobutton $fpath1 -variable $var -value select -text "select"]
	set b9 [new radiobutton $fpath1 -variable $var -value move -text "move selection"]
	set b10 [new radiobutton $fpath1 -variable $var -value scale -text "scale selection"]

	foreach b [list $b1 $b2 $b3 $b4 $b5 $b6 $b7 $b8 $b9 $b10] {
	    grid $widget::($b,path)
	}
	

	set f2 [new frame $labelpath]
	set fpath2 $widget::($f2,path) 
	
	set bt0 [new button $fpath2 -text "preview" -command [list fonteditor::preview $this]]
	set bt1 [new button $fpath2 -text "clear" -command [list fontedit::clear $edi] ]
	set bt2 [new scale $fpath2  -from -10  -to 10 \
		     -tickinterval 5.0 -resolution 0.1\
		     -command [list fontedit::scaleall $edi]]
	set bt3 [new button $fpath2 -text "set char" -command [list fontedit::savechar $edi]]

	set bt4 [new button $fpath2 -text "get char" -command [list fontedit::getchar $edi]]
	set bt5 [new button $fpath2 -text "prev" -command [list fontedit::prevchar $edi]]
	set bt6 [new button $fpath2 -text "next" -command [list fontedit::nextchar $edi]]		 
	set bt7 [new button $fpath2 -text "hor. invert" -command [list fontedit::invertlr $edi]]
	set bt8 [new button $fpath2 -text "vert. invert" -command [list fontedit::invertud $edi]]
	set bt9 [new entry $fpath2 -textvariable fontedit::($edi,char)]
	
	bind  $widget::($bt9,path) <Return> [list fontedit::getchar $edi]

	grid $widget::($bt2,path)
	grid   $widget::($bt1,path) $widget::($bt0,path) 
	grid   $widget::($bt3,path) $widget::($bt4,path) 
	grid   $widget::($bt5,path) $widget::($bt6,path) 
	grid   $widget::($bt7,path) $widget::($bt8,path)
	grid   $widget::($bt7,path) $widget::($bt8,path)
	grid   $widget::($bt9,path) -
	
	#set fpath3 [sesh  $widget::($this,path).thesesh -height 100 -htline 100 -draw_cursor 0]
	set sesh [new sesh $labelpath -height 100 -htline 100 -draw_cursor 0]
	set fpath3 $widget::($sesh,path)
	set ($this,preview) $sesh
 
	grid $fpath0 -
	grid $fpath1 $fpath2
	grid $fpath3 -
	composite::complete $this
	#after 100 fonteditor::updatepreview $this
    }

    proc updatepreview {this} {
	catch {preview $this}
	#after 10000 fonteditor::updatepreview $this
    }
    
    proc preview {this} {
	set s $widget::($fonteditor::($this,preview),path)
	set t [fontedit::astext $composite::($this,edi)]
	$s delete 0.0 end
	tksesh_defchar tmp $t
	$s insert insert tmp
    }
    
    proc showastext {this} {
	puts [fontedit::astext $composite::($this,edi)]
    }

    proc addmenus {this} {
	set menus $composite::($this,menu,path)
	menubutton $menus.file -text "File" -menu $menus.file.m	
	menubutton $menus.edit -text "Edit" -menu $menus.edit.m
	pack $menus.file $menus.edit -side left

	menu $menus.file.m
	menu $menus.edit.m

	$menus.file.m add command -label "load" \
	    -command [list fonteditor::loadfont $this]

	$menus.file.m add command -label "save" \
	    -command [list fonteditor::savefont $this]

	$menus.file.m add command -label "load image" \
	    -command [list fonteditor::loadimage $this]

	$menus.file.m add command -label "delete image" \
	    -command [list fonteditor::delete_image $this]

	$menus.file.m add command -label "export sign" \
	    -command [list fonteditor::export_sign $this]

	$menus.file.m add command -label "Quit" \
	    -command {destroy .}

	$menus.edit.m add command -label "cut" \
	    -accelerator "CTRL-x"\
	    -command [list fonteditor::cut $this]

	$menus.edit.m add command -label "copy" \
	    -accelerator "CTRL-c"\
	    -command [list fonteditor::copy $this]

	$menus.edit.m add command -label "paste" \
	    -accelerator "CTRL-v"\
	    -command [list fonteditor::paste $this]

    }

    proc ~fonteditor {this} {}

    proc cut {this} {
	copy $this
	fontedit::clear $composite::($this,edi)
    }

    proc copy {this} {
	clipboard clear 
	clipboard append [fontedit::astext $composite::($this,edi)]
    }

    proc paste {this} {
	set txt [selection get -selection CLIPBOARD]
	fontedit::addtext $composite::($this,edi) [lrange $txt 6 end]
    }

    proc loadfont {this} {
	set patterns {{{BZR file} {.bzr}} {Any *}}
	set name [tk_getOpenFile \
		      -defaultextension .bzr -filetypes $patterns]
	fontedit::load $composite::($this,edi) $name
    }

    proc loadimage {this} {
	set patterns {{{gif file} {.gif}} {Any *}}
	set name [tk_getOpenFile \
		      -defaultextension .gif -filetypes $patterns]
	fontedit::loadimg $composite::($this,edi) $name
    }

    proc delete_image {this} {
	fontedit::delimg $composite::($this,edi)
    }

    proc savefont {this} {
	set patterns {{{BZR file} {.bzr}} {Any *}}
	set name [tk_getSaveFile \
		      -defaultextension .bzr -filetypes $patterns]
	fontedit::save $composite::($this,edi) $name
    }

    proc options {this} {}

    proc export_sign {this} {
	set ($this,export_window) [toplevel .export_sign]
	set f [new export_sign $this .export_sign]
	set ($this,export_subwindow) $f
	pack $widget::($f,path)
    }

    proc delete_export {this} {
	delete  $fonteditor::($this,export_subwindow)
	#delete $fonteditor::($this,export_window)
	destroy $fonteditor::($this,export_window)
    }
    
    proc export {this name} {
	set txt [fontedit::astext $composite::($this,edi)]
	clipboard clear 
	clipboard append [list SGN $name $txt]
	delete_export $this
    }

    proc cancel_export {this} {
	delete_export $this
    }
}



##############################################################################
#
#  Realisation effective de l'interface utilisateur
#

set editor [new fonteditor .] 
pack $widget::($editor,path)



# Format des fontes en TCL :
# Un caractere :
# CHAR ::= "{" CODE WIDTH BB CONTOUR* "}"
# BB ::= llx lly urx ury
# CONTOUR ::= "{" TYPEC SENS POINT SEGMENTS* "}"
# TYPEC ::= "CMP" (composite)
# SENS ::= "P" | "N" | "T" | "S"
# SEGMENT ::= "{" TYPESEG POINT* "}"
#
# Un segment : {type points de controle, sauf le premier}
#
# Les codes segments sont l pour ligne et s pour spline
# les codes contours sont p pour positif n pour negatif
# c pour les lignes de contruction. A noter qu'il est possible de stocker dans une fonte 
# un ensemble de lignes de contruction.
# Les possibilites :
# 
# bzr::new => nouvelle fonte
# bzr::get fonte char
# bzr::set fonte char
# bzr::load fonte nom (charge une fonte nommee nom dans la fonte fonte)
# bzr::save fonte nom (le contraire)
