
package provide seshDB 1.0
package require sqlite

# An object of this class is an iterator for an SQL query.

class DBIterator {
    # db : the database file to use
    # sql : the sql query (DON'T end it with a ";")
    # bufferSize : the number of items to get.
    proc DBIterator {this db sql {bufferSize 1}} {
	set ($this,db) $db
	set ($this,sql) $sql
	set ($this,bufferSize) $bufferSize
	# min and max : represent the part of the query which is loaded.
	# all query result with indexes i such that min <= i < max are loaded.
	set ($this,min) -1
	set ($this,max) -1
	# the part of the query which should be retrieved
	set ($this,i) 0
	set ($this,direction) 1
    }

    proc ~DBIterator {this} {
    }
    

    # preloads at most bufferSize elements, 
    # including the current one
    proc preload {this} {
	# sets loading limits, depending of 
	# the way we are browsing the result set :

	if {$($this,direction) == 1} {
	    set offset $($this,i)
	    set ($this,min) $offset
	    set ($this,max) [expr $($this,i) + $($this,bufferSize)]
	} else {
	    set offset [expr $($this,i) - $($this,bufferSize) - 1]
	    if {$offset < 0} {set offset 0}
	    set ($this,min) $offset
	    set ($this,max) [expr $($this,i) + 1]
	}
	set size [expr $($this,max) - $($this,min)]
	set result [list]
	$($this,db) eval "$($this,sql) limit $size offset $offset" values {
	    lappend result [array get values]
	}
	set ($this,buffer) $result
	set ($this,max) [expr $offset + [llength $result]]
    }

    # PRIVATE :
    proc isLoaded {this} {
	# Note that if buffer is empty, max = min and isLoaded returns 0
	if {$($this,i) >= $($this,min) && $($this,i) < $($this,max)} {
	    return 1
	} else {
	    return 0
	}
    }

    # PRIVATE :
    proc loadIfNeeded {this} {
	if {! [isLoaded $this]} {
	    preload $this
	}
    }

    proc hasValue {this} {
	loadIfNeeded $this
	if {[isLoaded $this]} {
	    return 1
	} else {
	    return 0
	}
    }

    proc getValue {this} {
	loadIfNeeded $this
	return [lindex $($this,buffer) [expr $($this,i) - $($this,min)]]
    }

    proc next {this} {
	incr ($this,i)
	set ($this,direction) 1
    }

    proc previous {this} {
	incr ($this,i) -1
	set ($this,direction) -1
    }
}