# objectID.tcl --
#
#       utility functions for object numbering.
#	basically, if a given type is built by numbering objects,
#	it should contain a table for numbering. Then
#	when creating a new object, a simple 
#		set num [objectID::getNewID MYIDVAR]
#	will get a free object number
#	and, when the object is deleted, a 
#		objectID::freeID MYIDVAR $num
#	will free $num for further reuse.
#

#
# 
#  2000 Serge Rosmorduc,
#
#
# Tout droits rservs.
# 
# voyez le fichier ../LICENCE.txt pour plus d'informations
# See the ../LICENCEE.txt file for more information
#
# $Id$
#
#

#
# $Log$
#

# FUNCTION SUMMARY --

# known public functions are : 
# 	initID <IDVAR>
#	getNewID <IDVAR>
# 	freeID <IDVAR> <ID>

# DATA SUMMARY -- 

package provide objectID 0.0

# package require bidule

namespace eval objectID {
    variable VARS
    
    # objectID::initID --
    # 	initialize an ID variable
    # Parameters:
    #	IDVAR : name of the ID variable to use. It will be used as an array
    # Options:
    # Results:
    # Side Effects:
    #	the IDVAR is initialized
    # Use:
    #	PUBLIC
    # Called by:
    
    proc initID {IDVAR} {
	upvar $IDVAR table
	set table(freed) [list]
	set table(maxUsed) -1
    }
    
    # objectID::getNewID --
    # 	gets a new ID value for a given VARID.
    # Parameters:
    #	IDVAR : name of the ID variable to use.
    # Options:
    # Results:
    #	the next free ID available from VARID
    # Side Effects:
    #	IDVAR is updated
    # Use:
    #	PUBLIC
    # Called by:

    proc getNewID {IDVAR} {
	upvar $IDVAR table
	set l $table(freed)
	if {[llength $l] != 0} {
	    set result [lindex $l 0]
	    set table(freed) [lrange $l 1 end]
	} else {
	    incr table(maxUsed)
	    set result $table(maxUsed)
	}
	return $result
    }

    # objectID::freeID --
    # 	frees a ID
    # Parameters:
    #	IDVAR : name of the ID variable to use.
    #	id : the id to free
    # Options:
    # Results:
    # Side Effects:
    #	IDVAR is updated. num is put back in the list of available IDs
    # Use:
    #	PUBLIC
    # Called by:

    proc freeID {IDVAR id} {
	upvar $IDVAR table
	lappend table(freed) $id
    }
}
