# ortho.tcl --
#
#        orthographic form save and restore in database
#

#
# 
#  1999 Serge Rosmorduc,
# quipe EC.art, Laboratoire LRIA, Universit de Paris 8
#
# 15 rue Catulienne
# 93236 St Denis Cedex
# FRANCE
#
# Tout droits rservs.
# 
# voyez le fichier ../LICENCE.txt pour plus d'informations
# See the ../LICENCEE.txt file for more information
#
# $Id$
#
#

#
# $Log$
#

# FUNCTION SUMMARY --

# known public functions are : 
# 	
# DATA SUMMARY -- 
#	 VARS(ortho_hook) : hook for storing word orthography

package provide ortho 0.0

package require seshnormalize
package require seshbase


namespace eval ortho {
    variable VARS

    # ortho::write --
    # 	saves various new orthographies for trans
    #	THE INDEX DATABASE MUST BE OPENED
    # Parameters:
    # 	trans : a translitteration
    #	orths : various orthographies in manuel de codage
    # Options:
    # Results:
    # Side Effects:
    # 	orthographies are added to already known saved orthographies
    # Use:
    #	PUBLIC
    # Called by:

    proc write {trans orths} {
	variable VARS
	#openb idx
	set normtrans [seshnormalize::norm_translit $trans]
	seshbase::lwrite idx [list ORTHO $normtrans] $orths
	if [info exists VARS(ortho_hook)] {
	    $VARS(ortho_hook) $trans $orths
	}
	#closeb idx
    }

    # ortho::read --
    # 	reads orthographies for a word.
    # Parameters:
    # 	txt : the word translitteration
    # Options:
    # Results:
    # 	returns all possible written form for the given translitteration
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:

    
    proc read {txt} {
	seshbase::openb idx
	set normtxt [seshnormalize::norm_translit $txt]
	set res [seshbase::lread idx [list ORTHO $normtxt]]
	seshbase::closeb idx
	return $res
    }

    # set_hook --
    #   defines a hook function that will be called 
    #   when some new orthography is saved in the dictionnary.
    #	This can be used to update a cache of already known orthographies.
    #
    # args --
    #    hook : a function that takes two arguments
    #           trans  : a translitteration
    #           val    : new orthography value
    
    proc set_hook {hook} {
	variable VARS
	set VARS(ortho_hook) $hook
    }
}


