# seshtext.tcl --
#
#        finding and saving hieroglyphic text file 
#        from readable references.
#        SUBJECT TO CHANGE.
#
#  1999 Serge Rosmorduc,
# quipe EC.art, Laboratoire LRIA, Universit de Paris 8
#
# 15 rue Catulienne
# 93236 St Denis Cedex
# FRANCE
#
# Tout droits rservs.
# 
# voyez le fichier ../LICENCE.txt pour plus d'informations
# See the ../LICENCEE.txt file for more information
#
# $Id: seshtext.tcl,v 1.3 2000/10/31 17:19:29 rosmord Exp $
#
#

#
# $Log: seshtext.tcl,v $
# Revision 1.3  2000/10/31 17:19:29  rosmord
# Mac Adaptation
#
# Revision 1.2  2000/08/16 19:59:00  rosmord
# Changed from a directory based system to a database based system
#
#

# FUNCTION SUMMARY --

# known public functions are : 
# save --
# get --
# delete --
# directory -- get the list of available texts.
# 
# 	
# DATA SUMMARY -- 

package provide seshtext 1.0
package require seshuser
package require seshutils
package require tag

namespace eval seshtext {

    # seshtext::save --
    # 	saves a text into the textual database
    # Parameters:
    #	name: name of the text
    #   txt: MDC representation of the text
    #	ext: optional extension. hie is for the text itself, lst for
    #        the list of the text's hieroglyphs
    # Options:
    # Results:
    # Side Effects:
    #	the text is saved in the database
    # Use:
    #	PUBLIC
    # Called by:

    proc save {name txt {ext hie}} {
	global env
	seshbase::openb texts
	seshbase::writeb texts [tag::build $ext $name] $txt
	seshbase::closeb texts 
    }

    # seshtext::get --
    # 	loads a text from the textual database
    # Parameters:
    #	name: name of the text
    #	ext: optional extension. hie is for the text itself, lst for
    #        the list of the text's hieroglyphs
    # Options:
    # Results:
    #	the loaded text
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:

    proc get {name {ext hie}} {
	global env
	# getfrombase --
	# gets a text from the textual database,
	seshbase::openb texts
	set txt [seshbase::readb texts [tag::build $ext $name]]
	seshbase::closeb texts 	
	return $txt
    }

    # seshtext::delete --
    # 	deletes a text from the textual database
    # Parameters:
    #	name: name of the text
    # Options:
    # Results:
    #	none
    # Side Effects:
    #	the named text is destroyed from the user base, along with the corresponding
    #	extensions.
    # Use:
    #	PUBLIC
    # Called by:

    proc delete {name} {
	global env
	# getfrombase --
	# gets a text from the textual database,
	seshbase::openb texts
	seshbase::delete texts [tag::build hie $name]
	seshbase::delete texts [tag::build lst $name]
	seshbase::closeb texts 	
    }


    # seshtext::directory --
    # 	deletes a text from the textual database
    # Parameters:
    #	ext: kind of texts to look for (hie, lst)
    # Options:
    # Results: the list of available texts.
    #	the
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    
    # List all texts in a database.
    proc directory {{ext hie}} {
	set f [file join [seshuser::getvar PATH] texts]
	set db [open_db $f]
	set i [get_db_idx $db lst]
	# puts "first elt : $i [getfrom_db_idx -key $i]"
	close_db $db

	set res {}
	set scan [seshbase::open_scan $ext 0 texts]

	while 1 {
	    set l [seshbase::scanget $scan 10]
	    if {[llength $l] == 0} break;
	    set res [concat $res $l]
	}
	seshbase::close_scan $scan
	return $res
    }
    

    # seshtext::private --
    # 	Says if the text belong to the user
    # Parameters:
    #	name : name of the text
    #	ext: kind of texts to look for (hie, lst)
    # Options:
    # Results: 
    #	true if the text is private, false if it belongs
    #	to the PUBLIC database
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    #
    # A text is private if it doesn't exist yet
    # or if it is in a private directory
    #
    # NOTE :
    #   DOESN'T WORK YET
    # 	in fact, we should move this in the database stuff.
    #
    proc private {name {ext hie}} {
	global env
	set base [normalize $name $ext]
	set name1 [file join $env(SESHLIB) Textes $base]
	set name2 [file join [seshuser::getvar PATH] $base]
	if [file exists $name2] {
	    return 1
	}
	if [file exists $name1] {
	    return 0
	}
	return 1
    }

    # seshtext::exists --
    # 	Says if the text exists
    # Parameters:
    #	name : name of the text
    #	ext: kind of texts to look for (hie, lst)
    # Options:
    # Results: 
    #	true if the text exists in the database, else false
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    #
    # NOTE :
    #   DOESN'T WORK YET
    # 	in fact, we should move this in the database stuff.
    #

    proc exists  {name {ext hie}} {
	global xenv
	set base [normalize $name $ext]
	set name1 [file join $env(SESHLIB) Textes $base]
	set name2 [file join [seshuser::getvar PATH] $base]
	if [file exists $name2] {
	    return 1
	}
	if [file exists $name1] {
	    return 1
	}
	return 0
    }

}


if 0 {
    # DEPRECATED STUFF.
    # KEPT HERE IN CASE IT'S NOT THAT DEPRECATED.
    
    # replaces a free text name by a reasonnable one :
    # i.e. 
    # replaces & by &amp;
    # replaces / and \ by &slash; and &bslash;
    # replaces accented letters by their code
    #

    variable letter_list
    set letter_list {
	 &AElig;
	 &Aacute;
	 &Acirc;
	 &Agrave;
	 &Aring;
	 &Atilde;
	 &Auml;
	 &Ccedil;
	 &ETH;
	 &Eacute;
	 &Ecirc;
	 &Egrave;
	 &Euml;
	 &Iacute;
	 &Icirc;
	 &Igrave;
	 &Iuml;
	 &Ntilde;
	 &Oacute;
	 &Ocirc;
	 &Ograve;
	 &Oslash;
	 &Otilde;
	 &Ouml;
	 &THORN;
	 &Uacute;
	 &Ucirc;
	 &Ugrave;
	 &Uuml;
	 &Yacute;
	 &aacute;
	 &acirc;
	 &aelig;
	 &agrave;
	 &aring;
	 &atilde;
	 &auml;
	 &ccedil;
	 &eacute;
	 &ecirc;
	 &egrave;
	 &eth;
	 &euml;
	 &iacute;
	 &icirc;
	 &igrave;
	 &iuml;
	 &ntilde;
	 &oacute;
	 &ocirc;
	 &ograve;
	 &oslash;
	 &otilde;
	 &ouml;
	 &szlig;
	 &thorn;
	 &uacute;
	 &ucirc;
	 &ugrave;
	 &uuml;
	 &yacute;
	 &yuml;
    }

    proc normalize {name {ext hie}} {
	variable letter_list
	regsub -all {[ 	][ 	]*} $name { } name
	regsub -all {&} $name {\&amp;} name
	regsub -all {\.} $name {\&stop;} name
	regsub -all {/} $name {\&slash;} name
	regsub -all {\\} $name {\&bslash;} name
	foreach {l code} $letter_list {
	    regsub -all $l $name "\\$code" name
	}
	return $name.$ext
    }
    
    proc getfilename {name {ext hie}} {
	return [normalize $name $ext]
    }

    proc unnormalize {name} {
	variable letter_list
	set name [file rootname [file tail $name]]
	#set name [file rootname $name]
	# anything which 
	foreach {l code} $letter_list {
	    regsub -all $code $name "$l" name
	}       
	regsub -all {&slash;} $name {/} name
	regsub -all {&bslash;} $name \\ name
	regsub -all {&stop;} $name {.} name
	regsub -all {&amp;} $name {\&} name
	return $name
    }

    # OLD DIRECTORY STUFF
    
	global env
	set b1 ""
	set b2 ""
	catch {	set b1 [glob [file join $env(SESHLIB) Textes *.$ext]]}
	catch {set b2 [glob [file join [seshuser::getvar PATH] *.$ext]]}
	set b [concat $b1 $b2]
	set res {}
	foreach i $b {
	    lappend res [unnormalize $i]
	}
	set res [uniq $res]
	set res [lsort -dictionary $res]
	return $res
	
}
