# seshuser.tcl --
#
#        basic information on user
#        SUBJECT TO CHANGE.
#
# Copyright (c) 1998 Serge Rosmorduc
#

# THE NEW VERSION OF seshuser :
# some user data will be saved in the base. 
# You will be able to order the users in order to use their 
# additions to the database.

package provide seshuser 0.0

package require legendes
#package require fileentry

namespace eval seshuser {
    variable VARS

    namespace export getvar setvar
    namespace export dialogue loaduser
    namespace export cancel
    
    proc getvar {what} {
	variable VARS
	if [info exists VARS($what)] {
	    return $VARS($what) 
	} else {
	    return {}
	}
    }

    proc getUID {} {
	return [getvar ID]
    }

    proc setvar {what val} {
	variable VARS
	set VARS($what) $val
    }

    proc dialogue {} {
	variable VARS
	set w .userd
	toplevel $w
	wm title $w "User Information"
	label $w.expl -text [legendes::trans "Please enter your name, email, and ID below. \nTo get an ID and a password, please mail \n rosmord@iut.univ-paris8.fr. To try tksesh without ID, enter ID 1234567890"]
	#text $w.expl -width 50 -height 4
	#$w.expl insert insert 
	#$w.expl configure -state disabled
	label $w.name -text [legendes::trans "User Name"]
	entry $w.name_e
	label $w.code -text [legendes::trans "User ID"]
	entry $w.code_e
	label $w.email -text [legendes::trans "email"]
	entry $w.email_e
	label $w.lang -text [legendes::trans "language"]
	
	menubutton $w.lang_e -menu $w.lang_e.m -textvariable seshuser::VARS(language)\
	    -relief sunken -width 19 -font [$w.code_e cget -font] -justify left\
	    -takefocus 1 -highlightthickness [$w.code_e cget -highlightthickness]
	
	menu $w.lang_e.m -tearoff 0 
	
	foreach lang [legendes::listlanguages] {
	    $w.lang_e.m add radiobutton -label $lang -variable seshuser::VARS(language)
	}
	
	label $w.path -text [legendes::trans "texts directory"]
	entry $w.path_e
	#fileentry::create $w.path_e -type save
	button $w.ok -text  [legendes::trans "ok"] -command seshuser::save_info 
	button $w.cancel -text  [legendes::trans "cancel"] -command [list seshuser::cancel $w]
	catch {
	    $w.name_e insert insert [getvar NAME]
	    $w.code_e insert insert [getvar ID]
	    $w.email_e insert insert [getvar MAIL]
	    set VARS(language) [getvar LANG]
	    if {[string length $VARS(language)] == 0} {
		set VARS(language) "english"
	    }
	}

	if {[string compare [getvar PATH] {}] == 0} {
	    global env
	    #fileentry::settext $w.path_e [file join $env(HOME) Textes]
	    $w.path_e insert insert [file join $env(HOME) Textes]
	} else {
	    #fileentry::settext $w.path_e [getvar PATH]
	    $w.path_e insert insert [getvar PATH]
	}
	grid $w.expl -
	grid $w.name $w.name_e
	grid $w.code $w.code_e	
	grid $w.email $w.email_e
	grid $w.lang $w.lang_e
	grid $w.path $w.path_e
	grid $w.ok $w.cancel
	tkwait window $w
    }

    proc cancel {w} {
	variable VARS
	set lang $VARS(language)
	setvar LANG $lang
	catch {legendes::set$lang}
	destroy $w
    }

    proc save_info {} {
	variable VARS
	
	set w .userd
	set nom [$w.name_e get] 
	set id [$w.code_e get] 
	set mail [$w.email_e get] 
	#set path [fileentry::gettext $w.path_e]
	set path [$w.path_e get] 
	
	set lang $VARS(language)
	setvar LANG $lang
	catch {legendes::set$lang}
	
	if {[string compare $nom ""] != 0 && 
	    [string compare $id ""] != 0} {
	    setvar ID $id
	    setvar NAME $nom
	    setvar MAIL $mail
	    setvar PATH $path
	    if [ensure_exists $path] {
		save_user
		destroy $w
	    }
	}
    }

    proc loaduser {} {
	set fname [file join $::env(HOME) .seshrc]
	if [file exists $fname] {
	    setvar LANG english
	    source $fname
	    legendes::set[getvar LANG]
	    return 1
	} else {
	    return 0
	}
    }

    proc save_user {} {
	variable VARS
	set fname [file join $::env(HOME) .seshrc]
	if [info exists VARS(ID) ] {
	    set f [open $fname w]
	    puts $f [list ::seshuser::setvar NAME $VARS(NAME)]
	    puts $f [list ::seshuser::setvar ID $VARS(ID)]
	    puts $f [list ::seshuser::setvar MAIL $VARS(MAIL)]
	    puts $f [list ::seshuser::setvar PATH $VARS(PATH)]
	    puts $f [list ::seshuser::setvar LANG $VARS(LANG)]
	    close $f
	}
    }
    
    proc ensure_exists {path} {
	if { ! [file exists $path] } {
	    set q [tk_dialog .dialog "confirmation" "directory $path does not exist.\n Do you want to create it ?" "" 0 "Yes" "No"]
	    if {$q == 0} {
		file mkdir $path
	    } else {
		return 0
	    }
	} else {
	    if { ! [file isdirectory $path] } {
		tk_dialog .dialog "warning" "$path is not a directory.\n You should perhaps choose another name." "" "OK"
		return 0
	    }
	}
	return 1
    }
}


