# doctype.tcl --
#
#
# Copyright (c) 2000 Serge Rosmorduc
#

#
# 
#  1998,2000 Serge Rosmorduc,
# quipe EC.art, Laboratoire LRIA, Universit de Paris 8
#
# 15 rue Catulienne
# 93236 St Denis Cedex
# FRANCE
#
# Tout droits rservs.
# 
# voyez le fichier ../LICENCE.txt pour plus d'informations
# See the ../LICENCEE.txt file for more information
#
# $Id$
#
#

#
# $Log$
#

#
# Internal data representation
# information about doctypes is stored
# in the VARS variable, in the form :
# VARS(DOCTYPENAME,TAGNAME,FIELDNAME)
#
# TAG grammar
# The possible values for a tag can be :
#
# "choiceof" tagname ?tagname....?
#	if the tag can contain a list (possibly empty) of other tags
#	in arbitrary order
#
# empty
#	if the tag value is always empty
#
# pcdata
#	if the tag value is made of characters, without tags.
#
# sequence tagname ?tagname ...?
# 	if the tag contains a fixed list of other tags
#
# Very rough example :
# 	chapter		{seq title chaptercontent}
# 	title		pcdata
# 	chaptercontent	{choiceof section text}
# 	text		pcdata
#
# This is much more limited than XML, but it is possible with this
# system to ensure that any element created is grammatical.
# 

# Internal rep :
# VARS($type,$tag,kind) : kind of tag 
# VARS($type,$tag,content) : content grammar
# VARS($type,$tag,opt,$opt) : option value
package provide doctype 0.0

package require myopts 0.0

namespace eval doctype {
    variable VARS
    
    # doctype::create --
    # 	create a new document type
    # Parameters:
    #	name	: name of the document type
    # Options:
    # Results:
    # Side Effects:
    #	a new document type is created
    # Use:
    #	PUBLIC
    # Called by:
    
    proc create {name} {
    }

    # doctype::declare --
    # 	declares a new tag in a document type
    # Parameters:
    #	name	: name of the document type
    #	tag	: name of the tag
    #	content : grammar of the tag
    # Options:
    # 	-index	: should the tag value be saved in an index ?
    #	-description : short description the tag meaning. Used in menus, etc...
    #	-normalize : normalization function. Used for indexes. Takes the tag content as argument, and sends back a
    #		normalized form. For instance, for transliteration, transform "j" into "i" and the like.
    # Results:
    # Side Effects:
    #	a new document type is created
    # Use:
    #	PUBLIC
    # Called by:

    proc declare {name tag content args} {
	variable VARS
	# Grammar
	set VARS($name,$tag,kind) [lindex $content 0]
	set gramm [lindex $content 1]
	if {[string compare [lindex $gramm 0] "pcdata"] == 0} {
	    set VARS($name,$tag,hasData) 1
	    set gramm [lrange $gramm 1 end]
	} else {
	    set VARS($name,$tag,hasData) 0
	}
	set VARS($name,$tag,content) $gramm
	# Manages options :
	myopts::myopts opts { 
	    -index 0
	    -description {}
	    -normalize	identity
	} $args
        foreach {o v} [array get opts] {
	    set VARS($name,$tag,opt,$o) $v
	}
	
    }

    # doctype::get --
    # 	gets the value of some data associated with a tag option
    # Parameters:
    #	name	: name of the document type
    #	tag	: name of the tag
    #	opt	: option name (without "-")
    # Options:
    # Results:
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:
    
    proc get {doc tag opt} {
	variable VARS
	return $VARS($doc,$tag,opt,-$opt)
    }

    # doctype::getChoice --
    # 	returns the list of possible tags in a choiceof tag
    # Parameters:
    #	name	: name of the document type
    #	tag	: name of the tag
    # Options:
    # Results:
    #	list of possible tags in a choiceof tag
    # Side Effects:
    # Use:
    #	PUBLIC
    # Called by:

    proc getChoice {doc tag} {
	variable VARS
	return $VARS($doc,$tag,content)
    }

    proc getKind {doc tag} {
	variable VARS
	return $VARS($doc,$tag,kind)
    }

    proc hasData {doc tag} {
	variable VARS
	return $VARS($doc,$tag,hasData)	
    }

    proc identity {x} {
	return $x
    }

    proc tag_normalize {doc tag opts value} {
	set v [[get $doc $tag normalize] $value]
	return [list $tag $opts $v]
    }

}

# SKELETON FUNCTION
    # seshbase::getFileName --
    # 	get the path to the file associated with a base
    #   either in public or private mode
    # Parameters:
    #	base: name of the base
    #   own: owner of the base
    # Options:
    # Results:
    # 	the path to the file
    # Side Effects:
    # Use:
    #	PRIVATE
    # Called by:
