#!/bin/sh

set env(SESHLIB) /home/rosmord/tksesh/tcllib
set SESHLIB $env(SESHLIB)
lappend auto_path $env(SESHLIB) .

#for unix
set env(ldext) so
set env(DEFAULTSELECTION) CLIPBOARD
event add <<Paste>> <Control-y>
event add <<Paste>> <Button-2>

# FILES :

set FILES [list stext.tcl doctype.tcl seshdoc.tcl myopts.tcl]

# eval pkg_mkIndex -direct . $FILES 

package require stext
package require doctype
package require seshdoc
package require seshw
package require legendes

# Document Declaration

doctype::create book
doctype::declare book book {seq {title content}} -index 1
doctype::declare book title {choiceof pcdata} -description {titre}
doctype::declare book content {choiceof {chapter}} -description {contenu}
doctype::declare book chapter {seq {title text}} -description chapitre
doctype::declare book text {choiceof {pcdata it hiero chapter}} -description texte
doctype::declare book it {choiceof {pcdata chapter}} -description italique
doctype::declare book hiero {choiceof {pcdata}} -description hieroglyphes  -normalize norm_hiero

# Associated format declaration :

stext::createFormat book
stext::declare book book
stext::declare book title \
	-conf {-font {Helvetica 24} -justify center} \
	-beginconf {-background green} -begin [list plainInsert "\n"] \
	-endconf   {-background green -foreground yellow} -end   [list plainInsert "\n"]
stext::declare book content
stext::declare book chapter -conf {-background white} -begin [list plainInsert "Chapitre"]
stext::declare book text -init plaintext
stext::declare book plaintext -conf {-font {Times 12}}
stext::declare book it -conf {-font {Times 12 italic}}
stext::declare book hiero -save save_hiero -create insert_hieroglyphic_tag

# -create : 
# 	create is used for creating an empty new tag and to insert a brand new tag
#	the function receives window, tagType and index as arguments
#	(this allow one to use the same function for a number of tag types)

# -load :

# -save :
#	


proc insert_hieroglyphic_tag {w tagtype index {txt ""}} {
    set n [stext::createSubWindow $w $tagtype $index]
    set s [stext::getField $w $n windowName]
    ::seshw::create_passive $s
    stext::insertSubWindow $w $n
    if { $txt != "" } {
	::seshw::set_text $s $txt
	::seshw::autosize $s
    }
    bind $s <Double-Button-1> [list edit_hieroglyphs $w $s]
}

proc edit_hieroglyphs {w subwin} {
    set ed .dicohieredit
    toplevel $ed
    wm title $ed [legendes::trans "hieroglyphic word editor"]
    seshw::create_editor $ed.s -notitle
    grid $ed.s -
    $ed.s.s insert 0.0 [$subwin get 0.0 end]
    button $ed.ok -text [legendes::trans "Ok"]\
	    -command [list sethierofromedit $subwin $ed]
    button $ed.cancel -text [legendes::trans "Cancel"]\
	    -command [list destroy $ed]
    grid $ed.ok $ed.cancel
}
    
    
proc sethierofromedit {s orig} {
      	seshw::set_text $s [$orig.s.s get 0.0 end]
      	seshw::autosize $s
      	destroy $orig
}

set w1 [stext::create .w book]
pack .w
