/* lecture d'un mot  */
:- consult('torep.pl').
:- consult('lectures_pos.pl').
:- consult('regles_groupe.pl').
:- consult('listes.pl').
:- consult('determinatifs.pl').
:- consult('transposition_graphique.pl').
:- consult('mdc2liste').

/* TEST POUR SAVOIR SI UNE LISTE NUMEROTEE DE HIEROGLYPHES N'EST CONSTITUEE QUE DE DETERMINATIFS */

tout(Txt, Res):-
	les_mots(Txt, T1),
	tout_aux(T1, Res).
tout_aux([],[]).

tout_aux([A|B],[A1|B1]):- write(A), write(' => '), run(A,A1), !, write(A1), nl, tout_aux(B,B1).
tout_aux([A|B],[A|B1]):- write(A), nl, tout_aux(B,B1).

run(X,RES):- mdc2liste(X, X0), decode_liste_hieros(X0,L), cherche_mot(L,RES), RES \= ''.

determinatif(A/_) :- lectures_possibles(A, L), membre([], L).
liste_de_determinatifs([]).
liste_de_determinatifs([A | B]) :-
                 determinatif(A),
                 liste_de_determinatifs(B).

liste_de_determinatifs([A | B]) :-
                 (determinatif(A); determinatif_rare(A)),
                 liste_de_determinatifs(B),
		 longueur([A | B], N), 
		 (N =< 3).

/*PROCEDURE DE RECHERCHE DE TOUTES LES TRANSLITTERATIONS POSSIBLES */

/* premier cas : le mot contient un determinatif significatif :
   On teste si les lectures sont compatibles, et, si oui, on lit le mot
   de cette manire*/

/* second cas */
cherche_mot(L0,Ma):- % L0  mot en hieros, Ma mot en translitteration 
        
	ajouter_debut_fin(L0,L),
	regles_utilisables(L,REGLES),
	            /*REGLES = liste des regles utilisables sur le mot L*/
        
        tri_regles(REGLES,Rs),
                     /* Rs = liste des regles utilisables, rangees par
                                            ordre de pertinence */
	
        liste_numerotee(L,Ln),    
	appliquer_les_regles(Ln,Rs,M), 
           /*M est deductible de L a partir D'UNE PARTIE de Rs */
  
        trie_translitteration(M,Ms),
	 /* remet les lettres de la translitteration dans l'ordre */
        arrange_translitteration(Ms,Ma).

/* traite les eventuelles transpositions graphiques */

cherche_mot(H,T):- 
	conc(H1, H2, H), conc(L, Ha, H2),
	transposition(L, Lt),  
	conc(Lt, Ha, H2t), conc(H1, H2t, Ht),
	cherche_mot(Ht, T).
   



/* CREATION DE L'ENSEMBLE DES REGLES UTILISABLES */
regles_utilisables(L,Rs):-
	bagof(R,regle_utilisable(L,R),Rs).

regle_utilisable(L,[S,N,C,P]):- 
/* la regle [S,N,C,P] est utilisable sur le mot L en hieros */
   sousliste(S,L), S \= [],
   regle_groupe(S,N,C,P), C.

/* PROCEDURE DE TRI DES REGLES PAR ORDRE DE PERTINENCE */
tri_regles(Rs,Rs1):-
	predsort(meilleure_regle,Rs,Rs1).

/* criteres de pertinence */

/* critere 1 : la regle  correspond a un groupe de hieros precis
 -> la regle correspondante est prioritaire */ 

/* meilleure_regle([_,_,_,0], [_,_,_,N]):- N\= 0, !. */

/* critere 2 : Le nombre de hieros intervenant dans la 1ere regle est plus grand que celui des hieros intervenant dans la 2eme regle -> la 1ere regle est prioritaire */


meilleure_regle([_,_,_,V1], [_,_,_,V2]):-
	V1 < V2.



/* meilleure_regle([L1,_,_,_], [L2,_,_,_]):-
	length(L1,N1), length(L2,N2),
	N1 > N2.
*/

meilleure_regle([L1,_,_,V1], [L2,_,_,V1]):-
	length(L1,N1), length(L2,N2),
	N1 > N2.




/* APPLICATION DES REGLES */
 

appliquer_les_regles(L,[[S,N,_,_] | Rs],M):-
/* L est une liste numerotee, mais pas S, ni N */
	prive_num(L,S,L2), /* prive L de S, donne S2 */
	sousliste_num(S,L,P), Nn = [N/P], 
	conc(Nn,M2,M), 
	appliquer_les_regles(L2,Rs,M2).

appliquer_les_regles(L,[_| Rs],M):-
	appliquer_les_regles(L,Rs,M).
 
appliquer_les_regles(L,[],[]):- 
	liste_de_determinatifs(L),
	fin_de_mot(L).        



/* PROCEDURE DE TRI DES LETTRES DE LA TRANSLITTERATION */   
trie_translitteration(M,Ms):-
      predsort(plus_petit_numero,M,Ms). 

/* critere de tri : les numeros associes a chaque groupe */ 
plus_petit_numero(_/X1,_/X2):-
	X1 < X2.

/* Ajout de marques de debut et de fin au mot : */

ajouter_debut_fin(L,[debut|L1]):- ajouter_debut_fin_aux(L,L1).
ajouter_debut_fin_aux([],[fin]).
ajouter_debut_fin_aux([A|R], [A|R1]):- ajouter_debut_fin_aux(R,R1).


/* LISIBILITE DU RESULTAT */
arrange_translitteration(M,Ma) :-
	supprime_numeros(M,M1),
	linearisation(M1,M2),
	mot(M2,Ma),
	mot_non_vide(Ma).

/* suppression des numeros */
 
supprime_numeros([],[]).
supprime_numeros([X/_ | M], [X | Ma]) :- supprime_numeros(M,Ma).
  
/* linearisation */

linearisation([],[]).
linearisation([A|R], R2):- 
	linearisation(R,R1),
	append(A,R1,R2).


/* mise sous forme de mot */

mot([],''):- !.
mot([X | L], M) :-
	name(X,[Ax]),
	mot(L, M1),
	name(M1, Al),
	name(M,[Ax | Al]).

/*suppression des mots vides */
mot_non_vide(M) :- M\= ' '.


/* PROCEDURE POUR VERIFIER QU'UNE LISTE EST EN FIN DE MOT */

fin_de_mot([debut/_|R]):- !, fin_de_mot(R).

fin_de_mot([fin/_]).

fin_de_mot([]). 

fin_de_mot([_/N | [fin/P]]) :-
P is N + 1.


fin_de_mot([_/N | [Y/P | L]]) :-
 fin_de_mot([Y/P | L]),
 P is N + 1.

