:- consult('traitements.pl').

%
% Recherche en largeur.
% Les rsultats intermdiaires sont stocks dans une structure 
% trie grace a l'operation transl_inserer()
% La meilleure solution est recuperable par transl_meilleure()
%
%
% Pour chaque hypothse : on stocke : 
% no_hypothese l'Hypothse/EXTENSIONS/COUT_MEILLEURE_EXTENSION
% - tmp_hypothese(NO, COUT)
% - tmp_meilleure(NO) %% Voir si on peut s'en passer
% - tmp_suivante(NO_a,NO_b) : l'hypothese a suit l'hypothese b
% - on a une pseudo hypothese de depart : first, qui vaut -1, et a donc 
%   une meilleure valeur que toute autre

:- dynamic tmp_hypothese/2, tmp_suivante/2.

tmp_nettoyer:- 
	retractall(tmp_nbr(_)),	
	assert(tmp_nbr(0)),
	retractall(tmp_suivante(_,_)),
	retractall(tmp_hypothese(_,_)).

transl_inserer(H):- 
	gensym(n, Nouv),
	assert(tmp_hypothese(Nouv,H)),
	tmp_nbr(Num),
	Num0 is Num + 1,
	retractall(tmp_nbr(_)),
	assert(tmp_nbr(Num0)),	
	transl_inserer_aux(H/Nouv,first).

transl_inserer_aux(H/C/Nouv,N):-
	tmp_suivante(N,N0) -> 
	(
	    tmp_hypothese(N0,_/C0),
	    (
		C < C0 -> 
		(
		    assert(tmp_suivante(N,Nouv)),
		    assert(tmp_suivante(Nouv,N0)),
		    retract(tmp_suivante(N,N0))
		);
		transl_inserer_aux(H/C/Nouv, N0)
	    )
	) ;
	assert(tmp_suivante(N,Nouv)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% calculer_extensions --
%%	ETAT +
%%      REGLES +
%%      EXTENSIONS -
%%
%% EXTENSIONS est la liste des etats correspondant
%%  des extensions possibles de ETAT par REGLES.
%%
%% ETAT= TEXTE_A_TRAITER/VALEURS_TROUVEE/NUMERO_MIN_DE_REGLE/COUT
%% La premire extension est celle de plus bas prix.

calculer_extension(TXT/V/MINR/C, 
	REGLES,
	EXT):-
	setof(TXT1/V1/Num/C1, R^Num^(
	member(R/Num, REGLES), 
	Num > MINR,
	appliquer_une_regle(TXT, R, V, C, TXT1, V1, C1)),
	EXTENSIONS), !, 
	predsort(plus_petit_numero,EXTENSIONS,EXT).

calculer_extension(_,_,[]).

%% inserer_extensions(EXTENSIONS, H, H1)
%inserer_une_extension(E,[], [E]).
%inserer_une_extension(X/C1,[Y/C2|R], [X/C1,Y/C2|R]):- C1 =< C2.
%inserer_une_extension(X/C1,[Y/C2|R], [Y/C2|R1]):- 
%	C1 > C2,
%	inserer_une_extension(X/C1,R,R1).


etendre_tous([],_,[],_).
etendre_tous([TXT|RTXT],REGLES,[S1|RS],NBR):-
	tmp_nettoyer,
	etendre(TXT, REGLES, S1, NBR),
	etendre_tous(RTXT, REGLES, RS, NBR).


initialiser_recherche(TXT):-
	transl_inserer(TXT/[]/0/0/[TXT/[]/0/0]/0).

etendre(TXT, REGLES, SOLUTIONS, NBR):- 
	initialiser_recherche(TXT),
	etendre_aux(REGLES, [], SOLUTIONS, NBR).

tmp_meilleure(HYP):-
	tmp_suivante(first, N),
	tmp_hypothese(N,HYP),
	retract(tmp_hypothese(N,HYP)),
	retract(tmp_suivante(first,N)),
	(tmp_suivante(N,N0) ->
	    assert(tmp_suivante(first, N0));
	    true
	).


etendre_aux(_,SOL,SOL,NBR):-
	length(SOL,NBR).
	
etendre_aux(REGLES, SOL0, SOLF, NBR):-
	tmp_meilleure(TXT/VAL/_/_/EXTENSIONS/C) ->
	(
	    (
		TXT= [] -> 
		( 
		    trie_translitteration(VAL,TR0),
		    arrange_translitteration(TR0,TR),
		    SOL1= [TR/C|SOL0],
		    write(SOL1),nl
		) ;
		(
		    inserer_extensions(EXTENSIONS,  REGLES),
		    SOL1= SOL0
		)
	    ),
	    etendre_aux(REGLES, SOL1, SOLF, NBR)
	);
	SOLF= SOL0.

	

:- dynamic tmp_connue/1.

inserer_extensions([],_).


inserer_extensions([TXT1/V1/MINR1/C|Reste], REGLES):- 
	calculer_extension(TXT1/V1/MINR1/C,REGLES, EXT),
	(
	    EXT = [_/Cexts|_] ->
	    transl_inserer(TXT1/V1/MINR1/C/EXT/Cexts);
	    (
		TXT1 = [] ->
		transl_inserer(TXT1/V1/MINR1/C/[]/C);
		true
	    )
	),
	inserer_extensions(Reste,  REGLES).


essai1(X,RES):-
	tmp_nettoyer,
	preparer(X,Ln, REGLES),
	etendre_tous(Ln, REGLES, RES,3),
	tmp_nbr(N), write(N), nl.


trans1_aux([],[]).
trans1_aux([L|R],[L1|R1]):-
	predsort(plus_petit_numero,L, [L1/_|_]),
	trans1_aux(R,R1).

concat_atoms([], 0).
concat_atoms([A], A).
concat_atoms([A,B|Reste], Result):-
	concat(A,B,AB),
	concat_atoms([AB|Reste], Result).



trans1(SOL,Y):- essai2(Y,SOL

,6).
%	trans1_aux(RES,RES1),
%	concat_atoms(RES1,SOL).
	

preparer(X, LLn, REGLES1):- 
	mdc2liste(X, X0), 
	decode_liste_hieros(X0,L),
	ajouter_debut_fin(L,L0),
        liste_numerotee(L0,Ln),
	regles_utilisables(Ln, REGLES),
        tri_regles(REGLES,Rs),
	ilots(Ln, Rs, LLn),
	print_regles(Rs),
        length(Rs, Nregles), write(Nregles), write(' regles'), nl,
	liste_numerotee(Rs, REGLES1).


	

print_regles([]).
print_regles([[TXT,V,Cond,COUT]|Rs]):-
	write(TXT), write('=>'), write(V), write('  '), write(COUT), nl,
	print_regles(Rs).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Better solution (linear time)
%
% usable Rules should be asserted before
% build_best_trans --
% I : sign number. 
% N : Max sign number
% NTRANS : MAX number of translitterations wanted
%
%
% Algorithm :
%   the best translitterations until word I have been built.
%   hence we look for the best translitterations for word I,
%   we assert them, and we proceed to word I + 1
% 
%   if I = N we stop.
%
% database contains :
%   tmp_rule(LASTNUM, FIRSTNUM, VALUE, RULENUM)
%   tmp_link(WRD2,WRD1, VALUE, RULES_NUMBERS)
%   

:- dynamic tmp_rule/4, tmp_link/3.

nettoyer_lin :- 
	retractall(tmp_rule(_,_,_,_)),
	retractall(tmp_link(_,_,_)).

assert_rules([]).
assert_rules([[TXT, _ , _, VAL]/NUM|Rs]):-
	limites_texte(TXT, DEB, FIN),
	assert(tmp_rule(FIN, DEB, VAL, NUM)),
	assert_rules(Rs).
	

build_best_trans(N,M,_):-
	N is M + 1.

build_best_trans(I,N,MAX):-
	I =< N,
	bagof(RULES/VALUE,
	way_to(I, RULES, VALUE),
	L),
	predsort(plus_petit_numero, L, L1),
	first_n(L1,MAX,WAYS),
	assert_ways(I, WAYS),
	II is I + 1,
	build_best_trans(II, N, MAX).


first_n(_,0,[]):- !.
first_n([],_,[]).
first_n([A|R], N, [A|R1]):-
	N > 0,
	N1 is N - 1,
	first_n(R, N1, R1).

assert_ways(_, []).
assert_ways(I, [RULES/VALUE|R]):-
	assert(tmp_link(I, VALUE, RULES)),
	assert_ways(I, R).

%% find a way to I, with its value

way_to(0, [], 0).

way_to(I, WAY, VALUE):-	
	tmp_rule(I, B, V0, RULE),
	B0 is B - 1,
	tmp_link(B0, V1, RULES),
	VALUE is V0 + V1,
	WAY = [RULE|RULES].


prepare_essai2(X, Ln, REGLES1):- 
	mdc2liste(X, X0), 
	decode_liste_hieros(X0,L),
	ajouter_debut_fin(L,L0),
        liste_numerotee(L0,Ln),
	regles_utilisables(Ln, REGLES),
        tri_regles(REGLES,Rs),
	print_regles(Rs),
	liste_numerotee(Rs, REGLES1).


essai2(X, RESULT, NBR):- 
	nettoyer_lin,	
	prepare_essai2(X, Ln, REGLES), 	
	assert_rules(REGLES),
	limites_texte(Ln, _, FIN),
	build_best_trans(0,FIN, NBR),!,
	tmp_link(FIN, _, RULENUM),
	number_to_rules(RULENUM, REGLES, RULES),
	appliquer_les_regles(Ln, RULES, [], RES, 0, _),
	arrange_translitteration(RES,RESULT).
	
number_to_rules([],_, []).
number_to_rules([N|Ns], RULES, [R|Rs]):-
	member(R/N, RULES),
	number_to_rules(Ns, RULES, Rs).


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Search for a word presence in a text line
%
wordintext_a([],_).
wordintext_a(W, [DEB|REST]):-
	member(X/_, DEB),
	name(X,XL),
	conc(XL,WREST, W),
	wordintext_a(WREST, REST).

wordintext_a(W, [_|REST]):-
	wordintext_a(W,REST).

wordintext(MOT, TEXT):-
	essai1(TEXT, TXT1), !,
	name(MOT, MOTL),
	wordintext_a(MOTL, TXT1).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Fast Search : a word can be found if :
%   We meet a list of signs wich 
%      a) cover all consonnants of the word
%      b) contains only consonnants of the word and determinatives


liste_de_lettres([],[]).
liste_de_lettres([A|R],[A1|R1]):- 
	name(A1,[A]),
	liste_de_lettres(R,R1).

mot_dans_texte_aux(MOT,TXT):-
	name(MOT, M),
	liste_de_lettres(M,M1),
	mdc2liste(TXT, X0), 
	decode_liste_hieros(X0,L),
	trouve_mot(M1, L).

trouve_mot(MOT, L):-
	mot_possible(MOT, MOT, L).

trouve_mot(MOT, [_|R]):-
	trouve_mot(MOT,R).

mot_dans_texte(RES, MOT, TXT):-
	(mot_dans_texte_aux(MOT,TXT) -> RES = '1'; RES = '0'),
	write(RES), nl
	.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	mot_possible --
%	TOREAD : what still must be read
%       LETTRES : allowed letters
%       TXT : text to scan

mot_possible([], _, _).

mot_possible(TOREAD, LETTERS, [HEAD|TAIL]):-
	lectures_possibles(HEAD,VALS),
	member(VAL, VALS),
	sousliste(VAL, LETTERS),
	enleve_elements(VAL, TOREAD, REST),
	mot_possible(REST, LETTERS, TAIL).

mot_possible(TOREAD, LETTERS, [HEAD|TAIL]):-
	lectures_possibles(HEAD,VALS),
	member(VAL, VALS),
	sousliste(VAL, LETTERS),
	mot_possible(TOREAD, LETTERS, TAIL).

mot_possible(TOREAD, LETTERS, [HEAD|TAIL]):-
	lectures_possibles(HEAD,VALS),
	member('*', VALS),
	mot_possible(TOREAD, LETTERS, TAIL).


