/* %%%%%%% regles_groupe.pl %%%%%% */

/* REGLES DE LECTURE DE GROUPES DE HIEROGLYPHES */

/* Les regles ont la forme suivante :
regle_groupe(
	suite de hieroglyphes, 
	suite de caracteres en translitteration, 
	condition pour que la correspondance ait lieu, 
	nombre traduisant la priorite de la regle (0 : priorite maximale))
    */


lecture(A,X):- lectures_possibles(A, L), membre(X, L).
/* A peut se lire x */

/*##
 #
 # A la base, on organise le tout selon le nombre de signes impliqus
 #
 #*/

/*#############################################################################
                    signes doubles ou triples 
#############################################################################*/

/* cas speciaux */
/*==============*/
regle_groupe(['M17','M17'], ['y'], true, 1).
regle_groupe(['D4','D4'], ['m','A','A'], true,10).
regle_groupe(['D4','D4'], ['i','r','t',y], true,4).
regle_groupe(['F51','F51','F51'], ['H','a','w'], true ,0).
regle_groupe(['I3','I3'], ['i','t','y'], true, 1).
regle_groupe(['M22', 'M22'], ['n','n'], true, 1).

/* Lister toutes les formes de rdi */
regle_groupe([debut, 'D21', 'D36', fin], ['r','d'], true, 100).
regle_groupe([debut, 'D21', 'D36', 'X1', fin], ['r','d', 't'], true, 100).
regle_groupe([debut, 'D21', 'X1', 'D36', fin], ['r','d', 't'], true, 100).

/* signes triples */
/*================*/

regle_groupe([debut,A,A,A,fin], L, (lecture(A,L1), conc(L1,['w'],L)), 10).
regle_groupe([A,A,A,fin], L, (lecture(A,L1), conc(L1,['w'],L)), 10).
regle_groupe([debut,A,A,A], L, (lecture(A,L1), conc(L1,['w'],L)), 150).

/* signes doubles */
/*================*/

regle_groupe([A,A], L, (lecture(A,L1), conc(L1,['y'],L), longueur(L1,X), (X>=2)), 20).
regle_groupe([A,A], L, (lecture(A,L1), conc(L1,['w','y'],L), longueur(L1,X), (X>=2)), 20).


/*-----------------------------------------------------------------------------*/
/* QUADRILITERES */
/*---------------------------------------------------------------------------*/

/* SEULS */

regle_groupe([A], [W,X,Y,Z], (lecture(A, [W,X,Y,Z])), 400).

regle_groupe([MN,N,I,T, MNIT], [A,B,C,D], 
	(lecture(MN, [A,B]), lecture(N,[B]), lecture(I,[C]), lecture(T,[D]),
	lecture(MNIT,[A,B,C,D])), 1).

regle_groupe([MNIT,T], [A,B,C,D], 
	(lecture(T,[D]),
	lecture(MNIT,[A,B,C,D])), 1).

regle_groupe([M,N,I,T, MNIT], [A,B,C,D], 
	(lecture(M, [A]), lecture(N,[B]), lecture(I,[C]), lecture(T,[D]),
	lecture(MNIT,[A,B,C,D])), 1).

/*---------------------------------------------------------------------------------*/
/* UNILITERE(S) + BILITERE(S) + TRILITERE */
/*---------------------------------------------------------------------------*/

/* un bilitere et un trilitere presents */
/*=======================================*/

regle_groupe([A,B,C,D,E], [X,Y,Z], (lecture(A,[X]), lecture(B, [Y]), lecture(C, [X,Y,Z]), lecture(D, [Y,Z]), lecture(E, [Z])), 10). % cas de xrw


regle_groupe([A,B,C,D,E], [X,Y,Z], (lecture(A,[X]), lecture(B, [X,Y]), lecture(C, [Y]), lecture(D, [Z]), lecture(E, [X,Y,Z])), 10). % cas de bAH

regle_groupe([A,B,C,D], [X,Y,Z], (lecture(A,[X,Y]), lecture(B, [Y]), lecture(C, [Z]), lecture(D, [X,Y,Z])), 10). % cas de wAH et de sAH

regle_groupe([AAA,BB], [X,Y,Z], (lecture(AAA,[X,Y,Z]), lecture(BB, [Y,Z])), 60). 

regle_groupe([AA,BBB], [X,Y,Z], (lecture(AA,[X,Y]), lecture(BBB, [X,Y,Z])), 60). 

regle_groupe([A,BB, CCC], [X,Y,Z], (lecture(A,[X]), lecture(BB, [Y,Z]), lecture(CCC,[X,Y,Z])), 60). 
regle_groupe([AA,B, CCC], [X,Y,Z], (lecture(AA,[X,Y]), lecture(B, [Z]), lecture(CCC,[X,Y,Z])), 60). 

regle_groupe([AA,BBB], [X,Y,Z], (lecture(AA,[X,Y]), lecture(BBB, [X,Y,Z])), 60). 

/* deux biliteres presents */
/*==========================*/

regle_groupe(['U19','W24'], ['n','w'], true, 1).

regle_groupe([A,B,C,D], [X,Y,Z], (lecture(A,[X,Y]), lecture(B, [Y]), lecture(C, [Y,Z]), lecture(D, [Z])),10). % cas de Xnw

regle_groupe([A,B], [X,Y,Z], (lecture(A,[X,Y]), lecture(B, [Y,Z])), 10). 
/* cas de inw */


/* deux biliteres et un trilitere */
/*==================================*/

regle_groupe([A,B,C,D,E,F], [W,X,Y,Z], (lecture(A,[W,X]), lecture(B, [X]), lecture(C, [Y]), lecture(D, [Z]), lecture(E, [Y,Z]), lecture(F, [X,Y,Z])),10).
/* plus ou moins le cas de snTr, en rajouttant d'autres regles */

/*--------------------------------------------------------------------------*/
/* UN TRILITERE + UNILITERE(S) */
/*---------------------------------------------------------------------------*/

/* 3 uniliteres sont presents */
/*==============================*/

regle_groupe([A,B,C,D], [X,Y,Z], (lecture(A, [X]), lecture(B, [Y]), lecture(C, [Z]), lecture(D, [X,Y,Z])), 100).

regle_groupe([A,B,C,D], [X,Y,Z], (lecture(A, [X]), lecture(B, [Y]), lecture(C, [X,Y,Z]), lecture(D, [Z])), 100).


regle_groupe([A,B,C,D], [X,Y,Z], (lecture(A, [X]), lecture(B, [X,Y,Z]), lecture(C, [Y]), lecture(D, [Z])), 100).

regle_groupe([A,B,C,W,D], [X,Y,Z,'w'], (lecture(A, [X]), lecture(B, [Y]), lecture(C, [Z]), lecture(D, [X,Y,Z]), lecture(W,['w'])), 100).

/* 2 uniliteres sont presents */
/*==============================*/

regle_groupe([A,B,C], [X,Y,Z], (lecture(A, [X]), lecture(B, [Y]), lecture(C, [X,Y,Z])),100).

regle_groupe([A,B,C], [X,Y,Z], (lecture(A, [X]), lecture(B, [X,Y,Z]), lecture(C, [Z])), 100).


regle_groupe([A,B,C], [X,Y,Z], (lecture(A, [X,Y,Z]), lecture(B, [Y]), lecture(C, [Z])), 100).

/* 1 unilitere present */
/*======================*/

regle_groupe([A,B], [X,Y,Z], (lecture(A, [X]), lecture(B, [X,Y,Z])), 100).
regle_groupe([A,B], [X,Y,Z], (lecture(A, [X,Y,Z]), lecture(B, [Z])), 100).

/* 1 trilitere + 1 bilitere */
/*======================*/
regle_groupe([A,B], [X,Y,Z], (lecture(A, [X,Y,Z]), lecture(B, [X,Y])), 50).
regle_groupe([A,B], [X,Y,Z], (lecture(A, [X,Y]), lecture(B, [X,Y,Z])), 50).



/*aucun unilitere present */
/*========================*/

regle_groupe([A],[X,Y,Z], (lecture(A,[X,Y,Z])),900).



/*--------------------------------------------------------------------------*/
/* UN BILITERE + UNILITERE(S) */
/*--------------------------------------------------------------------------*/
regle_groupe(['U19','W24','Z7'], ['n','w'], true, 50).
regle_groupe(['U19','W24','G43'], ['n','w'], true, 50).
regle_groupe(['N35','U19','W24'], ['n','w'], true, 50).
regle_groupe(['N35','U19','W24','Z7'], ['n','w'], true, 50).
regle_groupe(['N35','U19','W24','G43'], ['n','w'], true, 50).

/* 2 uniliteres sont presents */
/*=============================*/
ou:- true.
regle_groupe([A,B,C], [X,Y], (lecture(A, [X]), lecture(B, [Y]),lecture(C,[X,Y])), 50). 
regle_groupe([A,B,C], [X,Y], (lecture(A, [X]), lecture(B, [X,Y]), lecture(C, [Y])), 50).
regle_groupe([A,B,C], [X,Y], (ou, lecture(A, [X,Y]), lecture(B, [X]), lecture(C, [Y])), 50).

regle_groupe([A,B,C,D], [X,Y,Z], (lecture(A, [X]), lecture(B, [Y]), lecture(C, [Z]), lecture(D, [X,Y])), 100).


/* 1 unilitere est present */
/*==========================*/

regle_groupe(['D4', 'D21'], ['i','r','r'], true,0).
regle_groupe([A,B], [X,Y], (lecture(A, [X,Y]), lecture(B, [Y])), 150).
regle_groupe([A,B], [X,Y], (lecture(A, [X,Y]), lecture(B, [X])), 160).
regle_groupe([A,B], [X,Y], (lecture(A, [X]), lecture(B, [X,Y])), 150).

/*aucun unilitere present */
/*========================*/

regle_groupe([A],[X,Y], lecture(A,[X,Y]), 400).


/*---------------------------------------------------------------------------*/
/* SUITE D'UNILITERES */
/*--------------------------------------------------------------------------*/

%%regle_groupe([A,B,C,D], [W,X,Y,Z], (lecture(A,[W]), lecture(B,[X]), lecture(C, [Y]), lecture(D, [Z])), 600).
%%regle_groupe([A,B,C], [W,X,Y], (lecture(A,[W]), lecture(B,[X]), lecture(C, [Y])),650).
%%regle_groupe([A,B], [W,X], (lecture(A,[W]), lecture(B,[X])), 900).

/* Cette rgle est nocive. La revoir avec A = vrai unilitaire */
regle_groupe([A], [W], (lecture(A,[W])), 380).

regle_groupe([debut,A,B], [m], (lecture(A,[m]), lecture(B,[a])), 140).

regle_groupe([debut,A,B,fin], ['m','a'], (lecture(A,[m]), lecture(B,[a])), 100).

/*---------------------------------------------------------------------------*/
/* Lectures spciales unilitaires */
/*--------------------------------------------------------------------------*/

/* le suffixe */
regle_groupe([debut, 'A1', fin], ['i'], true, 1).

/*---------------------------------------------------------------------------*/
/* REGLES SUR LES IDEOGRAMMES                                                */
/*--------------------------------------------------------------------------*/

regle_groupe([debut,'D21','Z1'], [r,'A'], true, 1).
regle_groupe(['D21','Z1'], [r,'A'], true, 300).
regle_groupe(['D21'], [r,'A'], true, 900).

/*---------------------------------------------------------------------------*/
/* Lectures spciales */
/*--------------------------------------------------------------------------*/

regle_groupe([debut,'D4','Q1'], ['w','s','i','r'],true,0).
regle_groupe([debut,'O1',fin], ['p'],true, 40).
regle_groupe([debut|R], [N], (est_liste_traits(R,N)),0).

regle_groupe(['N11','N14'], ['A','b', 'd'],true, 10).
regle_groupe(['N11','N14','D46'], ['A','b', 'd'],true, 10).

regle_groupe([debut,A, fin], VAL, (lecture(A,VAL)), 1).
regle_groupe([debut,A,Z,fin], VAL, (lecture(A,VAL), gramnombre(Z)), 1).
regle_groupe([debut,A,'X1',Z,fin], VAL, (lecture(A,VAL),gramnombre(Z),fin_en_t(VAL)), 1).
regle_groupe([debut,A,'X1',fin], VAL, (lecture(A,VAL),fin_en_t(VAL)), 5).
regle_groupe([debut,A,'X1',Z,fin], VAL1,
	(lecture(A,VAL),conc(VAL,[t],VAL1),gramnombre(Z)), 2).

regle_groupe([A,Z,fin], [], (lecture(A,*),gramnombre(Z)), 500).
regle_groupe([A,fin], [], (lecture(A,*)), 500).
regle_groupe([A,B,fin], [], (lecture(A,*),lecture(B,*)), 1000).
regle_groupe([A,B,Z,fin], [], (lecture(A,*),lecture(B,*),gramnombre(Z)), 1000).
regle_groupe([A,B,C,Z,fin], [], (lecture(A,*),lecture(B,*),lecture(C,*),gramnombre(Z)), 1500).
regle_groupe([A,B,C,fin], [], (lecture(A,*),lecture(B,*),lecture(C,*)), 1500).
regle_groupe([A], [], (lecture(A,*)), 1000).

fin_en_t([t]).
fin_en_t([_|R]):- fin_en_t(R).

gramnombre(X):- member(X,['Z1','Z2','Z3','Z3A', '3', '1']).

est_liste_traits([fin], 1).
est_liste_traits(['Z1'|R], N):- est_liste_traits(R,N1), N is N1 + 1.

%% Meta rgles :

regle_groupe(L, [A,B,C], true, 1):- 
  member(N,[3,4,5,6]),
  length(L,N),
  member(XXX, L),  lecture(XXX, [A,B,C]),
  member(XX, L),   lecture(XX, [LX1, LX2]),
  sousliste([LX1,LX2], [A,B,C]),
  enleve(XX,L,L1),
  enleve(XXX, L1, L2),
  tous_unilitaires_dans(L2, [A,B,C]).

tous_unilitaires_dans([],_).
tous_unilitaires_dans([X|R],VALS):-
	lecture(X,[A]),
	member(A, VALS),
	tous_unilitaires_dans(R,VALS).

est_unilitaire(X):- 
	lecture(X, [_]).

  

