/* lecture d'un mot  */
:- consult('torep.pl').
:- consult('lectures_pos.pl').
:- consult('regles_groupe.pl').
:- consult('listes.pl').
%% :- consult('determinatifs.pl').
%% :- consult('transposition_graphique.pl').
:- consult('mdc2liste').

tout(Txt , Res):-
	les_mots(Txt, T1),
	tout_aux(T1, Res).

tout_aux([],[]). 
tout_aux([A|B],[A1|B1]):- write(A), write(' => '), run(A,A1), !, write(A1), nl, tout_aux(B,B1).
tout_aux([A|B],[A|B1]):- write(A), nl, tout_aux(B,B1).
 
run(X,RES):- run_aux(X,RES,C), write('Cout :'), write(C), nl.

run_aux(X,RES,Cout):- mdc2liste(X, X0), decode_liste_hieros(X0,L), cherche_mot(L,RES, Cout), RES \= ''.

get_vals(X,RES):- setof((R/C), run_aux(X,R,C), L), 
	predsort(plus_petit_numero, L,RES).

/*PROCEDURE DE RECHERCHE DE TOUTES LES TRANSLITTERATIONS POSSIBLES */

/* premier cas : le mot contient un determinatif significatif :
   On teste si les lectures sont compatibles, et, si oui, on lit le mot
   de cette manire*/

/* second cas */
cherche_mot(L0,Ma, Cout):- % L0  mot en hieros, Ma mot en translitteration 
        
	ajouter_debut_fin(L0,L),
        liste_numerotee(L,Ln),
	regles_utilisables(Ln,REGLES),
	            /*REGLES = liste des regles utilisables sur le mot L*/
        tri_regles(REGLES,Rs),
                     /* Rs = liste des regles utilisables, rangees par
                                            ordre de pertinence */
	appliquer_les_regles(Ln,Rs,[], M, 0, Cout), 
           /*M est deductible de L a partir D'UNE PARTIE de Rs */
  
        trie_translitteration(M,Ms),
	 /* remet les lettres de la translitteration dans l'ordre */
        arrange_translitteration(Ms,Ma).


%/* traite les eventuelles transpositions graphiques */

%cherche_mot(H,T):- 
%	conc(H1, H2, H), conc(L, Ha, H2),
%	transposition(L, Lt),  
%	conc(Lt, Ha, H2t), conc(H1, H2t, Ht),
%	cherche_mot(Ht, T).

/* CREATION DE L ENSEMBLE DES REGLES UTILISABLES */

regles_utilisables(L,Rs):-
	bagof(R,regle_utilisable(L,R),Rs).

regle_utilisable(L,[S,N,C,P]):- 
   /* la regle [S,N,C,P] est utilisable sur le mot L en hieros */
   sousliste(S,L), S \= [],
   supprime_numeros(S,S0),	
   regle_groupe(S0,N,C,P), C.

/* PROCEDURE DE TRI DES REGLES PAR ORDRE DE PERTINENCE */
tri_regles(Rs,Rs1):-
	predsort(meilleure_regle,Rs,Rs1).

/* criteres de pertinence */

/* critere 1 : la regle  correspond a un groupe de hieros precis
 -> la regle correspondante est prioritaire */ 

/* meilleure_regle([_,_,_,0], [_,_,_,N]):- N\= 0, !. */

/* critere 2 : Le nombre de hieros intervenant dans la 1ere regle est plus grand que celui des hieros intervenant dans la 2eme regle -> la 1ere regle est prioritaire */


meilleure_regle([_,_,_,V1], [_,_,_,V2]):-
	V1 < V2.

meilleure_regle([L1,_,_,V1], [L2,_,_,V1]):-
	length(L1,N1), length(L2,N2),
	N1 > N2.




/* APPLICATION DES REGLES */
 
/* appliquer_les_regles
   L : liste numrote
   S : sous liste numrote
   
  L est une liste numerotee, mais pas S, ni N */

appliquer_une_regle(
	L, [S,N,_,C], Val, Cout_deb, 
	Reste, ValF, CoutF):-  
	
	sousliste(L, Avant, S, Apres),
	conc(Avant, Apres, Reste),
	S= [_/P|_],
	Nn = [N/P], 
	conc(Nn,Val,ValF), 
	CoutF is Cout_deb + C.

appliquer_les_regles(L,[Regle | Rs],VALI, VALF, Cout_deb, Cout):-
	appliquer_une_regle(L, Regle, VALI, Cout_deb, Reste, Result, Cout_Nouv),
	appliquer_les_regles(Reste,Rs, Result, VALF, Cout_Nouv, Cout).

appliquer_les_regles(L,[_| Rs],VALI, VALF, Cout_deb, Cout):-
	appliquer_les_regles(L,Rs,VALI, VALF, Cout_deb, Cout).
 
appliquer_les_regles([],[],VAL, VAL, Cout, Cout).

/* PROCEDURE DE TRI DES LETTRES DE LA TRANSLITTERATION */   
trie_translitteration(M,Ms):-
      predsort(plus_petit_numero,M,Ms). 

/* critere de tri : les numeros associes a chaque groupe */ 
plus_petit_numero(_/X1,_/X2):-
	X1 < X2.

/* Ajout de marques de debut et de fin au mot : */

ajouter_debut_fin(L,[debut|L1]):- ajouter_debut_fin_aux(L,L1).
ajouter_debut_fin_aux([],[fin]).
ajouter_debut_fin_aux([A|R], [A|R1]):- ajouter_debut_fin_aux(R,R1).


/* LISIBILITE DU RESULTAT */
arrange_translitteration(M,Ma) :-
	supprime_numeros(M,M1),
	linearisation(M1,M2),
	mot(M2,Ma),
	mot_non_vide(Ma).

/* suppression des numeros */
 
supprime_numeros([],[]).
supprime_numeros([X/_ | M], [X | Ma]) :- supprime_numeros(M,Ma).
  
/* linearisation */

linearisation([],[]).
linearisation([A|R], R2):- 
	linearisation(R,R1),
	append(A,R1,R2).


/* mise sous forme de mot */

mot([],''):- !.
mot([X | L], M) :-
	name(X,[Ax]),
	mot(L, M1),
	name(M1, Al),
	name(M,[Ax | Al]).

/*suppression des mots vides */
mot_non_vide(M) :- M\= ' ', M \= ''.





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Trouve les solutions dans l'ordre de plausibilit
%
%
% Mthode 2 :
%
% Une hypothse comprend :
%
% le texte  traiter
% la solution partielle
% ses extensions possibles
% 
%
% Dvelopper une hypothse : 
%
%  Si c'est une solution, la ranger dans la liste idoine
%     recommencer avec l'hypothse suivante si ladite liste n'est pas remplie
%
%  Sinon, prendre la liste des extensions de l'hypothse,
%  Les insrer dans la liste des hypothses, avec leurs extensions possibles
%
% On a besoin de savoir :
%
% Le cout de l'hypothese.
% Le cout de sa meilleure extension
% l'indice max. des numeros de regles utilisables
% 

%% TOO EXPENSIVE !
%calculer_extensions([]/V/_/C,
%	_,
%	[]/V/[]/C).

%calculer_extensions(
%	TXT/V/MINR/C, 
%	REGLES,
%	TXT/V/EXTENSIONS/COUT_EXT):-
%	setof(TXT1/V1/Num/C1, R^Num^(
%	member(R/Num, REGLES), 
%	Num > MINR,
%	appliquer_une_regle(TXT, R, V, C, TXT1, V1, C1)
%%	write('extension proposee'), write(TXT1/V1/C1), nl
%    ),
%	EXTENSIONS),
%	predsort(plus_petit_numero,EXTENSIONS,[_/COUT_EXT|_]).

%inserer_une_extension(E,[], [E]).
%inserer_une_extension(X/C1,[Y/C2|R], [X/C1,Y/C2|R]):- C1 =< C2.
%inserer_une_extension(X/C1,[Y/C2|R], [Y/C2|R1]):- 
%	C1 > C2,
%	inserer_une_extension(X/C1,R,R1).


%%% inserer_extensions(EXTENSIONS, H, H1)
%inserer_extensions([],_,H,H).

%inserer_extensions([E|R],REGLES, H,HF):-
%	(
%	    calculer_extensions(E, REGLES, EXT) ->
%	    inserer_une_extension(EXT,H,H1);
%	    H1= H),
%	inserer_extensions(R,REGLES, H1,HF).



%etendre(TXT, SOLUTIONS, NBR):- 
%	regles_utilisables(TXT, REGLES),
%        tri_regles(REGLES,Rs),
%	liste_numerotee(Rs, REGLES1),
%	etendre_aux([TXT/[]/[TXT/[]/0/0]/0], REGLES1, [], SOLUTIONS, NBR).


%etendre_aux([], _, SOL, SOL, _).

%etendre_aux([[]/V/_/C|R], REGLES, SOL0, SOLF, NBR):-
%	trie_translitteration(V,TR0),
%	arrange_translitteration(TR0,TR),
%	SOL1= [TR/C|SOL0],
%	write('solution :'), write(SOL1), nl,
%	length(SOL1, N),
%	(N = NBR -> 
%	    SOLF= SOL1;
%	    etendre_aux(R,REGLES,SOL1,SOLF,NBR)).
	    
%etendre_aux([TXT/VAL1/EXTENSIONS/C|R], REGLES, SOL0, SOLF, NBR):-
%	length(R,LN),
%	write(LN), nl,
%	write(TXT/VAL1/C), nl,
%	inserer_extensions(EXTENSIONS, REGLES, R, R1),
%	etendre_aux(R1, REGLES, SOL0, SOLF, NBR).
	

essai(X,RES):-
	mdc2liste(X, X0), 
	decode_liste_hieros(X0,L),
	ajouter_debut_fin(L,L0),
        liste_numerotee(L0,Ln),
	etendre(Ln, RES,5).


% Hypotheses : une liste reprsentant les hypothses dveloppes
% avec leur cout.
% 
% Algo : 
% a) chercher le meilleur couple Hypothese/REGLE
%    qui n'aie pas dj t test.
%    Pour avoir un ordre unique, les regles sont numrotes,
%    chaque hypothse est accompagne du numro de la dernire rgle qui lui
%    a t applique, et on ne peut lui appliquer que des rgles de numro >  celui-ci
% b) appliquer 
% d) recommencer

% a) => pour toutes les hypothses et toutes les rgles :
%       * essayer d'etendre
%       * prendre le meilleur rsultat. qui ne soit pas dj trouv.
% RMQ : Si les hypothses sont tries, et qu'on arrive 

% Hypotheses : liste des hypotheses, trie par ordre de pertinence.

% Une hypothese : TXT/VAL/MINRULE/COUT

etendre(TXT, SOLUTIONS, NBR):- 
	regles_utilisables(TXT, REGLES),
        tri_regles(REGLES,Rs),
	liste_numerotee(Rs, REGLES1),
	etendre_aux([TXT/[]/0/0], [], SOLUTIONS, REGLES1, NBR).


etendre_aux(HYP, SOL_CONNUES, SOLS, REGLES, NBR):- 
	meilleure_extension(HYP, EXT, REGLES), !,
	
	write(EXT), nl,
	traite_si_sol(EXT, SOL_CONNUES, NEWSOL), 
	(
	    (length(NEWSOL, N), N = NBR ) ->
	    SOLS = NEWSOL;
	    etendre_aux([EXT|HYP], NEWSOL, SOLS, REGLES, NBR)
	).

etendre_aux(_, SOLS, SOLS, _, _).
	

extension(Hs, REGLES, HYP, TXT1/VAL1/COUT1):-
	member(TXT/VAL/COUT, Hs),
	member(R, REGLES),
	appliquer_une_regle(TXT, R, VAL, COUT,
	TXT1, VAL1, COUT1),
	not member(TXT1/VAL1/COUT1, HYP), !.

meilleure_extension(HYP, EXT, REGLES):-
	EXT0=_/_/0/1000000000,
	meilleure_extension_aux(HYP, REGLES, HYP, EXT0, EXT).

meilleure_extension_aux([], _, _, EXT, EXT).

meilleure_extension_aux([_/_/_/C|_], _, _, EXT, EXT):-
	EXT = _/C0,
	C > C0.

meilleure_extension_aux([H|Hs], REGLES, HYP, EXT0, EXT):- 
	EXT0 = _/C0,
	( etendre_hypothese(H, REGLES, HYP, EXT1) ->
	    ( EXT1 = _/C1,
	      ( C0 > C1 -> EXT2 = EXT1 ; EXT2 = EXT0)) ;
         EXT2= EXT0),
    meilleure_extension_aux(Hs, REGLES, HYP, EXT2, EXT).
    

etendre_hypothese(TXT/V/MINR/C, REGLES, HYP, EXT):-
	
	setof(TXT1/V1/Num/C1, R^Num^(
	member(R/Num, REGLES), 
	Num > MINR,
	appliquer_une_regle(TXT, R, V, C, TXT1, V1, C1)),
	L),

	enleve_elements(HYP,L, L1),
	predsort(plus_petit_numero,L1,[EXT|_]).


traite_si_sol([]/V/_/C, S0, [TR/C|S0]):- 
	trie_translitteration(V,TR0),
	arrange_translitteration(TR0,TR).

traite_si_sol([_|_]/_/_/_, S0, S0).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
brute(_,[], S, S).

brute(HYP, [R|Rs], SOLA, SOLS):-
	length(HYP, LH), write(LH) , write(' Hypothses '), 
	length(SOLA, LS), write(LS), write(' Solutions '), nl,
	brute_to_hyp(HYP, R, HYP1, SOLI),
	conc(SOLA, SOLI, SOLB),
	conc(HYP1, HYP, HYPI),
	brute(HYPI, Rs, SOLB, SOLS).

brute_to_hyp([], _, [], []).

brute_to_hyp([TXT/V|Hs], R, HR, SOL1):-
	(
	    appliquer_une_regle(TXT, R, V, 0, TXT1, V1, _) ->
	    ( 
		TXT1 = [] -> 
		(   
		    trie_translitteration(V1,V2),
		    arrange_translitteration(V2,V3),
		    SOL1= [V3|SOL2], HR= HR1);
		    (HR= [TXT1/V1|HR1], SOL1= SOL2));
	    HR= HR1, SOL1 = SOL2
	),
    brute_to_hyp(Hs, R, HR1, SOL2).

essai_brute(X, RES):- 
	mdc2liste(X, X0), 
	decode_liste_hieros(X0,L),
	ajouter_debut_fin(L,L0),
        liste_numerotee(L0,Ln),
	regles_utilisables(Ln, REGLES),
	brute([Ln/[]], REGLES, [], RES).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Extension sauvant les extensions possibles comme