/* lecture d'un mot  */
:- consult('torep.pl').
:- consult('lectures_pos.pl').
:- consult('regles_groupe.pl').
:- consult('listes.pl').
%% :- consult('determinatifs.pl').
%% :- consult('transposition_graphique.pl').
:- consult('mdc2liste').


/* CREATION DE L ENSEMBLE DES REGLES UTILISABLES */

regles_utilisables(L,Rs):-
	bagof(R,regle_utilisable(L,R),Rs).

/* la regle [S,N,C,P] est utilisable sur le mot L en hieros 
P est normalement le cout estime de la regle, selon la formule
P= cout_regle*(somme proba de signes)/(nbr de signes)
*/

regle_utilisable(L,[S,N,C,P1]):- 
	sousliste(S,L), S \= [],
	supprime_numeros(S,S0),	
	regle_groupe(S0,N,C,P), 
	verifier_conditions(C,P, P1, 0, 0).



verifier_conditions((C,Cs), P, P1, TOT, NBR):-
	verifier_condition(C,TOT, NBR,TOT1,NBR1),
	verifier_conditions(Cs, P, P1, TOT1, NBR1).

verifier_conditions(C, P, P1, TOT, NBR):-
	C \= (_,_),
	verifier_condition(C,TOT,NBR,TOT1, NBR1),
	(
	    NBR1 = 0 ->
	    P1 = P ;
	    P1 is P / (TOT1 / NBR1)
	).

se_lit(A,X,1):-
	lectures_possibles(A, L), 
	( (membre(X, L), C = 1); ( membre(X/C,L) )).

verifier_condition(lecture(A,X), TOT, NBR, TOT1, NBR1):- 
	se_lit(A, X,Conf),
	NBR1 is NBR + 1,
	TOT1 is TOT + Conf.

verifier_condition(C, TOT, NBR, TOT, NBR):- 
	C \= lecture(_,_),
	C.


/* PROCEDURE DE TRI DES REGLES PAR ORDRE DE PERTINENCE */
tri_regles(Rs,Rs1):-
	predsort(best_regle,Rs,Rs1).

/* criteres de pertinence */

/* critere 1 : la regle  correspond a un groupe de hieros precis
 -> la regle correspondante est prioritaire */ 

/* meilleure_regle([_,_,_,0], [_,_,_,N]):- N\= 0, !. */

/* critere 2 : Le nombre de hieros intervenant dans la 1ere regle est plus grand que celui des hieros intervenant dans la 2eme regle -> la 1ere regle est prioritaire */


best_regle(=,Y,Y):- !.

best_regle(X,Y,Z):-
	meilleure_regle(Y,Z) -> X = '<' ; X = '>'.

  
meilleure_regle([_,_,_,V1], [_,_,_,V2]):-
	V1 < V2.

meilleure_regle([L1,_,_,V1], [L2,_,_,V1]):-
	length(L1,N1), length(L2,N2),
	N1 > N2.


/* APPLICATION DES REGLES */
 
/* appliquer_les_regles
   L : liste numrote
   S : sous liste numrote
   
  L est une liste numerotee, mais pas S, ni N */

appliquer_une_regle(
	L, [S,N,_,C], Val, Cout_deb, 
	Reste, ValF, CoutF):-  
	
	sousliste(L, Avant, S, Apres),
	conc(Avant, Apres, Reste),
	S= [_/P|_],
	Nn = [N/P], 
	conc(Nn,Val,ValF), 
	CoutF is Cout_deb + C.

appliquer_les_regles(L,[Regle | Rs],VALI, VALF, Cout_deb, Cout):-
	appliquer_une_regle(L, Regle, VALI, Cout_deb, Reste, Result, Cout_Nouv),
	appliquer_les_regles(Reste,Rs, Result, VALF, Cout_Nouv, Cout).

/*
appliquer_les_regles(L,[_| Rs],VALI, VALF, Cout_deb, Cout):-
	appliquer_les_regles(L,Rs,VALI, VALF, Cout_deb, Cout).
*/

appliquer_les_regles([],[],VAL, VAL, Cout, Cout).

/* PROCEDURE DE TRI DES LETTRES DE LA TRANSLITTERATION */   
trie_translitteration(M,Ms):-
      predsort(plus_petit_numero,M,Ms). 

/* critere de tri : les numeros associes a chaque groupe */ 
plus_petit_numero(<, _/X1,_/X2):-
	X1 < X2.

plus_petit_numero(>, _/X1,_/X2):-
	X1 > X2.

plus_petit_numero(=, _/X1,_/X2):-
	X1 = X2.

/* Ajout de marques de debut et de fin au mot : */

ajouter_debut_fin(L,[debut|L1]):- ajouter_debut_fin_aux(L,L1).
ajouter_debut_fin_aux([],[fin]).
ajouter_debut_fin_aux([A|R], [A|R1]):- ajouter_debut_fin_aux(R,R1).


/* LISIBILITE DU RESULTAT */
arrange_translitteration(M,Ma) :-
	supprime_numeros(M,M1),
	linearisation(M1,M2),
	mot(M2,Ma),
	mot_non_vide(Ma).

/* suppression des numeros */
 
supprime_numeros([],[]).
supprime_numeros([X/_ | M], [X | Ma]) :- supprime_numeros(M,Ma).
  
/* linearisation */

linearisation([],[]).
linearisation([A|R], R2):- 
	linearisation(R,R1),
	append(A,R1,R2).


/* mise sous forme de mot */

mot([],''):- !.
mot([X | L], M) :-
	name(X,[Ax]),
	mot(L, M1),
	name(M1, Al),
	name(M,[Ax | Al]).

/*suppression des mots vides */
mot_non_vide(M) :- M\= ' ', M \= ''.



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Decoupage du mot en sous-ensembles 
%
%
% Au depart, un sous ensemble par signe.
% A chaque nouvelle regle :
%  chercher les sous ensembles concernes par la regles.
%  les unifier
%
%  [[1],[2],...[10]]
%  regle concernant [3..4] =>
%  [[1],[2],[3,4]..[10]]
%  regle concernant [1..3] =>
%  [[1,4],[5]..[10]]
% etc.
% 
% [1] signifie [1,1]
% grouper deux ensembles : [A,B] U [C,D] =[A,D]
% grouper _ selon _ regle :
%   Si la tete du premier groupe est > max des signes concernes
%   par la regle , fin
%
%   Sinon, grouper le premier groupe et celui des signes de la regle
%   avancer au groupe suivant.

%% ilots --
%% TXT + : liste de signes numrote (code/numro)
%% REGLES + : 
%% SUBTXT -
%% SUBTXT est une partition de TXT en ilts indpendants selon les
%% rgles de REGLES 

ilots(TXT, REGLES, SUBTXT):- 
	prepare_txt_ilots(TXT, TXT0),
	ilots_aux(TXT0, REGLES, SUBTXT).

prepare_txt_ilots([], []).
prepare_txt_ilots([T|R], [[T]|R1]):- 
	prepare_txt_ilots(R,R1).

limites_texte([A/DEB|R], DEB, FIN):-
	limite_fin_texte([A/DEB|R], FIN).

limite_fin_texte([_/FIN], FIN).
limite_fin_texte([_|TXT], FIN):-
	limite_fin_texte(TXT, FIN).


ilots_aux(TXT, [], TXT).
ilots_aux(TXT, [R|Rs], RESULT):- 
	%% limites de la regle
	R= [TXTR, _, _, _],
	limites_texte(TXTR,DEB, FIN),
	%% 
	grouper_ilot(TXT, DEB, FIN, TXT1),
	ilots_aux(TXT1, Rs, RESULT).

grouper_ilot([], _, _ , []).


grouper_ilot([TETE|R], DEB, FIN, [TETE1|R1]):-
	limites_texte(TETE, A,B),
	(
	    DEB > B ->				%% Avant la rgle	
	    (
		TETE= TETE1, grouper_ilot(R, DEB, FIN, R1)
	    );
	    (
		FIN < A ->			% Aprs la rgle :
		(
		    TETE1= TETE, R1= R	
		);
		(
		    (
			FIN =< B, DEB >= A
		    ) ->			% inclusion 
		    (
			TETE= TETE1, R1= R
		    );
		    %% A <= DEB  <= B < FIN :
		    %% Il y a forcment un lment aprs :
		    (
			R=[NEXT|R2],
			conc(TETE,NEXT, TETE2),
			grouper_ilot([TETE2|R2], DEB, FIN, [TETE1|R1])
		    )
		)
	    )
	).



%% C'est tout. Les autres hypothses 
%% sont impossibles par construction.
