#!/bin/sh
# the next line restarts using wish\
exec tclsh8.0 "$0" "$@"

# Code for updating dictionnary text from old version to new version.
# 

# Old version had an implicit rule for text based on tags being lists.
# New version has a new tag, T, for plain text. (T is shorter than PCDATA !)
# Hence the new syntax is :
#
# simple tags are those accepting AT MOST pcdata as value.
# composite tags are the other one.
#
# simple_tags::="T"|"HIE"|"MHIE"|"WRI"|"DET"|"TRI"|"COP"|"MTRI"|
#             "REF"|"AREF"|MOT|QQUN..
# text::= tag
# tag::= composite_tag value1
#        | simple_tags PCDATA
#
# value1::= "{" tags* "}"
#
# The nice thing is that 
# a) all data is now in the form TAG data
# b) the list system works normally now.
#

array set tagkind {
    GRP OTHER
    WRI DATA
    MAX OTHER
    DET DATA
    HIE DATA
    COM OTHER
    TRAD OTHER
    TRI PCDATA
    COP PCDATA
    MTRI PCDATA
    REF PCDATA
    AREF PCDATA
    USE OTHER
    EXE OTHER
    MOT DATA
    QQUN DATA
    QQCHOSE DATA
    GRA DATA
    LATTER DATA
    OK DATA
    NK DATA
    LE DATA
    VAR DATA
    AUT PCDATA
    EMP OTHER
    NOTE OTHER
    TITLE OTHER
    SUBTITLE OTHER
    SECTION OTHER
    PAR OTHER
}

proc maketag {TAG val} {
    if { $val == "" } {
	return [list $TAG {}]
    } else {
	if {[llength $val] > 1} {
	    return [list $TAG $val]
	} else {
	    return "$TAG {$val}"
	}
    }
}

proc update {list} {
    global tagkind
    if {[llength $list] != 1} {
	set tag [lindex $list 0]
	set value [lindex $list 1]
	switch -- $tagkind($tag) {
	    DATA {return $list}
	    PCDATA {return $list}
	    OTHER {
		set res {}
		set lastt {}
		set pending 0
		foreach l $value {
		    set newl [update $l]
		    if {[string compare [lindex $newl 0] T] == 0} {
			set pending 1
			lappend lastt [lindex $newl 1]
		    } else {
			if {$pending} {
			    lappend res [list T $lastt]
			    set lastt {}
			    set pending 0
			}
			lappend res $newl
		    }
		}
		if {$pending} {
		    lappend res [list T $lastt]
		}
		return [maketag $tag $res]
	    }
	}
    } else {
	#this is data. 
	return [maketag T $list]
    }
}

proc old2new {list} {
    foreach {k v} $list {
	lappend res $k [update $v]
    }
    return $res
}

#set f [open stdin]
puts [old2new [read stdin]]
#close $f
